/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: ControllerAdapter.java,v 1.41 2004/02/06 06:04:25 christianfoltin Exp $*/

package freemind.modes;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;




import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.JTextComponent;

import freemind.main.FreeMind;
import freemind.controller.Controller;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.ExampleFileFilter;
import freemind.main.XMLParseException;
import freemind.view.MapModule;
import freemind.view.mindmapview.MapView;
import freemind.view.mindmapview.NodeView;


/**
 * Derive from this class to implement the Controller for your mode. Overload the methods
 * you need for your data model, or use the defaults. There are some default Actions you may want
 * to use for easy editing of your model. Take MindMapController as a sample.
 */
public abstract class ControllerAdapter implements ModeController {

    Mode mode;
    private int noOfMaps = 0; //The number of currently open maps
    private Clipboard clipboard;
    private int status;

    public Action copy = null;
    public Action copySingle = null;
    public Action cut = null;
    public Action paste = null;

    static final Color selectionColor = new Color(200,220,200);

    public ControllerAdapter() {/*_I*/_prof.prof.cnt[5196]++;/*I_*/
    }

    public ControllerAdapter(Mode mode) {/*_I*/_prof.prof.cnt[5197]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5198]++;/*I_*/this.mode = mode;/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5199]++;/*I_*/cut = new CutAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5200]++;/*I_*/paste = new PasteAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5201]++;/*I_*/copy = new CopyAction(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5202]++;/*I_*/copySingle = new CopySingleAction(this);/*_I*/}/*I_*/

        /*_I*/_prof.prof.cnt[5203]++;/*I_*/DropTarget dropTarget = new DropTarget(getFrame().getViewport(),
                                               new FileOpener());

        /*_I*/{_prof.prof.cnt[5204]++;/*I_*/clipboard = getFrame().getViewport().getToolkit().getSystemSelection();/*_I*/}/*I_*/

        // SystemSelection is a strange clipboard used for instance on
        // Linux. To get data into this clipboard user just selects the area
        // without pressing Ctrl+C like on Windows.
        
        /*_I*/{_prof.prof.cnt[5205]++;/*I_*/if (clipboard == null) {
           /*_I*/{_prof.prof.cnt[5206]++;/*I_*/clipboard = getFrame().getViewport().getToolkit().getSystemClipboard();/*_I*/}/*I_*/ }/*_I*/}/*I_*/

    }

    //
    // Methods that should be overloaded
    //

    protected abstract MindMapNode newNode();

    /**
     * You _must_ implement this if you use one of the following actions:
     * OpenAction, NewMapAction.
     */
    public MapAdapter newModel() {/*_I*/_prof.prof.cnt[5207]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5208]++;/*I_*/throw new java.lang.UnsupportedOperationException();/*_I*/}/*I_*/
    }

    /**
     * You may want to implement this...
     * It returns the FileFilter that is used by the open() and save()
     * JFileChoosers.
     */
    protected FileFilter getFileFilter() {/*_I*/_prof.prof.cnt[5209]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5210]++;/*I_*/return null;/*_I*/}/*I_*/
    }

    public void nodeChanged(MindMapNode n) {/*_I*/_prof.prof.cnt[5211]++;/*I_*/
    }

    public void anotherNodeSelected(MindMapNode n) {/*_I*/_prof.prof.cnt[5212]++;/*I_*/
    }

    public void doubleClick(MouseEvent e) {/*_I*/_prof.prof.cnt[5213]++;/*I_*/
        /* perform action only if one selected node.*/
        /*_I*/{_prof.prof.cnt[5214]++;/*I_*/if(getSelecteds().size() != 1)
            /*_I*/{_prof.prof.cnt[5215]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5216]++;/*I_*/MindMapNode node = ((NodeView)(e.getComponent())).getModel();
        // edit the node only if the node is a leaf (fc 0.7.1)
        /*_I*/{_prof.prof.cnt[5217]++;/*I_*/if (node.hasChildren()) {
            // the emulate the plain click. 
            /*_I*/{_prof.prof.cnt[5218]++;/*I_*/plainClick(e);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5219]++;/*I_*/return;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5220]++;/*I_*/if (!e.isAltDown() 
            && !e.isControlDown() 
            && !e.isShiftDown() 
            && !e.isPopupTrigger()
            && e.getButton() == MouseEvent.BUTTON1
            && (node.getLink() == null)) {
            /*_I*/{_prof.prof.cnt[5221]++;/*I_*/edit(null, false, false);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public void plainClick(MouseEvent e) {/*_I*/_prof.prof.cnt[5222]++;/*I_*/
        /* perform action only if one selected node.*/
        /*_I*/{_prof.prof.cnt[5223]++;/*I_*/if(getSelecteds().size() != 1)
            /*_I*/{_prof.prof.cnt[5224]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5225]++;/*I_*/MindMapNode node = ((NodeView)(e.getComponent())).getModel();
        /*_I*/{_prof.prof.cnt[5226]++;/*I_*/if (getView().getSelected().followLink(e.getX())) {
            /*_I*/{_prof.prof.cnt[5227]++;/*I_*/loadURL();/*_I*/}/*I_*/ }
        else {
            /*_I*/{_prof.prof.cnt[5228]++;/*I_*/if (!node.hasChildren()) {
                // the emulate the plain click. 
                /*_I*/{_prof.prof.cnt[5229]++;/*I_*/doubleClick(e);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5230]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5231]++;/*I_*/toggleFolded();/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
    }

    //
    // Map Management
    //

    /**
     * Get text identification of the map
     */

    protected String getText(String textId) {/*_I*/_prof.prof.cnt[5232]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5233]++;/*I_*/return getController().getResourceString(textId);/*_I*/}/*I_*/ }

    protected boolean binOptionIsTrue(String option) {/*_I*/_prof.prof.cnt[5234]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5235]++;/*I_*/return getFrame().getProperty(option).equals("true");/*_I*/}/*I_*/}

    public void newMap() {/*_I*/_prof.prof.cnt[5236]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5237]++;/*I_*/getController().getMapModuleManager().newMapModule(newModel());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5238]++;/*I_*/mapOpened(true);/*_I*/}/*I_*/
    }

    protected void newMap(MindMap map) {/*_I*/_prof.prof.cnt[5239]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5240]++;/*I_*/getController().getMapModuleManager().newMapModule(map);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5241]++;/*I_*/mapOpened(true);/*_I*/}/*I_*/
    }

    /**
     * You may decide to overload this or take the default
     * and implement the functionality in your MapModel (implements MindMap)
     */
    public void load (File file) throws FileNotFoundException, IOException, XMLParseException {/*_I*/_prof.prof.cnt[5242]++;/*I_*/
        /*_I*/_prof.prof.cnt[5243]++;/*I_*/MapAdapter model = newModel();
        /*_I*/{_prof.prof.cnt[5244]++;/*I_*/model.load(file);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5245]++;/*I_*/getController().getMapModuleManager().newMapModule(model);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5246]++;/*I_*/mapOpened(true);/*_I*/}/*I_*/
    }

    public boolean save() {/*_I*/_prof.prof.cnt[5247]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5248]++;/*I_*/if (getModel().isSaved()) /*_I*/{_prof.prof.cnt[5249]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5250]++;/*I_*/if (getModel().getFile() == null || getModel().isReadOnly()) {
           /*_I*/{_prof.prof.cnt[5251]++;/*I_*/return saveAs();/*_I*/}/*I_*/ }
        else {
           /*_I*/{_prof.prof.cnt[5252]++;/*I_*/return save(getModel().getFile());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    /** fc, 24.1.2004: having two methods getSelecteds with different return values (linkedlists of models resp. views) is asking for trouble. @see MapView */
    protected LinkedList getSelecteds() {/*_I*/_prof.prof.cnt[5253]++;/*I_*/
	/*_I*/_prof.prof.cnt[5254]++;/*I_*/LinkedList selecteds = new LinkedList();
	/*_I*/_prof.prof.cnt[5255]++;/*I_*/ListIterator it = getView().getSelecteds().listIterator();
	/*_I*/{_prof.prof.cnt[5256]++;/*I_*/if (it != null) {
	    /*_I*/{_prof.prof.cnt[5257]++;/*I_*/while(it.hasNext()) {
		/*_I*/_prof.prof.cnt[5258]++;/*I_*/NodeView selected = (NodeView)it.next();
		/*_I*/{_prof.prof.cnt[5259]++;/*I_*/selecteds.add( selected.getModel() );/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[5260]++;/*I_*/return selecteds;/*_I*/}/*I_*/
    }

    

    /**
     * Return false is the action was cancelled, e.g. when
     * it has to lead to saving as.
     */
    public boolean save(File file) {/*_I*/_prof.prof.cnt[5261]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5262]++;/*I_*/return getModel().save(file);/*_I*/}/*I_*/ }      

    /** @return returns the new JMenuItem.*/
    protected JMenuItem add(JMenu menu, Action action, String keystroke) {/*_I*/_prof.prof.cnt[5263]++;/*I_*/ 
       /*_I*/_prof.prof.cnt[5264]++;/*I_*/JMenuItem item = menu.add(action);
       /*_I*/{_prof.prof.cnt[5265]++;/*I_*/item.setAccelerator(KeyStroke.getKeyStroke(getFrame().getProperty(keystroke)));/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[5266]++;/*I_*/return item;/*_I*/}/*I_*/
    }

    protected void add(JMenu menu, Action action) {/*_I*/_prof.prof.cnt[5267]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5268]++;/*I_*/menu.add(action);/*_I*/}/*I_*/ }

    //
    // Dialogs with user
    //

    public void open() {/*_I*/_prof.prof.cnt[5269]++;/*I_*/
        /*_I*/_prof.prof.cnt[5270]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[5271]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
//         if ((getMap() != null) && (getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile());
//         } else {
//             chooser = new JFileChooser();
//         }
        //chooser.setLocale(currentLocale);
        /*_I*/{_prof.prof.cnt[5272]++;/*I_*/if (getFileFilter() != null) {
            /*_I*/{_prof.prof.cnt[5273]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5274]++;/*I_*/int returnVal = chooser.showOpenDialog(getView());
        /*_I*/{_prof.prof.cnt[5275]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
            /*_I*/{_prof.prof.cnt[5276]++;/*I_*/try {
                /*_I*/{_prof.prof.cnt[5277]++;/*I_*/load(chooser.getSelectedFile());/*_I*/}/*I_*/
            } catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[5278]++;/*I_*/handleLoadingException (ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/ {
            }
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5279]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/
    }

    public void handleLoadingException (Exception ex) {/*_I*/_prof.prof.cnt[5280]++;/*I_*/
       /*_I*/_prof.prof.cnt[5281]++;/*I_*/String exceptionType = ex.getClass().getName();
       /*_I*/{_prof.prof.cnt[5282]++;/*I_*/if (exceptionType.equals("freemind.main.XMLParseException")) {
          /*_I*/_prof.prof.cnt[5283]++;/*I_*/int showDetail = JOptionPane.showConfirmDialog
             (getView(), getText("map_corrupted"),"FreeMind",JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE);
          /*_I*/{_prof.prof.cnt[5284]++;/*I_*/if (showDetail==JOptionPane.YES_OPTION) {
             /*_I*/{_prof.prof.cnt[5285]++;/*I_*/getController().errorMessage(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
       else /*_I*/{_prof.prof.cnt[5286]++;/*I_*/if (exceptionType.equals("java.io.FileNotFoundException")) {
          /*_I*/{_prof.prof.cnt[5287]++;/*I_*/getController().errorMessage(ex.getMessage());/*_I*/}/*I_*/ }
       else {
          /*_I*/{_prof.prof.cnt[5288]++;/*I_*/getController().errorMessage(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    /**
     * Save as; return false is the action was cancelled
     */
    public boolean saveAs() {/*_I*/_prof.prof.cnt[5289]++;/*I_*/
        /*_I*/_prof.prof.cnt[5290]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[5291]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
//         if ((getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile()); }
//         else {
//            chooser = new JFileChooser();
//            chooser.setSelectedFile(new File(((MindMapNode)getMap().getRoot()).toString()+".mm"));
//         }
        //chooser.setLocale(currentLocale);
        /*_I*/{_prof.prof.cnt[5292]++;/*I_*/if (getFileFilter() != null) {
            /*_I*/{_prof.prof.cnt[5293]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        
        /*_I*/{_prof.prof.cnt[5294]++;/*I_*/chooser.setDialogTitle(getText("save_as"));/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5295]++;/*I_*/int returnVal = chooser.showSaveDialog(getView());
        /*_I*/{_prof.prof.cnt[5296]++;/*I_*/if (returnVal != JFileChooser.APPROVE_OPTION) {// not ok pressed
        	/*_I*/{_prof.prof.cnt[5297]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        
        // |= Pressed O.K.    
        /*_I*/_prof.prof.cnt[5298]++;/*I_*/File f = chooser.getSelectedFile();
        //Force the extension to be .mm
        /*_I*/_prof.prof.cnt[5299]++;/*I_*/String ext = Tools.getExtension(f.getName());
        /*_I*/{_prof.prof.cnt[5300]++;/*I_*/if(!ext.equals("mm")) {
           /*_I*/{_prof.prof.cnt[5301]++;/*I_*/f = new File(f.getParent(),f.getName()+".mm");/*_I*/}/*I_*/ }/*_I*/}/*I_*/        
                
        /*_I*/{_prof.prof.cnt[5302]++;/*I_*/if (f.exists()) { // If file exists, ask before overwriting.
			/*_I*/_prof.prof.cnt[5303]++;/*I_*/int overwriteMap = JOptionPane.showConfirmDialog
			   (getView(), getText("map_already_exists"), "FreeMind", JOptionPane.YES_NO_OPTION );
			/*_I*/{_prof.prof.cnt[5304]++;/*I_*/if (overwriteMap != JOptionPane.YES_OPTION) {
			   /*_I*/{_prof.prof.cnt[5305]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

		/*_I*/{_prof.prof.cnt[5306]++;/*I_*/try { // We have to lock the file of the map even when it does not exist yet
		   /*_I*/_prof.prof.cnt[5307]++;/*I_*/String lockingUser = getModel().tryToLock(f);
		   /*_I*/{_prof.prof.cnt[5308]++;/*I_*/if (lockingUser != null) {          
		      /*_I*/{_prof.prof.cnt[5309]++;/*I_*/getFrame().getController().informationMessage(
			    Tools.expandPlaceholders(getText("map_locked_by_save_as"), f.getName(), lockingUser));/*_I*/}/*I_*/
		      /*_I*/{_prof.prof.cnt[5310]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
		catch (Exception e){ // Throwed by tryToLock
		  /*_I*/{_prof.prof.cnt[5311]++;/*I_*/getFrame().getController().informationMessage(
		    Tools.expandPlaceholders(getText("locking_failed_by_save_as"), f.getName()));/*_I*/}/*I_*/	 
		  /*_I*/{_prof.prof.cnt[5312]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/ 	        	                              
              
        /*_I*/{_prof.prof.cnt[5313]++;/*I_*/save(f);/*_I*/}/*I_*/
        //Update the name of the map
        /*_I*/{_prof.prof.cnt[5314]++;/*I_*/getController().getMapModuleManager().updateMapModuleName();/*_I*/}/*I_*/        
        /*_I*/{_prof.prof.cnt[5315]++;/*I_*/return true;/*_I*/}/*I_*/
    }
    /**
     * Return false if user has canceled. 
     */
    public boolean close() {/*_I*/_prof.prof.cnt[5316]++;/*I_*/
        /*_I*/_prof.prof.cnt[5317]++;/*I_*/String[] options = {getText("yes"),
                            getText("no"),
                            getText("cancel")};
        /*_I*/{_prof.prof.cnt[5318]++;/*I_*/if (!getModel().isSaved()) {
            /*_I*/_prof.prof.cnt[5319]++;/*I_*/String text = getText("save_unsaved")+"\n"+getMapModule().toString();
            /*_I*/_prof.prof.cnt[5320]++;/*I_*/String title = getText("save");
            /*_I*/_prof.prof.cnt[5321]++;/*I_*/int returnVal = JOptionPane.showOptionDialog(getFrame().getContentPane(),text,title,JOptionPane.YES_NO_CANCEL_OPTION,
                                                         JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
            /*_I*/{_prof.prof.cnt[5322]++;/*I_*/if (returnVal==JOptionPane.YES_OPTION) {
               /*_I*/_prof.prof.cnt[5323]++;/*I_*/boolean savingNotCancelled = save();
               /*_I*/{_prof.prof.cnt[5324]++;/*I_*/if (!savingNotCancelled) {
               	  /*_I*/{_prof.prof.cnt[5325]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
			else /*_I*/{_prof.prof.cnt[5326]++;/*I_*/if (returnVal==JOptionPane.CANCEL_OPTION) {
				/*_I*/{_prof.prof.cnt[5327]++;/*I_*/return false;/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*/
                
        /*_I*/{_prof.prof.cnt[5328]++;/*I_*/getModel().destroy();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5329]++;/*I_*/mapOpened(false);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5330]++;/*I_*/return true;/*_I*/}/*I_*/ }
    

    /**
     * Call this method if you have opened a map for this mode with true,
     * and if you have closed a map of this mode with false. It updates the Actions
     * that are dependent on whether there is a map or not.
     * --> What to do if either newMap or load or close are overwritten by a concrete
     * implementation? uups.
     */
     public void mapOpened(boolean open) {/*_I*/_prof.prof.cnt[5331]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5332]++;/*I_*/if (open) {
           /*_I*/{_prof.prof.cnt[5333]++;/*I_*/if (noOfMaps == 0) {
              //opened the first map
              /*_I*/{_prof.prof.cnt[5334]++;/*I_*/setAllActions(true);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5335]++;/*I_*/if (cut!=null) /*_I*/{_prof.prof.cnt[5336]++;/*I_*/cut.setEnabled(true);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5337]++;/*I_*/if (copy!=null) /*_I*/{_prof.prof.cnt[5338]++;/*I_*/copy.setEnabled(true);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5339]++;/*I_*/if (copySingle!=null) /*_I*/{_prof.prof.cnt[5340]++;/*I_*/copySingle.setEnabled(true);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5341]++;/*I_*/if (paste!=null) /*_I*/{_prof.prof.cnt[5342]++;/*I_*/paste.setEnabled(true);/*_I*/}/*I_*//*_I*/}/*I_*/
           }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5343]++;/*I_*/if (getFrame().getView()!=null) {
              /*_I*/_prof.prof.cnt[5344]++;/*I_*/DropTarget dropTarget = new DropTarget
                 (getFrame().getView(), new FileOpener() );
           }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5345]++;/*I_*/noOfMaps++;/*_I*/}/*I_*/
        } else {
           /*_I*/{_prof.prof.cnt[5346]++;/*I_*/noOfMaps--;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5347]++;/*I_*/if (noOfMaps == 0) {
              //closed the last map
              /*_I*/{_prof.prof.cnt[5348]++;/*I_*/setAllActions(false);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5349]++;/*I_*/if (cut!=null) /*_I*/{_prof.prof.cnt[5350]++;/*I_*/cut.setEnabled(false);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5351]++;/*I_*/if (copy!=null) /*_I*/{_prof.prof.cnt[5352]++;/*I_*/copy.setEnabled(false);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5353]++;/*I_*/if (copySingle!=null) /*_I*/{_prof.prof.cnt[5354]++;/*I_*/copySingle.setEnabled(true);/*_I*/}/*I_*//*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5355]++;/*I_*/if (paste!=null) /*_I*/{_prof.prof.cnt[5356]++;/*I_*/paste.setEnabled(false);/*_I*/}/*I_*//*_I*/}/*I_*/
           }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    /**
     * Overwrite this to set all of your actions which are
     * dependent on whether there is a map or not.
     */
    protected void setAllActions(boolean enabled) {/*_I*/_prof.prof.cnt[5357]++;/*I_*/
    }

    //
    // Node editing
    //

    private JPopupMenu popupmenu;

    /** listener, that blocks the controler if the menu is active (PN)
        Take care! This listener is also used for modelpopups (as for graphical links).*/
    private class ControllerPopupMenuListener implements PopupMenuListener  {
      public void popupMenuWillBecomeVisible(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5358]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5359]++;/*I_*/setBlocked(true);/*_I*/}/*I_*/         // block controller
      }
      public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5360]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5361]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/        // unblock controller
      }
      public void popupMenuCanceled(PopupMenuEvent e) {/*_I*/_prof.prof.cnt[5362]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5363]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/        // unblock controller
      }

    }
    /** Take care! This listener is also used for modelpopups (as for graphical links).*/
    protected final ControllerPopupMenuListener popupListenerSingleton
        = new ControllerPopupMenuListener();
    
    public void showPopupMenu(MouseEvent e) {/*_I*/_prof.prof.cnt[5364]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5365]++;/*I_*/if (e.isPopupTrigger()) {
        /*_I*/_prof.prof.cnt[5366]++;/*I_*/JPopupMenu popupmenu = getPopupMenu();
        /*_I*/{_prof.prof.cnt[5367]++;/*I_*/if (popupmenu != null) {
          // adding listener could be optimized but without much profit...
          /*_I*/{_prof.prof.cnt[5368]++;/*I_*/popupmenu.addPopupMenuListener( this.popupListenerSingleton );/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5369]++;/*I_*/popupmenu.show(e.getComponent(),e.getX(),e.getY());/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5370]++;/*I_*/e.consume();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
    }

    /** Default implementation: no context menu.*/
    public JPopupMenu getPopupForModel(java.lang.Object obj) {/*_I*/_prof.prof.cnt[5371]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5372]++;/*I_*/return null;/*_I*/}/*I_*/
    }
        


    private static final int SCROLL_SKIPS = 8;
    private static final int SCROLL_SKIP = 10;
    private static final int HORIZONTAL_SCROLL_MASK 
       = InputEvent.SHIFT_MASK | InputEvent.BUTTON1_MASK 
         | InputEvent.BUTTON2_MASK | InputEvent.BUTTON3_MASK;
    private static final int ZOOM_MASK 
       = InputEvent.CTRL_MASK;
      // |=   oldX >=0 iff we are in the drag

    public void mouseWheelMoved(MouseWheelEvent e) {/*_I*/_prof.prof.cnt[5373]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5374]++;/*I_*/if (isBlocked()) {
         /*_I*/{_prof.prof.cnt[5375]++;/*I_*/return;/*_I*/}/*I_*/ // block the scroll during edit (PN)
       }/*_I*/}/*I_*/
        
       /*_I*/{_prof.prof.cnt[5376]++;/*I_*/if ((e.getModifiers() & ZOOM_MASK) != 0) {
           // fc, 18.11.2003: when control pressed, then the zoom is changed.
           /*_I*/_prof.prof.cnt[5377]++;/*I_*/float newZoomFactor = 1f + Math.abs((float) e.getWheelRotation())/10f;
           /*_I*/{_prof.prof.cnt[5378]++;/*I_*/if(e.getWheelRotation() < 0)
               /*_I*/{_prof.prof.cnt[5379]++;/*I_*/newZoomFactor = 1 / newZoomFactor;/*_I*/}/*I_*//*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5380]++;/*I_*/float newZoom = ((MapView)e.getComponent()).getZoom() * newZoomFactor;
           // round the value due to possible rounding problems.
           /*_I*/{_prof.prof.cnt[5381]++;/*I_*/newZoom =  (float) Math.rint(newZoom*1000f)/1000f;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5382]++;/*I_*/getController().setZoom(newZoom);/*_I*/}/*I_*/
           // end zoomchange
       } else /*_I*/{_prof.prof.cnt[5383]++;/*I_*/if ((e.getModifiers() & HORIZONTAL_SCROLL_MASK) != 0) {
          /*_I*/{_prof.prof.cnt[5384]++;/*I_*/for (int i=0; i < SCROLL_SKIPS; i++) {
             /*_I*/{_prof.prof.cnt[5385]++;/*I_*/((MapView)e.getComponent()).scrollBy(
                 SCROLL_SKIP * e.getWheelRotation(), 0);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[5386]++;/*I_*/for (int i=0; i < SCROLL_SKIPS; i++) {
             /*_I*/{_prof.prof.cnt[5387]++;/*I_*/((MapView)e.getComponent()).scrollBy(0, 
                 SCROLL_SKIP * e.getWheelRotation());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    // edit begins with home/end or typing (PN 6.2)
    public void edit(KeyEvent e, boolean addNew, boolean editLong) {/*_I*/_prof.prof.cnt[5388]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5389]++;/*I_*/if (getView().getSelected() != null) {
        /*_I*/{_prof.prof.cnt[5390]++;/*I_*/if (e == null || !addNew) {
          /*_I*/{_prof.prof.cnt[5391]++;/*I_*/edit(getView().getSelected(),getView().getSelected(), e, false, false, editLong);/*_I*/}/*I_*/
        }
        else /*_I*/{_prof.prof.cnt[5392]++;/*I_*/if (!isBlocked()) {
          /*_I*/{_prof.prof.cnt[5393]++;/*I_*/addNew(getView().getSelected(), NEW_SIBLING_BEHIND, e);/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5394]++;/*I_*/if (e != null) {
          /*_I*/{_prof.prof.cnt[5395]++;/*I_*/e.consume();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
    }

   private void changeComponentHeight(JComponent component, int difference, int minimum) {/*_I*/_prof.prof.cnt[5396]++;/*I_*/
      /*_I*/_prof.prof.cnt[5397]++;/*I_*/Dimension preferredSize = component.getPreferredSize();
      /*_I*/{_prof.prof.cnt[5398]++;/*I_*/System.out.println("pf:"+preferredSize);/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[5399]++;/*I_*/if (preferredSize.getHeight() + difference >= minimum) {
         /*_I*/{_prof.prof.cnt[5400]++;/*I_*/System.out.println("pf:"+preferredSize);/*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[5401]++;/*I_*/component.setPreferredSize(new Dimension((int)preferredSize.getWidth(),
                                                  (int)preferredSize.getHeight() + difference));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    /** Private variable to hold the last value of the "Enter confirms" state.*/
    private static Tools.BooleanHolder booleanHolderForConfirmState;

    private void editLong(final NodeView node,
                            final String text,
                            final KeyEvent firstEvent) {/*_I*/_prof.prof.cnt[5402]++;/*I_*/

        /*_I*/_prof.prof.cnt[5403]++;/*I_*/final int BUTTON_OK     = 0;
        /*_I*/_prof.prof.cnt[5404]++;/*I_*/final int BUTTON_CANCEL = 1;
        /*_I*/_prof.prof.cnt[5405]++;/*I_*/final int BUTTON_SPLIT  = 2;

        /*_I*/_prof.prof.cnt[5406]++;/*I_*/final JDialog dialog = new JDialog((JFrame)getFrame(), getText("edit_long_node"), /*modal=*/true);

        /*_I*/_prof.prof.cnt[5407]++;/*I_*/final JTextArea textArea = new JTextArea(text);
        /*_I*/{_prof.prof.cnt[5408]++;/*I_*/textArea.setLineWrap(true);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5409]++;/*I_*/textArea.setWrapStyleWord(true);/*_I*/}/*I_*/ // wrap around words rather than characters
        /*_I*/{_prof.prof.cnt[5410]++;/*I_*/if (firstEvent != null) {
          /*_I*/{_prof.prof.cnt[5411]++;/*I_*/switch (firstEvent.getKeyCode()) {
            case KeyEvent.VK_HOME:
              /*_I*/{_prof.prof.cnt[5412]++;/*I_*/textArea.setCaretPosition(0);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5413]++;/*I_*/break;/*_I*/}/*I_*/

            default:
              /*_I*/{_prof.prof.cnt[5414]++;/*I_*/textArea.setCaretPosition(text.length());/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5415]++;/*I_*/break;/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }
        else {
          /*_I*/{_prof.prof.cnt[5416]++;/*I_*/textArea.setCaretPosition(text.length());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/


        /*_I*/_prof.prof.cnt[5417]++;/*I_*/final JScrollPane editorScrollPane = new JScrollPane(textArea);
        /*_I*/{_prof.prof.cnt[5418]++;/*I_*/editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);/*_I*/}/*I_*/

        //int preferredHeight = new Integer(getFrame().getProperty("el__default_window_height")).intValue();
        /*_I*/_prof.prof.cnt[5419]++;/*I_*/int preferredHeight = node.getHeight();
        /*_I*/{_prof.prof.cnt[5420]++;/*I_*/preferredHeight =
           Math.max (preferredHeight, Integer.parseInt(getFrame().getProperty("el__min_default_window_height")));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5421]++;/*I_*/preferredHeight =
           Math.min (preferredHeight, Integer.parseInt(getFrame().getProperty("el__max_default_window_height")));/*_I*/}/*I_*/

        /*_I*/_prof.prof.cnt[5422]++;/*I_*/int preferredWidth = node.getWidth();
        /*_I*/{_prof.prof.cnt[5423]++;/*I_*/preferredWidth =
           Math.max (preferredWidth, Integer.parseInt(getFrame().getProperty("el__min_default_window_width")));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5424]++;/*I_*/preferredWidth =
           Math.min (preferredWidth, Integer.parseInt(getFrame().getProperty("el__max_default_window_width")));/*_I*/}/*I_*/
                           
        /*_I*/{_prof.prof.cnt[5425]++;/*I_*/editorScrollPane.setPreferredSize(new Dimension(preferredWidth, preferredHeight));/*_I*/}/*I_*/
        //textArea.setPreferredSize(new Dimension(500, 160));

        /*_I*/_prof.prof.cnt[5426]++;/*I_*/final JPanel panel = new JPanel();

        //String performedAction;
        /*_I*/_prof.prof.cnt[5427]++;/*I_*/final Tools.IntHolder eventSource = new Tools.IntHolder();
        /*_I*/_prof.prof.cnt[5428]++;/*I_*/final JButton okButton = new JButton("OK");
        /*_I*/_prof.prof.cnt[5429]++;/*I_*/final JButton cancelButton = new JButton(getText("cancel"));
        /*_I*/_prof.prof.cnt[5430]++;/*I_*/final JButton splitButton = new JButton(getText("split"));
        /*_I*/_prof.prof.cnt[5431]++;/*I_*/final JCheckBox enterConfirms =
           new JCheckBox(getText("enter_confirms"), binOptionIsTrue("el__enter_confirms_by_default"));

        /*_I*/{_prof.prof.cnt[5432]++;/*I_*/if(booleanHolderForConfirmState == null) {
            /*_I*/{_prof.prof.cnt[5433]++;/*I_*/booleanHolderForConfirmState = new Tools.BooleanHolder();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5434]++;/*I_*/booleanHolderForConfirmState.setValue(enterConfirms.isSelected());/*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[5435]++;/*I_*/enterConfirms.setSelected(booleanHolderForConfirmState.getValue());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/            

        /*_I*/{_prof.prof.cnt[5436]++;/*I_*/okButton.setMnemonic(KeyEvent.VK_O);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5437]++;/*I_*/enterConfirms.setMnemonic(KeyEvent.VK_E);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5438]++;/*I_*/splitButton.setMnemonic(KeyEvent.VK_S);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5439]++;/*I_*/cancelButton.setMnemonic(KeyEvent.VK_C);/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5440]++;/*I_*/okButton.addActionListener (new ActionListener() {
              public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5441]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5442]++;/*I_*/eventSource.setValue(BUTTON_OK);/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5443]++;/*I_*/dialog.dispose();/*_I*/}/*I_*/ }});/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5444]++;/*I_*/cancelButton.addActionListener (new ActionListener() {
              public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5445]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5446]++;/*I_*/eventSource.setValue(BUTTON_CANCEL);/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5447]++;/*I_*/dialog.dispose();/*_I*/}/*I_*/ }});/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5448]++;/*I_*/splitButton.addActionListener (new ActionListener() {
              public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5449]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5450]++;/*I_*/eventSource.setValue(BUTTON_SPLIT);/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5451]++;/*I_*/dialog.dispose();/*_I*/}/*I_*/ }});/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5452]++;/*I_*/enterConfirms.addActionListener (new ActionListener() {
              public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5453]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5454]++;/*I_*/textArea.requestFocus();/*_I*/}/*I_*/ 
                 /*_I*/{_prof.prof.cnt[5455]++;/*I_*/booleanHolderForConfirmState.setValue(enterConfirms.isSelected());/*_I*/}/*I_*/
              }});/*_I*/}/*I_*/

        // On Enter act as if OK button was pressed

        /*_I*/{_prof.prof.cnt[5456]++;/*I_*/textArea.addKeyListener(new KeyListener() {
              public void keyPressed(KeyEvent e) {/*_I*/_prof.prof.cnt[5457]++;/*I_*/
                 // escape key in long text editor (PN)
                 /*_I*/{_prof.prof.cnt[5458]++;/*I_*/if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
                   /*_I*/{_prof.prof.cnt[5459]++;/*I_*/e.consume();/*_I*/}/*I_*/
                   /*_I*/{_prof.prof.cnt[5460]++;/*I_*/eventSource.setValue(BUTTON_CANCEL);/*_I*/}/*I_*/
                   /*_I*/{_prof.prof.cnt[5461]++;/*I_*/dialog.dispose();/*_I*/}/*I_*/ 
                 }
                 else /*_I*/{_prof.prof.cnt[5462]++;/*I_*/if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    /*_I*/{_prof.prof.cnt[5463]++;/*I_*/if (enterConfirms.isSelected() == ((e.getModifiers() & KeyEvent.CTRL_MASK) == 0)) {
                       /*_I*/{_prof.prof.cnt[5464]++;/*I_*/e.consume();/*_I*/}/*I_*/
                       /*_I*/{_prof.prof.cnt[5465]++;/*I_*/eventSource.setValue(BUTTON_OK);/*_I*/}/*I_*/
                       /*_I*/{_prof.prof.cnt[5466]++;/*I_*/dialog.dispose();/*_I*/}/*I_*/ }
                    else /*_I*/{_prof.prof.cnt[5467]++;/*I_*/if (enterConfirms.isSelected() && (e.getModifiers() & KeyEvent.CTRL_MASK) != 0) {
                       /*_I*/{_prof.prof.cnt[5468]++;/*I_*/e.consume();/*_I*/}/*I_*/
                       /*_I*/{_prof.prof.cnt[5469]++;/*I_*/textArea.insert("\n",textArea.getCaretPosition());/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*/}

                 /*
                 // Daniel: I tried to make editor resizable. It worked somehow, but not
                 // quite. When I increased the size and then decreased again to the original
                 // size, it stopped working. The main idea here is to change the preferred
                 // size. I also tried to change the size but it did not do anything sensible.
                 //
                 // One possibility would be to disable decreasing the size.
                 //
                 // Another thing which was far from nice was that it flickered.
                 //
                 // If someone wants to find a solution, let it be.

                 else if (e.getKeyCode() == KeyEvent.VK_DOWN &&
                          (e.getModifiers() & KeyEvent.CTRL_MASK) != 0) {

                    changeComponentHeight(editorScrollPane, 60, 180);
                    dialog.doLayout();
                    dialog.pack();
                    e.consume();
                 }
                 else if (e.getKeyCode() == KeyEvent.VK_UP &&
                          (e.getModifiers() & KeyEvent.CTRL_MASK) != 0) {
                    changeComponentHeight(editorScrollPane, -60, 180);
                    dialog.doLayout();
                    dialog.pack();
                    e.consume(); }} */
              public void keyTyped(KeyEvent e) {/*_I*/_prof.prof.cnt[5470]++;/*I_*/}
              public void keyReleased(KeyEvent e) {/*_I*/_prof.prof.cnt[5471]++;/*I_*/}
           });/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5472]++;/*I_*/textArea.addMouseListener(new MouseListener() {
              public void mouseClicked(MouseEvent e) {/*_I*/_prof.prof.cnt[5473]++;/*I_*/}
              public void mouseEntered( MouseEvent e ) {/*_I*/_prof.prof.cnt[5474]++;/*I_*/}
              public void mouseExited( MouseEvent e ) {/*_I*/_prof.prof.cnt[5475]++;/*I_*/}

              public void mousePressed( MouseEvent e ) {/*_I*/_prof.prof.cnt[5476]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5477]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/ }
              
              public void mouseReleased( MouseEvent e ) {/*_I*/_prof.prof.cnt[5478]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5479]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/ }

              private void conditionallyShowPopup(MouseEvent e) {/*_I*/_prof.prof.cnt[5480]++;/*I_*/
                 /*_I*/{_prof.prof.cnt[5481]++;/*I_*/if (e.isPopupTrigger()) {
                    /*_I*/_prof.prof.cnt[5482]++;/*I_*/JPopupMenu popupMenu = new EditPopupMenu(textArea);
                    /*_I*/{_prof.prof.cnt[5483]++;/*I_*/popupMenu.show(e.getComponent(),e.getX(),e.getY());/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5484]++;/*I_*/e.consume();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
           });/*_I*/}/*I_*/


        /*_I*/{_prof.prof.cnt[5485]++;/*I_*/textArea.setFont(node.getFont());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5486]++;/*I_*/textArea.setForeground(node.getForeground());/*_I*/}/*I_*/

        //panel.setPreferredSize(new Dimension(500, 160));
        //editorScrollPane.setPreferredSize(new Dimension(500, 160));

        /*_I*/_prof.prof.cnt[5487]++;/*I_*/JPanel buttonPane = new JPanel();
        /*_I*/{_prof.prof.cnt[5488]++;/*I_*/buttonPane.add(enterConfirms);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5489]++;/*I_*/buttonPane.add(okButton);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5490]++;/*I_*/buttonPane.add(cancelButton);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5491]++;/*I_*/buttonPane.add(splitButton);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5492]++;/*I_*/buttonPane.setMaximumSize(new Dimension(1000, 20));/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5493]++;/*I_*/if (getFrame().getProperty("el__buttons_position").equals("above")) {
           /*_I*/{_prof.prof.cnt[5494]++;/*I_*/panel.add(buttonPane);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5495]++;/*I_*/panel.add(editorScrollPane);/*_I*/}/*I_*/ }
        else {
           /*_I*/{_prof.prof.cnt[5496]++;/*I_*/panel.add(editorScrollPane);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5497]++;/*I_*/panel.add(buttonPane);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5498]++;/*I_*/panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));/*_I*/}/*I_*/
      
        /*_I*/{_prof.prof.cnt[5499]++;/*I_*/dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5500]++;/*I_*/dialog.setContentPane(panel);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5501]++;/*I_*/dialog.pack();/*_I*/}/*I_*/  // calculate the size
        
        // set position
        /*_I*/{_prof.prof.cnt[5502]++;/*I_*/getView().scrollNodeToVisible(node, 0);/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5503]++;/*I_*/Point frameScreenLocation = getFrame().getLayeredPane().getLocationOnScreen();
        /*_I*/_prof.prof.cnt[5504]++;/*I_*/double posX = node.getLocationOnScreen().getX() - frameScreenLocation.getX();
        /*_I*/_prof.prof.cnt[5505]++;/*I_*/double posY = node.getLocationOnScreen().getY() - frameScreenLocation.getY()
           + (binOptionIsTrue("el__position_window_below_node") ? node.getHeight() : 0);
        /*_I*/{_prof.prof.cnt[5506]++;/*I_*/if (posX + dialog.getWidth() > getFrame().getLayeredPane().getWidth()) {
          /*_I*/{_prof.prof.cnt[5507]++;/*I_*/posX = getFrame().getLayeredPane().getWidth() - dialog.getWidth();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5508]++;/*I_*/if (posY + dialog.getHeight() > getFrame().getLayeredPane().getHeight()) {
          /*_I*/{_prof.prof.cnt[5509]++;/*I_*/posY = getFrame().getLayeredPane().getHeight() - dialog.getHeight();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5510]++;/*I_*/posX = ((posX < 0) ? 0 : posX) + frameScreenLocation.getX();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5511]++;/*I_*/posY = ((posY < 0) ? 0 : posY) + frameScreenLocation.getY();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5512]++;/*I_*/dialog.setLocation(new Double(posX).intValue(), new Double(posY).intValue());/*_I*/}/*I_*/


        /*_I*/{_prof.prof.cnt[5513]++;/*I_*/textArea.requestFocusInWindow();/*_I*/}/*I_*/  // make the text area focused when the dialog comes
        /*_I*/{_prof.prof.cnt[5514]++;/*I_*/dialog.show();/*_I*/}/*I_*/
        //dialog.setVisible(true);

        /*_I*/{_prof.prof.cnt[5515]++;/*I_*/if (eventSource.getValue() == BUTTON_OK) {
           /*_I*/{_prof.prof.cnt[5516]++;/*I_*/getModel().changeNode(node.getModel(), textArea.getText());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5517]++;/*I_*/if (eventSource.getValue() == BUTTON_SPLIT) {
           //getModel().changeNode(node.getModel(), textArea.getText());
           /*_I*/{_prof.prof.cnt[5518]++;/*I_*/getModel().splitNode(node.getModel(),
                                textArea.getCaretPosition(),
                                textArea.getText());/*_I*/}/*I_*/ 
           /*_I*/{_prof.prof.cnt[5519]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // focus fix
        }/*_I*/}/*I_*/

    }

    // this enables from outside close the edit mode
    private FocusListener textFieldListener = null;
    
    private void closeEdit() {/*_I*/_prof.prof.cnt[5520]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5521]++;/*I_*/if (this.textFieldListener != null) {
        /*_I*/{_prof.prof.cnt[5522]++;/*I_*/textFieldListener.focusLost(null);/*_I*/}/*I_*/ // hack to close the edit
      }/*_I*/}/*I_*/
    }
    
    // status, currently: default, blocked  (PN)
    // (blocked to protect against particular events e.g. in edit mode)
    private boolean isBlocked = false;

    public boolean isBlocked() {/*_I*/_prof.prof.cnt[5523]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5524]++;/*I_*/return this.isBlocked;/*_I*/}/*I_*/
    }
    private void setBlocked(boolean isBlocked) {/*_I*/_prof.prof.cnt[5525]++;/*I_*/
      /*_I*/{_prof.prof.cnt[5526]++;/*I_*/this.isBlocked = isBlocked;/*_I*/}/*I_*/
    }
    
    private void edit(final NodeView node, 
                        final NodeView prevSelected,   // when new->esc: node be selected
                        final KeyEvent firstEvent,
                        final boolean isNewNode,      // when new->esc: cut the node
                        final boolean parentFolded,   // when new->esc: fold prevSelected
                        final boolean editLong) {/*_I*/_prof.prof.cnt[5527]++;/*I_*/
		/*_I*/{_prof.prof.cnt[5528]++;/*I_*/if (node == null){
			/*_I*/{_prof.prof.cnt[5529]++;/*I_*/return;/*_I*/}/*I_*/	}/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5530]++;/*I_*/closeEdit();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5531]++;/*I_*/setBlocked(true);/*_I*/}/*I_*/ // locally "modal" stated

        /*_I*/_prof.prof.cnt[5532]++;/*I_*/String text = node.getModel().toString();
        /*_I*/{_prof.prof.cnt[5533]++;/*I_*/if (node.getIsLong() || editLong) {
           /*_I*/{_prof.prof.cnt[5534]++;/*I_*/editLong(node, text, firstEvent);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5535]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5536]++;/*I_*/return;/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/

        //if (isNewNode) {
        //  if (firstEvent instanceof KeyEvent
        //      && ((KeyEvent)firstEvent).getKeyChar() != KeyEvent.CHAR_UNDEFINED) {
        //  }
        //  //else if (text.length() == 0) {
        //    // new node text if the user did not press a key and the text would be empty
        //    // text = getText("new_node");
        //  //}
        //}
        
        /*_I*/_prof.prof.cnt[5537]++;/*I_*/final JTextField textField = (text.length() < 8)
           ? new JTextField(text,8)     //Make fields for short texts editable
           : new JTextField(text);

        // Set textFields's properties

        /* fc, 12.10.2003: the following method is not correct. Even more with the zoom factors!*/
        /*_I*/_prof.prof.cnt[5538]++;/*I_*/int linkIconWidth = 16;
        /*_I*/_prof.prof.cnt[5539]++;/*I_*/int textFieldBorderWidth = 2;
        /*_I*/_prof.prof.cnt[5540]++;/*I_*/int cursorWidth = 1;
        /*_I*/_prof.prof.cnt[5541]++;/*I_*/int xOffset = -1 * textFieldBorderWidth + node.getLeftWidthOverhead() - 1;
        /*_I*/_prof.prof.cnt[5542]++;/*I_*/int yOffset = -1; // Optimized for Windows style; basically ad hoc
        /*_I*/_prof.prof.cnt[5543]++;/*I_*/int widthAddition = 2 * textFieldBorderWidth + cursorWidth - 2 * node.getLeftWidthOverhead() + 2;
        /*_I*/_prof.prof.cnt[5544]++;/*I_*/int heightAddition = 2;
        /*_I*/{_prof.prof.cnt[5545]++;/*I_*/if (node.getModel().getLink() != null) {
           /*_I*/{_prof.prof.cnt[5546]++;/*I_*/xOffset += linkIconWidth;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5547]++;/*I_*/widthAddition -= linkIconWidth;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5548]++;/*I_*/if (node.getModel().getIcons().size() != 0) { // fc, 24.9.2003 full ok for the moment, that an icon has the same size as the link icon.
           /*_I*/{_prof.prof.cnt[5549]++;/*I_*/xOffset += linkIconWidth * node.getModel().getIcons().size();/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5550]++;/*I_*/widthAddition -= linkIconWidth;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /* fc, 12.10.2003: end buggy method*/

        // minimal width for input field of leaf or folded node (PN)
        /*_I*/_prof.prof.cnt[5551]++;/*I_*/final int MINIMAL_LEAF_WIDTH = 150;
        /*_I*/_prof.prof.cnt[5552]++;/*I_*/final int MINIMAL_WIDTH = 50;

        /*_I*/_prof.prof.cnt[5553]++;/*I_*/int xSize = node.getWidth() + widthAddition;
        /*_I*/_prof.prof.cnt[5554]++;/*I_*/int xExtraWidth = 0;
        /*_I*/{_prof.prof.cnt[5555]++;/*I_*/if (MINIMAL_LEAF_WIDTH > xSize 
            && (node.getModel().isFolded() || !node.getModel().hasChildren())) {
          // leaf or folded node with small size
          /*_I*/{_prof.prof.cnt[5556]++;/*I_*/xExtraWidth = MINIMAL_LEAF_WIDTH - xSize;/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5557]++;/*I_*/xSize = MINIMAL_LEAF_WIDTH;/*_I*/}/*I_*/ // increase minimum size
          /*_I*/{_prof.prof.cnt[5558]++;/*I_*/if (node.isLeft()) { // left leaf
            /*_I*/{_prof.prof.cnt[5559]++;/*I_*/xExtraWidth = - xExtraWidth;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5560]++;/*I_*/textField.setHorizontalAlignment(JTextField.RIGHT);/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }
        else /*_I*/{_prof.prof.cnt[5561]++;/*I_*/if (MINIMAL_WIDTH > xSize) {
          // opened node with small size
          /*_I*/{_prof.prof.cnt[5562]++;/*I_*/xExtraWidth = MINIMAL_WIDTH - xSize;/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[5563]++;/*I_*/xSize = MINIMAL_WIDTH;/*_I*/}/*I_*/ // increase minimum size
          /*_I*/{_prof.prof.cnt[5564]++;/*I_*/if (node.isLeft()) { // left node
            /*_I*/{_prof.prof.cnt[5565]++;/*I_*/xExtraWidth = - xExtraWidth;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5566]++;/*I_*/textField.setHorizontalAlignment(JTextField.RIGHT);/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5567]++;/*I_*/textField.setSize(xSize, node.getHeight() + heightAddition);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5568]++;/*I_*/textField.setFont(node.getFont());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5569]++;/*I_*/textField.setForeground(node.getForeground());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5570]++;/*I_*/textField.setSelectedTextColor(node.getForeground());/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5571]++;/*I_*/textField.setSelectionColor(selectionColor);/*_I*/}/*I_*/
        // textField.selectAll(); // no selection on edit (PN)
        
        /*_I*/_prof.prof.cnt[5572]++;/*I_*/final int INIT   = 0;
        /*_I*/_prof.prof.cnt[5573]++;/*I_*/final int EDIT   = 1;
        /*_I*/_prof.prof.cnt[5574]++;/*I_*/final int CANCEL = 2;
        /*_I*/_prof.prof.cnt[5575]++;/*I_*/final Tools.IntHolder eventSource = new Tools.IntHolder();
        /*_I*/{_prof.prof.cnt[5576]++;/*I_*/eventSource.setValue(INIT);/*_I*/}/*I_*/

        // listener class
        class TextFieldListener implements KeyListener, 
                                             FocusListener, MouseListener {

          public void focusGained(FocusEvent e) {/*_I*/_prof.prof.cnt[5577]++;/*I_*/
            // the first time the edit field gains a focus
            // process the predefined first key (if any)

            /*_I*/{_prof.prof.cnt[5578]++;/*I_*/if (eventSource.getValue() == INIT) {
              /*_I*/{_prof.prof.cnt[5579]++;/*I_*/eventSource.setValue(EDIT);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5580]++;/*I_*/if (firstEvent instanceof KeyEvent) {
                /*_I*/_prof.prof.cnt[5581]++;/*I_*/KeyEvent firstKeyEvent = (KeyEvent)firstEvent;
                /*_I*/{_prof.prof.cnt[5582]++;/*I_*/if (firstKeyEvent.getKeyChar() == KeyEvent.CHAR_UNDEFINED) {
                  // for the char_undefined the scenario with dispatching 
                  // doesn't work => hard code dispatching :-(
                  // // dispatch action key events as it came
                  // textField.dispatchEvent(firstKeyEvent);
                  
                  // dispatch 2 known events (+ special for insert:) (hardcoded)
                  /*_I*/{_prof.prof.cnt[5583]++;/*I_*/switch (firstKeyEvent.getKeyCode()) {
                    case KeyEvent.VK_HOME:
                      /*_I*/{_prof.prof.cnt[5584]++;/*I_*/textField.setCaretPosition(0);/*_I*/}/*I_*/
                      /*_I*/{_prof.prof.cnt[5585]++;/*I_*/break;/*_I*/}/*I_*/
                    case KeyEvent.VK_END:
                      /*_I*/{_prof.prof.cnt[5586]++;/*I_*/textField.setCaretPosition(textField.getText().length());/*_I*/}/*I_*/
                      /*_I*/{_prof.prof.cnt[5587]++;/*I_*/break;/*_I*/}/*I_*/
                  }/*_I*/}/*I_*/
                }
                else {
                  // or create new "key type" event for printable key
                  /*_I*/_prof.prof.cnt[5588]++;/*I_*/KeyEvent keyEv;
                  /*_I*/{_prof.prof.cnt[5589]++;/*I_*/keyEv = new KeyEvent(
                      firstKeyEvent.getComponent(),
                      KeyEvent.KEY_TYPED, 
                      firstKeyEvent.getWhen(), 
                      firstKeyEvent.getModifiers(), 
                      KeyEvent.VK_UNDEFINED,
                      firstKeyEvent.getKeyChar(), 
                      KeyEvent.KEY_LOCATION_UNKNOWN );/*_I*/}/*I_*/
                  /*_I*/{_prof.prof.cnt[5590]++;/*I_*/textField.selectAll();/*_I*/}/*I_*/ // to enable overwrite
                  /*_I*/{_prof.prof.cnt[5591]++;/*I_*/textField.dispatchEvent(keyEv);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
              }/*_I*/}/*I_*/ // 1st key event defined
            }/*_I*/}/*I_*/ // first focus
          } // focus gained


          public void focusLost(FocusEvent e) {/*_I*/_prof.prof.cnt[5592]++;/*I_*/
            
            // %%% open problems:
            // - adding of a child to the rightmost node
            // - scrolling while in editing mode (it can behave just like other viewers)
            // - block selected events while in editing mode
        
            /*_I*/{_prof.prof.cnt[5593]++;/*I_*/if (e == null) { // can be when called explicitly
              /*_I*/{_prof.prof.cnt[5594]++;/*I_*/getModel().changeNode(node.getModel(), textField.getText());/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5595]++;/*I_*/getFrame().getLayeredPane().remove(textField);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5596]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
              /*_I*/{_prof.prof.cnt[5597]++;/*I_*/textFieldListener = null;/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5598]++;/*I_*/eventSource.setValue(CANCEL);/*_I*/}/*I_*/ // disallow real focus lost
            }
            else /*_I*/{_prof.prof.cnt[5599]++;/*I_*/if (eventSource.getValue() != CANCEL) {
              // always confirm the text if not yet
              /*_I*/{_prof.prof.cnt[5600]++;/*I_*/getModel().changeNode(node.getModel(), textField.getText());/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5601]++;/*I_*/getFrame().getLayeredPane().remove(textField);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5602]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
              /*_I*/{_prof.prof.cnt[5603]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5604]++;/*I_*/textFieldListener = null;/*_I*/}/*I_*/
            }/*_I*/}/*I_*//*_I*/}/*I_*/
          }

          public void keyPressed(KeyEvent e) {/*_I*/_prof.prof.cnt[5605]++;/*I_*/

            /*_I*/{_prof.prof.cnt[5606]++;/*I_*/if (e.isAltDown() || e.isControlDown()) {
                /*_I*/{_prof.prof.cnt[5607]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/

            /*_I*/_prof.prof.cnt[5608]++;/*I_*/boolean commit = true;

            /*_I*/{_prof.prof.cnt[5609]++;/*I_*/switch (e.getKeyCode()) {
              case KeyEvent.VK_ESCAPE:
                /*_I*/{_prof.prof.cnt[5610]++;/*I_*/commit = false;/*_I*/}/*I_*/
              case KeyEvent.VK_ENTER:
                /*_I*/{_prof.prof.cnt[5611]++;/*I_*/e.consume();/*_I*/}/*I_*/

                /*_I*/{_prof.prof.cnt[5612]++;/*I_*/eventSource.setValue(CANCEL);/*_I*/}/*I_*/ // do not process loose of focus
                /*_I*/{_prof.prof.cnt[5613]++;/*I_*/if (commit) {
                  /*_I*/{_prof.prof.cnt[5614]++;/*I_*/getModel().changeNode(node.getModel(), textField.getText());/*_I*/}/*I_*/
                }
                else /*_I*/{_prof.prof.cnt[5615]++;/*I_*/if (isNewNode) { // delete also the node and set focus to the parent
                  /*_I*/{_prof.prof.cnt[5616]++;/*I_*/getView().selectAsTheOnlyOneSelected(node);/*_I*/}/*I_*/
                  /*_I*/{_prof.prof.cnt[5617]++;/*I_*/getModel().cut();/*_I*/}/*I_*/
                  /*_I*/{_prof.prof.cnt[5618]++;/*I_*/select(prevSelected);/*_I*/}/*I_*/ // include max level for navigation
                  /*_I*/{_prof.prof.cnt[5619]++;/*I_*/if (parentFolded) {
                    /*_I*/{_prof.prof.cnt[5620]++;/*I_*/getModel().setFolded(prevSelected.getModel(), true);/*_I*/}/*I_*/
                  }/*_I*/}/*I_*/
                }/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5621]++;/*I_*/getFrame().getLayeredPane().remove(textField);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5622]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
                /*_I*/{_prof.prof.cnt[5623]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5624]++;/*I_*/textFieldListener = null;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5625]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // hack: to keep the focus
                /*_I*/{_prof.prof.cnt[5626]++;/*I_*/break;/*_I*/}/*I_*/

             case KeyEvent.VK_SPACE: 
               /*_I*/{_prof.prof.cnt[5627]++;/*I_*/e.consume();/*_I*/}/*I_*/
             }/*_I*/}/*I_*/
          }
          public void keyTyped(KeyEvent e) {/*_I*/_prof.prof.cnt[5628]++;/*I_*/ }
          public void keyReleased(KeyEvent e) {/*_I*/_prof.prof.cnt[5629]++;/*I_*/ }

           public void mouseClicked(MouseEvent e) {/*_I*/_prof.prof.cnt[5630]++;/*I_*/}
           public void mouseEntered( MouseEvent e ) {/*_I*/_prof.prof.cnt[5631]++;/*I_*/}
           public void mouseExited( MouseEvent e ) {/*_I*/_prof.prof.cnt[5632]++;/*I_*/}

           public void mousePressed( MouseEvent e ) {/*_I*/_prof.prof.cnt[5633]++;/*I_*/
              /*_I*/{_prof.prof.cnt[5634]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/ }
              
           public void mouseReleased( MouseEvent e ) {/*_I*/_prof.prof.cnt[5635]++;/*I_*/
              /*_I*/{_prof.prof.cnt[5636]++;/*I_*/conditionallyShowPopup(e);/*_I*/}/*I_*/ }

           private void conditionallyShowPopup(MouseEvent e) {/*_I*/_prof.prof.cnt[5637]++;/*I_*/
              /*_I*/{_prof.prof.cnt[5638]++;/*I_*/if (e.isPopupTrigger()) {
                 /*_I*/_prof.prof.cnt[5639]++;/*I_*/JPopupMenu popupMenu = new EditPopupMenu(textField);
                 /*_I*/{_prof.prof.cnt[5640]++;/*I_*/popupMenu.show(e.getComponent(),e.getX(),e.getY());/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5641]++;/*I_*/e.consume();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

        }

        // create the listener
        /*_I*/_prof.prof.cnt[5642]++;/*I_*/final TextFieldListener textFieldListener = new TextFieldListener();
        
        // Add listeners
        /*_I*/{_prof.prof.cnt[5643]++;/*I_*/this.textFieldListener = textFieldListener;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5644]++;/*I_*/textField.addFocusListener( textFieldListener );/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5645]++;/*I_*/textField.addKeyListener( textFieldListener );/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5646]++;/*I_*/textField.addMouseListener( textFieldListener );/*_I*/}/*I_*/

        // screen positionining ---------------------------------------------

        // SCROLL if necessary
        /*_I*/{_prof.prof.cnt[5647]++;/*I_*/getView().scrollNodeToVisible(node, xExtraWidth);/*_I*/}/*I_*/

        // NOTE: this must be calculated after scroll because the pane location changes
        /*_I*/_prof.prof.cnt[5648]++;/*I_*/Point frameScreenLocation = getFrame().getLayeredPane().getLocationOnScreen();
        /*_I*/_prof.prof.cnt[5649]++;/*I_*/Point nodeScreenLocation = node.getLocationOnScreen();

        /*_I*/_prof.prof.cnt[5650]++;/*I_*/int xLeft = (int)(nodeScreenLocation.getX() -  frameScreenLocation.getX() + xOffset);
        /*_I*/{_prof.prof.cnt[5651]++;/*I_*/if (xExtraWidth < 0) {
          /*_I*/{_prof.prof.cnt[5652]++;/*I_*/xLeft += xExtraWidth;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5653]++;/*I_*/textField.setLocation(xLeft, (int)(nodeScreenLocation.getY()
                                          - frameScreenLocation.getY() + yOffset));/*_I*/}/*I_*/
        
        /*_I*/{_prof.prof.cnt[5654]++;/*I_*/getFrame().getLayeredPane().add(textField);/*_I*/}/*I_*/ // 2000);
        /*_I*/{_prof.prof.cnt[5655]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/
 
        /*_I*/{_prof.prof.cnt[5656]++;/*I_*/SwingUtilities.invokeLater( new Runnable() { // PN 0.6.2
             public void run () {/*_I*/_prof.prof.cnt[5657]++;/*I_*/ /*_I*/{_prof.prof.cnt[5658]++;/*I_*/textField.requestFocus();/*_I*/}/*I_*/ }});/*_I*/}/*I_*/
    }

    public final int NEW_CHILD_WITHOUT_FOCUS = 1;  // old model of insertion
    public final int NEW_CHILD = 2;
    public final int NEW_SIBLING_BEHIND = 3;
    public final int NEW_SIBLING_BEFORE = 4;

    public void addNew(final NodeView target, final int newNodeMode, final KeyEvent e) {/*_I*/_prof.prof.cnt[5659]++;/*I_*/
       /*_I*/{_prof.prof.cnt[5660]++;/*I_*/closeEdit();/*_I*/}/*I_*/
       
       /*_I*/_prof.prof.cnt[5661]++;/*I_*/MindMapNode newNode = newNode();
       /*_I*/_prof.prof.cnt[5662]++;/*I_*/final MindMapNode targetNode = target.getModel();

       /*_I*/{_prof.prof.cnt[5663]++;/*I_*/switch (newNodeMode) {
         case NEW_SIBLING_BEFORE:
         case NEW_SIBLING_BEHIND:
           /*_I*/{_prof.prof.cnt[5664]++;/*I_*/if (targetNode.isRoot()) {
             /*_I*/{_prof.prof.cnt[5665]++;/*I_*/getController().errorMessage(
                 getText("new_node_as_sibling_not_possible_for_the_root"));/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[5666]++;/*I_*/setBlocked(false);/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[5667]++;/*I_*/return;/*_I*/}/*I_*/ 
           }/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5668]++;/*I_*/MindMapNode parent = targetNode.getParentNode();
           /*_I*/_prof.prof.cnt[5669]++;/*I_*/int childPosition = parent.getChildPosition(targetNode);
           /*_I*/{_prof.prof.cnt[5670]++;/*I_*/if (newNodeMode == NEW_SIBLING_BEHIND) {
              /*_I*/{_prof.prof.cnt[5671]++;/*I_*/childPosition++;/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5672]++;/*I_*/if(targetNode.isLeft()!= null) {
               /*_I*/{_prof.prof.cnt[5673]++;/*I_*/newNode.setLeft(targetNode.isLeft().getValue());/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5674]++;/*I_*/getModel().insertNodeInto(newNode, parent, childPosition);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5675]++;/*I_*/select(newNode.getViewer());/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5676]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
           /*_I*/{_prof.prof.cnt[5677]++;/*I_*/edit(newNode.getViewer(), target, e, true, false, false);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5678]++;/*I_*/break;/*_I*/}/*I_*/
  
         case NEW_CHILD:
         case NEW_CHILD_WITHOUT_FOCUS:
           /*_I*/_prof.prof.cnt[5679]++;/*I_*/final boolean parentFolded = targetNode.isFolded();
           /*_I*/{_prof.prof.cnt[5680]++;/*I_*/if (parentFolded) {
             /*_I*/{_prof.prof.cnt[5681]++;/*I_*/getModel().setFolded(targetNode,false);/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5682]++;/*I_*/int position = getFrame().getProperty("placenewbranches").equals("last") ?
              targetNode.getChildCount() : 0;
           // Here the NodeView is created for the node. }
           /*_I*/{_prof.prof.cnt[5683]++;/*I_*/getModel().insertNodeInto(newNode, targetNode, position);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5684]++;/*I_*/getFrame().repaint();/*_I*/}/*I_*/ //  getLayeredPane().repaint();
           /*_I*/{_prof.prof.cnt[5685]++;/*I_*/if (newNodeMode == NEW_CHILD) {
             /*_I*/{_prof.prof.cnt[5686]++;/*I_*/select(newNode.getViewer());/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5687]++;/*I_*/final NodeView editView = newNode.getViewer();
           /*_I*/{_prof.prof.cnt[5688]++;/*I_*/edit(editView, targetNode.getViewer(), e, true, parentFolded, false);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5689]++;/*I_*/break;/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

//     public void toggleFolded() {
//         MindMapNode node = getSelected();
//         // fold the node only if the node is not a leaf (PN 0.6.2)
//         if (node.hasChildren()
//             || node.isFolded()
//             || Tools.safeEquals(getFrame()
//                 .getProperty("enable_leaves_folding"),"true")) {
//           getModel().setFolded(node, !node.isFolded());
//         }
//         getView().selectAsTheOnlyOneSelected(node.getViewer());
//     }

    public void toggleFolded() {/*_I*/_prof.prof.cnt[5690]++;/*I_*/
        /* Retrieve the information whether or not all nodes have the same folding state. */
        /*_I*/_prof.prof.cnt[5691]++;/*I_*/Tools.BooleanHolder state = null; 
        /*_I*/_prof.prof.cnt[5692]++;/*I_*/boolean allNodeHaveSameFoldedStatus = true;
        /*_I*/{_prof.prof.cnt[5693]++;/*I_*/for (ListIterator it = getSelecteds().listIterator();it.hasNext();) {
            /*_I*/_prof.prof.cnt[5694]++;/*I_*/MindMapNode node = (MindMapNode)it.next();
            /*_I*/{_prof.prof.cnt[5695]++;/*I_*/if(state == null) {
                /*_I*/{_prof.prof.cnt[5696]++;/*I_*/state = new Tools.BooleanHolder();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5697]++;/*I_*/state.setValue(node.isFolded());/*_I*/}/*I_*/
            } else {
                /*_I*/{_prof.prof.cnt[5698]++;/*I_*/if(node.isFolded() != state.getValue()) {
                    /*_I*/{_prof.prof.cnt[5699]++;/*I_*/allNodeHaveSameFoldedStatus = false;/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5700]++;/*I_*/break;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /* if the folding state is ambiguous, the nodes are folded. */
        /*_I*/_prof.prof.cnt[5701]++;/*I_*/boolean fold = true;
        /*_I*/{_prof.prof.cnt[5702]++;/*I_*/if(allNodeHaveSameFoldedStatus && state != null) {
            /*_I*/{_prof.prof.cnt[5703]++;/*I_*/fold = !state.getValue();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5704]++;/*I_*/MindMapNode lastNode = null;
        /*_I*/{_prof.prof.cnt[5705]++;/*I_*/for (ListIterator it = getView().getSelectedsByDepth().listIterator();it.hasNext();) {
            /*_I*/_prof.prof.cnt[5706]++;/*I_*/MindMapNode node = ((NodeView)it.next()).getModel();
            // fold the node only if the node is not a leaf (PN 0.6.2)
            /*_I*/{_prof.prof.cnt[5707]++;/*I_*/if (node.hasChildren() || node.isFolded() || Tools.safeEquals(getFrame().getProperty("enable_leaves_folding"),"true"))   {
                /*_I*/{_prof.prof.cnt[5708]++;/*I_*/getModel().setFolded(node, fold);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5709]++;/*I_*/lastNode = node;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5710]++;/*I_*/if(lastNode != null)
            /*_I*/{_prof.prof.cnt[5711]++;/*I_*/getView().selectAsTheOnlyOneSelected(lastNode.getViewer());/*_I*/}/*I_*//*_I*/}/*I_*/
    }


    /**
     * If any children are folded, unfold all folded children.
     * Otherwise, fold all children.
     */
    protected void toggleChildrenFolded() {/*_I*/_prof.prof.cnt[5712]++;/*I_*/
        // have NodeAdapter; need NodeView
        /*_I*/_prof.prof.cnt[5713]++;/*I_*/MindMapNode parent = getSelected();
        /*_I*/_prof.prof.cnt[5714]++;/*I_*/ListIterator children_it = parent.getViewer().getChildrenViews().listIterator();
        /*_I*/_prof.prof.cnt[5715]++;/*I_*/boolean areAnyFolded = false;
        /*_I*/{_prof.prof.cnt[5716]++;/*I_*/while(children_it.hasNext() && !areAnyFolded) {
            /*_I*/_prof.prof.cnt[5717]++;/*I_*/NodeView child = (NodeView)children_it.next();
            /*_I*/{_prof.prof.cnt[5718]++;/*I_*/if(child.getModel().isFolded()) {
                /*_I*/{_prof.prof.cnt[5719]++;/*I_*/areAnyFolded = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        // fold the node only if the node is not a leaf (PN 0.6.2)
        /*_I*/_prof.prof.cnt[5720]++;/*I_*/boolean enableLeavesFolding = 
            Tools.safeEquals(getFrame().
               getProperty("enable_leaves_folding"),"true");

        /*_I*/{_prof.prof.cnt[5721]++;/*I_*/children_it = parent.getViewer().getChildrenViews().listIterator();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5722]++;/*I_*/while(children_it.hasNext()) {
            /*_I*/_prof.prof.cnt[5723]++;/*I_*/MindMapNode child = ((NodeView)children_it.next()).getModel();
            /*_I*/{_prof.prof.cnt[5724]++;/*I_*/if (child.hasChildren() 
                || enableLeavesFolding
                || child.isFolded()) {
              /*_I*/{_prof.prof.cnt[5725]++;/*I_*/getModel().setFolded(child, !areAnyFolded);/*_I*/}/*I_*/ // (PN 0.6.2)
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5726]++;/*I_*/getView().selectAsTheOnlyOneSelected(parent.getViewer());/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5727]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // focus fix
    }

    protected void setLinkByTextField() {/*_I*/_prof.prof.cnt[5728]++;/*I_*/
        // Requires J2SDK1.4.0!
        /*_I*/_prof.prof.cnt[5729]++;/*I_*/String inputValue = JOptionPane.showInputDialog
           (getText("edit_link_manually"), getModel().getLink(getSelected()));
        /*_I*/{_prof.prof.cnt[5730]++;/*I_*/if (inputValue != null) {
           /*_I*/{_prof.prof.cnt[5731]++;/*I_*/if (inputValue.equals("")) {
              /*_I*/{_prof.prof.cnt[5732]++;/*I_*/inputValue = null;/*_I*/}/*I_*/        // In case of no entry unset link
           }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5733]++;/*I_*/getModel().setLink(getSelected(),inputValue);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    protected void setLinkByFileChooser() {/*_I*/_prof.prof.cnt[5734]++;/*I_*/
		/*_I*/_prof.prof.cnt[5735]++;/*I_*/String relative = getLinkByFileChooser(getFileFilter());
		/*_I*/{_prof.prof.cnt[5736]++;/*I_*/if (relative != null) /*_I*/{_prof.prof.cnt[5737]++;/*I_*/getModel().setLink(getSelected(),relative);/*_I*/}/*I_*//*_I*/}/*I_*/
	}
	
	protected void setImageByFileChooser() {/*_I*/_prof.prof.cnt[5738]++;/*I_*/
		/*_I*/_prof.prof.cnt[5739]++;/*I_*/ExampleFileFilter filter = new ExampleFileFilter();
		/*_I*/{_prof.prof.cnt[5740]++;/*I_*/filter.addExtension("jpg");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5741]++;/*I_*/filter.addExtension("jpeg");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5742]++;/*I_*/filter.addExtension("png");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5743]++;/*I_*/filter.addExtension("gif");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5744]++;/*I_*/filter.setDescription("JPG, PNG and GIF Images");/*_I*/}/*I_*/

                // Are there any selected nodes with pictures?                
                /*_I*/_prof.prof.cnt[5745]++;/*I_*/boolean picturesAmongSelecteds = false;
                /*_I*/{_prof.prof.cnt[5746]++;/*I_*/for (ListIterator e = getSelecteds().listIterator();e.hasNext();) {
                   /*_I*/_prof.prof.cnt[5747]++;/*I_*/String link = ((MindMapNode)e.next()).getLink();
                   /*_I*/{_prof.prof.cnt[5748]++;/*I_*/if (link != null) {
                      /*_I*/{_prof.prof.cnt[5749]++;/*I_*/if (filter.accept(new File(link))) {
                         /*_I*/{_prof.prof.cnt[5750]++;/*I_*/picturesAmongSelecteds = true;/*_I*/}/*I_*/
                         /*_I*/{_prof.prof.cnt[5751]++;/*I_*/break;/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/
                   }/*_I*/}/*I_*/
                }/*_I*/}/*I_*/

                /*_I*/{_prof.prof.cnt[5752]++;/*I_*/try {
                   /*_I*/{_prof.prof.cnt[5753]++;/*I_*/if (picturesAmongSelecteds) {
                      /*_I*/{_prof.prof.cnt[5754]++;/*I_*/for (ListIterator e = getSelecteds().listIterator();e.hasNext();) {
                         /*_I*/_prof.prof.cnt[5755]++;/*I_*/MindMapNode node = (MindMapNode)e.next();
                         /*_I*/{_prof.prof.cnt[5756]++;/*I_*/if (node.getLink() != null) {
                            /*_I*/_prof.prof.cnt[5757]++;/*I_*/String possiblyRelative = node.getLink();
                            /*_I*/_prof.prof.cnt[5758]++;/*I_*/String relative = Tools.isAbsolutePath(possiblyRelative) ?
                               new File(possiblyRelative).toURL().toString() : possiblyRelative;
                            /*_I*/{_prof.prof.cnt[5759]++;/*I_*/if (relative != null) {
                               /*_I*/_prof.prof.cnt[5760]++;/*I_*/String strText = "<html><img src=\"" + relative + "\">"; 
                               /*_I*/{_prof.prof.cnt[5761]++;/*I_*/node.setLink(null);/*_I*/}/*I_*/
                               /*_I*/{_prof.prof.cnt[5762]++;/*I_*/getModel().changeNode(node,strText);/*_I*/}/*I_*/
                            }/*_I*/}/*I_*/
                         }/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/
                   }
                   else {
                      /*_I*/_prof.prof.cnt[5763]++;/*I_*/String relative = getLinkByFileChooser(filter);
                      /*_I*/{_prof.prof.cnt[5764]++;/*I_*/if (relative != null) {
                         /*_I*/_prof.prof.cnt[5765]++;/*I_*/String strText = "<html><img src=\"" + relative + "\">"; 
                         /*_I*/{_prof.prof.cnt[5766]++;/*I_*/getModel().changeNode((MindMapNode)getSelected(),strText);/*_I*/}/*I_*/
                      }/*_I*/}/*I_*/ 
                   }/*_I*/}/*I_*/
                }
                catch (MalformedURLException e) {/*_I*/{_prof.prof.cnt[5767]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	}
   
	protected String getLinkByFileChooser(FileFilter fileFilter) {/*_I*/_prof.prof.cnt[5768]++;/*I_*/
        /*_I*/_prof.prof.cnt[5769]++;/*I_*/URL link;
		/*_I*/_prof.prof.cnt[5770]++;/*I_*/String relative = null;
        /*_I*/_prof.prof.cnt[5771]++;/*I_*/File input;
        /*_I*/_prof.prof.cnt[5772]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
        /*_I*/_prof.prof.cnt[5773]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
        /*_I*/{_prof.prof.cnt[5774]++;/*I_*/if (getMap().getFile() == null) {
            /*_I*/{_prof.prof.cnt[5775]++;/*I_*/JOptionPane.showMessageDialog(getFrame().getContentPane(), getText("not_saved_for_link_error"), 
                                          "FreeMind", JOptionPane.WARNING_MESSAGE);/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5776]++;/*I_*/return null;/*_I*/}/*I_*/
            // In the previous version Freemind automatically displayed save
            // dialog. It happened very often, that user took this save
            // dialog to be an open link dialog; as a result, the new map
            // overwrote the linked map.

        }/*_I*/}/*I_*/
//         if ((getMap().getFile() != null) && (getMap().getFile().getParentFile() != null)) {
//             chooser = new JFileChooser(getMap().getFile().getParentFile());
//         } else {
//             chooser = new JFileChooser();
//         }

		/*_I*/{_prof.prof.cnt[5777]++;/*I_*/if (fileFilter != null) {
           // Set filters, make sure AcceptAll filter comes first
		   /*_I*/{_prof.prof.cnt[5778]++;/*I_*/chooser.setFileFilter(fileFilter);/*_I*/}/*I_*/
		} else {
			/*_I*/{_prof.prof.cnt[5779]++;/*I_*/chooser.setFileFilter(chooser.getAcceptAllFileFilter());/*_I*/}/*I_*/
		}/*_I*/}/*I_*/
 
        /*_I*/_prof.prof.cnt[5780]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
        /*_I*/{_prof.prof.cnt[5781]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
            /*_I*/{_prof.prof.cnt[5782]++;/*I_*/input = chooser.getSelectedFile();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5783]++;/*I_*/try {
                /*_I*/{_prof.prof.cnt[5784]++;/*I_*/link = input.toURL();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5785]++;/*I_*/relative = link.toString();/*_I*/}/*I_*/
            } catch (MalformedURLException ex) {
                /*_I*/{_prof.prof.cnt[5786]++;/*I_*/getController().errorMessage(getText("url_error"));/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5787]++;/*I_*/return null;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5788]++;/*I_*/if (getFrame().getProperty("links").equals("relative")) {
                //Create relative URL
                /*_I*/{_prof.prof.cnt[5789]++;/*I_*/try {
                    /*_I*/{_prof.prof.cnt[5790]++;/*I_*/relative = Tools.toRelativeURL(getMap().getFile().toURL(), link);/*_I*/}/*I_*/
                } catch (MalformedURLException ex) {
                    /*_I*/{_prof.prof.cnt[5791]++;/*I_*/getController().errorMessage(getText("url_error"));/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5792]++;/*I_*/return null;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[5793]++;/*I_*/return relative;/*_I*/}/*I_*/
    }

    public void loadURL(String relative) {/*_I*/_prof.prof.cnt[5794]++;/*I_*/
        /*_I*/_prof.prof.cnt[5795]++;/*I_*/URL absolute = null;
        /*_I*/{_prof.prof.cnt[5796]++;/*I_*/if (getMap().getFile() == null) {
            /*_I*/{_prof.prof.cnt[5797]++;/*I_*/getFrame().out("You must save the current map first!");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5798]++;/*I_*/save();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5799]++;/*I_*/try {
           /*_I*/{_prof.prof.cnt[5800]++;/*I_*/if (Tools.isAbsolutePath(relative)) {
              // Protocol can be identified by rexep pattern "[a-zA-Z]://.*".
              // This should distinguish a protocol path from a file path on most platforms.
              // 1)  UNIX / Linux - obviously
              // 2)  Windows - relative path does not contain :, in absolute path is : followed by \.
              // 3)  Mac - cannot remember

              // If relative is an absolute path, then it cannot be a protocol.
              // At least on Unix and Windows. But this is not true for Mac!!

              // Here is hidden an assumption that the existence of protocol implies !Tools.isAbsolutePath(relative).
              // The code should probably be rewritten to convey more logical meaning, on the other hand
              // it works on Windows and Linux.

              //absolute = new URL("file://"+relative); }
              /*_I*/{_prof.prof.cnt[5801]++;/*I_*/absolute = new File(relative).toURL();/*_I*/}/*I_*/ }
            else {
              /*_I*/{_prof.prof.cnt[5802]++;/*I_*/absolute = new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/
              // Remark: getMap().getFile().toURL() returns URLs like file:/C:/...
              // It seems, that it does not cause any problems.
            }/*_I*/}/*I_*/

           /*_I*/_prof.prof.cnt[5803]++;/*I_*/String extension = Tools.getExtension(absolute.toString());
           /*_I*/{_prof.prof.cnt[5804]++;/*I_*/if ((extension != null) && extension.equals("mm")) {   // ---- Open Mind Map
              /*_I*/_prof.prof.cnt[5805]++;/*I_*/String fileName = absolute.getFile();
              /*_I*/_prof.prof.cnt[5806]++;/*I_*/File file = new File(fileName);
              /*_I*/{_prof.prof.cnt[5807]++;/*I_*/if(!getController().getMapModuleManager().tryToChangeToMapModule(file.getName())) {
                 //this can lead to confusion if the user handles multiple maps with the same name.
                 /*_I*/{_prof.prof.cnt[5808]++;/*I_*/getFrame().setWaitingCursor(true);/*_I*/}/*I_*/
                 /*_I*/{_prof.prof.cnt[5809]++;/*I_*/load(file);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
           else {                                                 // ---- Open URL in browser
               // fc, 14.12.2003: The following code seems not very good. Imagine file names with spaces. Then they occur as %20, now the OS does not find the file, 
               // etc. If this is necessary, this should be done in the openDocument command.
//               if (absolute.getProtocol().equals("file")) {                 
//                  File file = new File (Tools.urlGetFile(absolute));
//                  // If file does not exist, try http protocol (but only if it is reasonable)
//                  if (!file.exists()) {
//                     if (relative.matches("^[-\\.a-z0-9]*/?$")) {
//                        absolute = new URL("http://"+relative); }
//                     else {
//                        // This cannot be a base, to which http:// may be added.
//                        getController().errorMessage("File \""+file+"\" does not exist.");
//                        return; }}}
              /*_I*/{_prof.prof.cnt[5810]++;/*I_*/getFrame().openDocument(absolute);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
        catch (MalformedURLException ex) {
            /*_I*/{_prof.prof.cnt[5811]++;/*I_*/getController().errorMessage(getText("url_error")+"\n"+ex);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5812]++;/*I_*/return;/*_I*/}/*I_*/ }
        catch (FileNotFoundException e) {
            /*_I*/_prof.prof.cnt[5813]++;/*I_*/int returnVal = JOptionPane.showConfirmDialog
               (getView(),
                getText("repair_link_question"),
                getText("repair_link"),
                JOptionPane.YES_NO_OPTION);
            /*_I*/{_prof.prof.cnt[5814]++;/*I_*/if (returnVal==JOptionPane.YES_OPTION) {
               /*_I*/{_prof.prof.cnt[5815]++;/*I_*/setLinkByTextField();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
        catch (Exception e) { /*_I*/{_prof.prof.cnt[5816]++;/*I_*/e.printStackTrace();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5817]++;/*I_*/getFrame().setWaitingCursor(false);/*_I*/}/*I_*/
    }

    public void loadURL() {/*_I*/_prof.prof.cnt[5818]++;/*I_*/
        /*_I*/_prof.prof.cnt[5819]++;/*I_*/String link = getSelected().getLink();
        /*_I*/{_prof.prof.cnt[5820]++;/*I_*/if (link != null) {
            /*_I*/{_prof.prof.cnt[5821]++;/*I_*/loadURL(link);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    //
    // Convenience methods
    //

    protected Mode getMode() {/*_I*/_prof.prof.cnt[5822]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5823]++;/*I_*/return mode;/*_I*/}/*I_*/
    }

    protected void setMode(Mode mode) {/*_I*/_prof.prof.cnt[5824]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5825]++;/*I_*/this.mode = mode;/*_I*/}/*I_*/
    }

    protected MapModule getMapModule() {/*_I*/_prof.prof.cnt[5826]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5827]++;/*I_*/return getController().getMapModuleManager().getMapModule();/*_I*/}/*I_*/
    }

    public MapAdapter getMap() {/*_I*/_prof.prof.cnt[5828]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5829]++;/*I_*/if (getMapModule() != null) {
            /*_I*/{_prof.prof.cnt[5830]++;/*I_*/return (MapAdapter)getMapModule().getModel();/*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[5831]++;/*I_*/return null;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public URL getResource (String name) {/*_I*/_prof.prof.cnt[5832]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5833]++;/*I_*/return getFrame().getResource(name);/*_I*/}/*I_*/
    }

    public Controller getController() {/*_I*/_prof.prof.cnt[5834]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5835]++;/*I_*/return getMode().getController();/*_I*/}/*I_*/
    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[5836]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5837]++;/*I_*/return getController().getFrame();/*_I*/}/*I_*/
    }

    private MapAdapter getModel() {/*_I*/_prof.prof.cnt[5838]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5839]++;/*I_*/return (MapAdapter)getController().getModel();/*_I*/}/*I_*/
    }

    public MapView getView() {/*_I*/_prof.prof.cnt[5840]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5841]++;/*I_*/return getController().getView();/*_I*/}/*I_*/
    }

    protected void updateMapModuleName() {/*_I*/_prof.prof.cnt[5842]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5843]++;/*I_*/getController().getMapModuleManager().updateMapModuleName();/*_I*/}/*I_*/
    }

    private NodeAdapter getSelected() {/*_I*/_prof.prof.cnt[5844]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5845]++;/*I_*/return (NodeAdapter)getView().getSelected().getModel();/*_I*/}/*I_*/
    }

    public boolean extendSelection(MouseEvent e) {/*_I*/_prof.prof.cnt[5846]++;/*I_*/
        /*_I*/_prof.prof.cnt[5847]++;/*I_*/NodeView newlySelectedNodeView = (NodeView)e.getSource();
        //MindMapNode newlySelectedNode = newlySelectedNodeView.getModel();
        /*_I*/_prof.prof.cnt[5848]++;/*I_*/boolean extend = e.isControlDown(); 
        /*_I*/_prof.prof.cnt[5849]++;/*I_*/boolean range = e.isShiftDown(); 
        /*_I*/_prof.prof.cnt[5850]++;/*I_*/boolean branch = e.isAltGraphDown() || e.isAltDown(); /* windows alt, linux altgraph .... */ 
        /*_I*/_prof.prof.cnt[5851]++;/*I_*/boolean retValue = false;

        /*_I*/{_prof.prof.cnt[5852]++;/*I_*/if (extend || range || branch || !getView().isSelected(newlySelectedNodeView)) {
            /*_I*/{_prof.prof.cnt[5853]++;/*I_*/if (!range) {
                /*_I*/{_prof.prof.cnt[5854]++;/*I_*/if (extend)
                    /*_I*/{_prof.prof.cnt[5855]++;/*I_*/getView().toggleSelected(newlySelectedNodeView);/*_I*/}/*I_*/
                else
                    /*_I*/{_prof.prof.cnt[5856]++;/*I_*/select(newlySelectedNodeView);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5857]++;/*I_*/retValue = true;/*_I*/}/*I_*/
            }
            else {
                /*_I*/{_prof.prof.cnt[5858]++;/*I_*/retValue = getView().selectContinuous(newlySelectedNodeView);/*_I*/}/*I_*/ 
//                 /* fc, 25.1.2004: replace getView by controller methods.*/
//                 if (newlySelectedNodeView != getView().getSelected() &&
//                     newlySelectedNodeView.isSiblingOf(getView().getSelected())) {
//                     getView().selectContinuous(newlySelectedNodeView); 
//                     retValue = true;
//                 } else {
//                     /* if shift was down, but no range can be selected, then the new node is simply selected: */
//                     if(!getView().isSelected(newlySelectedNodeView)) {
//                         getView().toggleSelected(newlySelectedNodeView);
//                         retValue = true;
//                     }
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5859]++;/*I_*/if(branch) {
                /*_I*/{_prof.prof.cnt[5860]++;/*I_*/getView().selectBranch(newlySelectedNodeView, extend);/*_I*/}/*I_*/ 
                /*_I*/{_prof.prof.cnt[5861]++;/*I_*/retValue = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/    
        }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[5862]++;/*I_*/if(retValue) {
            /*_I*/{_prof.prof.cnt[5863]++;/*I_*/e.consume();/*_I*/}/*I_*/
        
            // Display link in status line
            /*_I*/_prof.prof.cnt[5864]++;/*I_*/String link = newlySelectedNodeView.getModel().getLink();
            /*_I*/{_prof.prof.cnt[5865]++;/*I_*/link = (link != null ? link : " ");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5866]++;/*I_*/getController().getFrame().out(link);/*_I*/}/*I_*/ 
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5867]++;/*I_*/return retValue;/*_I*/}/*I_*/
    }

    private void select( NodeView node) {/*_I*/_prof.prof.cnt[5868]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5869]++;/*I_*/getView().selectAsTheOnlyOneSelected(node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5870]++;/*I_*/getView().setSiblingMaxLevel(node.getModel().getNodeLevel());/*_I*/}/*I_*/ // this level is default
    }

    ////////////
    //  Actions
    ///////////

    protected class NewMapAction extends AbstractAction {
        ControllerAdapter c;
        public NewMapAction(ControllerAdapter controller) {
            super(getText("new"), new ImageIcon(getResource("images/New24.gif")));/*_I*/_prof.prof.cnt[5871]++;/*I_*//*_I*/_prof.prof.cnt[5872]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5873]++;/*I_*/c = controller;/*_I*/}/*I_*/
            //Workaround to get the images loaded in jar file.
            //they have to be added to jar manually with full path from root
            //I really don't like this, but it's a bug of java
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5874]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5875]++;/*I_*/c.newMap();/*_I*/}/*I_*/
        }
    }

    protected class OpenAction extends AbstractAction {
        ControllerAdapter mc;
        public OpenAction(ControllerAdapter modeController) {
            super(getText("open"), new ImageIcon(getResource("images/Open24.gif")));/*_I*/_prof.prof.cnt[5876]++;/*I_*//*_I*/_prof.prof.cnt[5877]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5878]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5879]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5880]++;/*I_*/mc.open();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5881]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

    protected class SaveAction extends AbstractAction {
        ControllerAdapter mc;
        public SaveAction(ControllerAdapter modeController) {
            super(getText("save"), new ImageIcon(getResource("images/Save24.gif")));/*_I*/_prof.prof.cnt[5882]++;/*I_*//*_I*/_prof.prof.cnt[5883]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5884]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5885]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5886]++;/*I_*/mc.save();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5887]++;/*I_*/getFrame().out(getText("saved"));/*_I*/}/*I_*/ // perhaps... (PN)
			/*_I*/{_prof.prof.cnt[5888]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

    protected class SaveAsAction extends AbstractAction {
        ControllerAdapter mc;
        public SaveAsAction(ControllerAdapter modeController) {
            super(getText("save_as"), new ImageIcon(getResource("images/SaveAs24.gif")));/*_I*/_prof.prof.cnt[5889]++;/*I_*//*_I*/_prof.prof.cnt[5890]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5891]++;/*I_*/mc = modeController;/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5892]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5893]++;/*I_*/mc.saveAs();/*_I*/}/*I_*/
			/*_I*/{_prof.prof.cnt[5894]++;/*I_*/getController().setTitle();/*_I*/}/*I_*/ // Possible update of read-only
        }
    }

    protected class FindAction extends AbstractAction {
        public FindAction() {
           super(getText("find"));/*_I*/_prof.prof.cnt[5895]++;/*I_*//*_I*/_prof.prof.cnt[5896]++;/*I_*/ }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5897]++;/*I_*/
           /*_I*/_prof.prof.cnt[5898]++;/*I_*/String what = JOptionPane.showInputDialog(getView().getSelected(),
                                                     getText("find_what"));
           /*_I*/{_prof.prof.cnt[5899]++;/*I_*/if (what == null || what.equals("")) {
              /*_I*/{_prof.prof.cnt[5900]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5901]++;/*I_*/boolean found = getView().getModel().find
              (getView().getSelected().getModel(), what, /*caseSensitive=*/ false);
           /*_I*/{_prof.prof.cnt[5902]++;/*I_*/getView().repaint();/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5903]++;/*I_*/if (!found) {
              /*_I*/{_prof.prof.cnt[5904]++;/*I_*/getController().informationMessage
                 (getText("no_found_from").replaceAll("\\$1",what).
                  replaceAll("\\$2", getView().getModel().getFindFromText()),
                  getView().getSelected());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class FindNextAction extends AbstractAction {
        public FindNextAction() {
           super(getText("find_next"));/*_I*/_prof.prof.cnt[5905]++;/*I_*//*_I*/_prof.prof.cnt[5906]++;/*I_*/ }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5907]++;/*I_*/
           /*_I*/_prof.prof.cnt[5908]++;/*I_*/String what = getView().getModel().getFindWhat();
           /*_I*/{_prof.prof.cnt[5909]++;/*I_*/if (what == null) {
              /*_I*/{_prof.prof.cnt[5910]++;/*I_*/getController().informationMessage(getText("no_previous_find"), getView().getSelected());/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5911]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[5912]++;/*I_*/boolean found = getView().getModel().findNext();
           /*_I*/{_prof.prof.cnt[5913]++;/*I_*/getView().repaint();/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[5914]++;/*I_*/if (!found) {
              /*_I*/{_prof.prof.cnt[5915]++;/*I_*/getController().informationMessage
                 (getText("no_more_found_from").replaceAll("\\$1",what).
                  replaceAll("\\$2", getView().getModel().getFindFromText()),
                  getView().getSelected());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class GotoLinkNodeAction extends AbstractAction {
        MindMapNode source;
        public GotoLinkNodeAction(String text, MindMapNode source) {
            super("", new ImageIcon(getResource("images/Link.png")));/*_I*/_prof.prof.cnt[5916]++;/*I_*//*_I*/_prof.prof.cnt[5917]++;/*I_*/
            // only display a reasonable part of the string. the rest is available via the short description (tooltip).
            /*_I*/_prof.prof.cnt[5918]++;/*I_*/String adaptedText = new String(text);
            /*_I*/{_prof.prof.cnt[5919]++;/*I_*/adaptedText = adaptedText.replaceAll("<html>", "");/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5920]++;/*I_*/if(adaptedText.length() > 40)
                /*_I*/{_prof.prof.cnt[5921]++;/*I_*/adaptedText = adaptedText.substring(0,40) + " ...";/*_I*/}/*I_*//*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5922]++;/*I_*/putValue(Action.NAME, getText("follow_link") + adaptedText );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5923]++;/*I_*/putValue(Action.SHORT_DESCRIPTION, text);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5924]++;/*I_*/this.source = source;/*_I*/}/*I_*/
        }

        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5925]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5926]++;/*I_*/getMap().displayNode(source, null);/*_I*/}/*I_*/
        }
    }


    //
    // Node editing
    //

    protected class EditAction extends AbstractAction {
        public EditAction() {
            super(getText("edit"));/*_I*/_prof.prof.cnt[5927]++;/*I_*//*_I*/_prof.prof.cnt[5928]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5929]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5930]++;/*I_*/edit(null, false, false);/*_I*/}/*I_*/
        }
    }

    protected class EditLongAction extends AbstractAction {
        public EditLongAction() {
            super(getText("edit_long_node"));/*_I*/_prof.prof.cnt[5931]++;/*I_*//*_I*/_prof.prof.cnt[5932]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5933]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5934]++;/*I_*/edit(null, false, true);/*_I*/}/*I_*/
        }
    }

    // old model of inserting node
    protected class NewChildWithoutFocusAction extends AbstractAction {
        public NewChildWithoutFocusAction() {
            super(getText("new_node"));/*_I*/_prof.prof.cnt[5935]++;/*I_*//*_I*/_prof.prof.cnt[5936]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5937]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5938]++;/*I_*/addNew(getView().getSelected(), NEW_CHILD_WITHOUT_FOCUS, null);/*_I*/}/*I_*/
        }
    }

    // new model of inserting node
    protected class NewSiblingAction extends AbstractAction {
        public NewSiblingAction() {
            super(getText("new_sibling_behind"));/*_I*/_prof.prof.cnt[5939]++;/*I_*//*_I*/_prof.prof.cnt[5940]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5941]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5942]++;/*I_*/addNew(getView().getSelected(), NEW_SIBLING_BEHIND, null);/*_I*/}/*I_*/
        }
    }

    protected class NewChildAction extends AbstractAction {
        public NewChildAction() {
            super(getText("new_child"));/*_I*/_prof.prof.cnt[5943]++;/*I_*//*_I*/_prof.prof.cnt[5944]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5945]++;/*I_*/
           /*_I*/{_prof.prof.cnt[5946]++;/*I_*/addNew(getView().getSelected(), NEW_CHILD, null);/*_I*/}/*I_*/
        }
    }

    protected class NewPreviousSiblingAction extends AbstractAction {
        public NewPreviousSiblingAction() {
            super(getText("new_sibling_before"));/*_I*/_prof.prof.cnt[5947]++;/*I_*//*_I*/_prof.prof.cnt[5948]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5949]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5950]++;/*I_*/addNew(getView().getSelected(), NEW_SIBLING_BEFORE, null);/*_I*/}/*I_*/
        }
    }


    protected class RemoveAction extends AbstractAction {
        public RemoveAction() {
            super(getText("remove_node"));/*_I*/_prof.prof.cnt[5951]++;/*I_*//*_I*/_prof.prof.cnt[5952]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5953]++;/*I_*/
           /*_I*/{_prof.prof.cnt[5954]++;/*I_*/if (getMapModule() != null) {
              /*_I*/{_prof.prof.cnt[5955]++;/*I_*/getView().getModel().cut();/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[5956]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class NodeUpAction extends AbstractAction {
        public NodeUpAction() {
            super(getText("node_up"));/*_I*/_prof.prof.cnt[5957]++;/*I_*//*_I*/_prof.prof.cnt[5958]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5959]++;/*I_*/
            /*_I*/_prof.prof.cnt[5960]++;/*I_*/MindMapNode selected = getView().getSelected().getModel();
            /*_I*/{_prof.prof.cnt[5961]++;/*I_*/if(!selected.isRoot()) {
                /*_I*/_prof.prof.cnt[5962]++;/*I_*/MindMapNode parent = selected.getParentNode();
                /*_I*/_prof.prof.cnt[5963]++;/*I_*/int index = getModel().getIndexOfChild(parent, selected);
                /*_I*/_prof.prof.cnt[5964]++;/*I_*/int newIndex = getModel().moveNodeTo(selected,parent,index, -1);
                /*_I*/{_prof.prof.cnt[5965]++;/*I_*/getModel().removeNodeFromParent(selected);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5966]++;/*I_*/getModel().insertNodeInto(selected,parent,newIndex);/*_I*/}/*I_*/
//                 int maxindex = parent.getChildCount(); // (PN)
//                 if(index - 1 <0) {
//                     getModel().insertNodeInto(selected,parent,maxindex, -1);
//                 } else {
//                     getModel().insertNodeInto(selected,parent,index - 1, -1);
//                 }
                /*_I*/{_prof.prof.cnt[5967]++;/*I_*/getModel().nodeStructureChanged(parent);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5968]++;/*I_*/getView().selectAsTheOnlyOneSelected(selected.getViewer());/*_I*/}/*I_*/

                /*_I*/{_prof.prof.cnt[5969]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // focus fix
            }/*_I*/}/*I_*/
        }
    }

    protected class NodeDownAction extends AbstractAction {
        public NodeDownAction() {
            super(getText("node_down"));/*_I*/_prof.prof.cnt[5970]++;/*I_*//*_I*/_prof.prof.cnt[5971]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5972]++;/*I_*/
            /*_I*/_prof.prof.cnt[5973]++;/*I_*/MindMapNode selected = getView().getSelected().getModel();
            /*_I*/{_prof.prof.cnt[5974]++;/*I_*/if(!selected.isRoot()) {
                /*_I*/_prof.prof.cnt[5975]++;/*I_*/MindMapNode parent = selected.getParentNode();
                /*_I*/_prof.prof.cnt[5976]++;/*I_*/int index = getModel().getIndexOfChild(parent, selected);
                /*_I*/_prof.prof.cnt[5977]++;/*I_*/int newIndex = getModel().moveNodeTo(selected,parent,index, 1);
                /*_I*/{_prof.prof.cnt[5978]++;/*I_*/getModel().removeNodeFromParent(selected);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5979]++;/*I_*/getModel().insertNodeInto(selected,parent,newIndex);/*_I*/}/*I_*/
//                 int maxindex = parent.getChildCount(); // (PN)
//                 if(index + 1 > maxindex) {
//                     getModel().insertNodeInto(selected,parent,0, 1);
//                 } else {
//                     getModel().insertNodeInto(selected,parent,index + 1, 1);
//                 }
                /*_I*/{_prof.prof.cnt[5980]++;/*I_*/getModel().nodeStructureChanged(parent);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5981]++;/*I_*/getView().selectAsTheOnlyOneSelected(selected.getViewer());/*_I*/}/*I_*/
                
                /*_I*/{_prof.prof.cnt[5982]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ // focus fix
            }/*_I*/}/*I_*/
        }
    }

    protected class ToggleFoldedAction extends AbstractAction {
        public ToggleFoldedAction() {
            super(getText("toggle_folded"));/*_I*/_prof.prof.cnt[5983]++;/*I_*//*_I*/_prof.prof.cnt[5984]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5985]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5986]++;/*I_*/toggleFolded();/*_I*/}/*I_*/
        }
    }

    protected class ToggleChildrenFoldedAction extends AbstractAction {
        public ToggleChildrenFoldedAction() {
            super(getText("toggle_children_folded"));/*_I*/_prof.prof.cnt[5987]++;/*I_*//*_I*/_prof.prof.cnt[5988]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5989]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5990]++;/*I_*/toggleChildrenFolded();/*_I*/}/*I_*/
        }
    }

    protected class SetLinkByFileChooserAction extends AbstractAction {
        public SetLinkByFileChooserAction() {
            super(getText("set_link_by_filechooser"));/*_I*/_prof.prof.cnt[5991]++;/*I_*//*_I*/_prof.prof.cnt[5992]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5993]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5994]++;/*I_*/setLinkByFileChooser();/*_I*/}/*I_*/
        }
    }

	protected class SetImageByFileChooserAction extends AbstractAction {
		public SetImageByFileChooserAction() {
			super(getText("set_image_by_filechooser"));/*_I*/_prof.prof.cnt[5995]++;/*I_*//*_I*/_prof.prof.cnt[5996]++;/*I_*/
		}
		public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[5997]++;/*I_*/
                       /*_I*/{_prof.prof.cnt[5998]++;/*I_*/setImageByFileChooser();/*_I*/}/*I_*/
                        /*_I*/{_prof.prof.cnt[5999]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/
		}
	}

    protected class SetLinkByTextFieldAction extends AbstractAction {
        public SetLinkByTextFieldAction() {
            super(getText("set_link_by_textfield"));/*_I*/_prof.prof.cnt[6000]++;/*I_*//*_I*/_prof.prof.cnt[6001]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6002]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6003]++;/*I_*/setLinkByTextField();/*_I*/}/*I_*/
        }
    }


    protected class FollowLinkAction extends AbstractAction {
        public FollowLinkAction() {
            super(getText("follow_link"));/*_I*/_prof.prof.cnt[6004]++;/*I_*//*_I*/_prof.prof.cnt[6005]++;/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6006]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6007]++;/*I_*/loadURL();/*_I*/}/*I_*/
        }
    }

    protected class CopyAction extends AbstractAction {
        public CopyAction(Object controller) {
            super(getText("copy"), new ImageIcon(getResource("images/Copy24.gif")));/*_I*/_prof.prof.cnt[6008]++;/*I_*//*_I*/_prof.prof.cnt[6009]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6010]++;/*I_*/setEnabled(false);/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6011]++;/*I_*/
           /*_I*/{_prof.prof.cnt[6012]++;/*I_*/if(getMapModule() != null) {
              /*_I*/_prof.prof.cnt[6013]++;/*I_*/Transferable copy = getView().getModel().copy();
              /*_I*/{_prof.prof.cnt[6014]++;/*I_*/if (copy != null) {
                 /*_I*/{_prof.prof.cnt[6015]++;/*I_*/clipboard.setContents(copy,null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/}}

    protected class CopySingleAction extends AbstractAction {
        public CopySingleAction(Object controller) {
           super(getText("copy_single"));/*_I*/_prof.prof.cnt[6016]++;/*I_*//*_I*/_prof.prof.cnt[6017]++;/*I_*/
           /*_I*/{_prof.prof.cnt[6018]++;/*I_*/setEnabled(false);/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6019]++;/*I_*/
           /*_I*/{_prof.prof.cnt[6020]++;/*I_*/if(getMapModule() != null) {
              /*_I*/_prof.prof.cnt[6021]++;/*I_*/Transferable copy = getView().getModel().copySingle();
              /*_I*/{_prof.prof.cnt[6022]++;/*I_*/if (copy != null) {
                 /*_I*/{_prof.prof.cnt[6023]++;/*I_*/clipboard.setContents(copy,null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/}}

    protected class CutAction extends AbstractAction {
        public CutAction(Object controller) {
            super(getText("cut"), new ImageIcon(getResource("images/Cut24.gif")));/*_I*/_prof.prof.cnt[6024]++;/*I_*//*_I*/_prof.prof.cnt[6025]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6026]++;/*I_*/setEnabled(false);/*_I*/}/*I_*/
        }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6027]++;/*I_*/
           /*_I*/{_prof.prof.cnt[6028]++;/*I_*/if (getMapModule() != null) {
              /*_I*/_prof.prof.cnt[6029]++;/*I_*/Transferable copy = getView().getModel().cut();
              /*_I*/{_prof.prof.cnt[6030]++;/*I_*/if (copy != null) {
                 /*_I*/{_prof.prof.cnt[6031]++;/*I_*/clipboard.setContents(copy,null);/*_I*/}/*I_*/ 
                 /*_I*/{_prof.prof.cnt[6032]++;/*I_*/getController().obtainFocusForSelected();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/}}

    protected class PasteAction extends AbstractAction {
        public PasteAction(Object controller) {
            super(getText("paste"),new ImageIcon(getResource("images/Paste24.gif")));/*_I*/_prof.prof.cnt[6033]++;/*I_*//*_I*/_prof.prof.cnt[6034]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6035]++;/*I_*/setEnabled(false);/*_I*/}/*I_*/ }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6036]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6037]++;/*I_*/if(clipboard != null) {
               /*_I*/{_prof.prof.cnt[6038]++;/*I_*/getModel().paste(clipboard.getContents(this), getView().getSelected().getModel());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class FileOpener implements DropTargetListener {
        private boolean isDragAcceptable(DropTargetDragEvent event) {/*_I*/_prof.prof.cnt[6039]++;/*I_*/
            // check if there is at least one File Type in the list
            /*_I*/_prof.prof.cnt[6040]++;/*I_*/DataFlavor[] flavors = event.getCurrentDataFlavors();
            /*_I*/{_prof.prof.cnt[6041]++;/*I_*/for (int i = 0; i < flavors.length; i++) {
                /*_I*/{_prof.prof.cnt[6042]++;/*I_*/if (flavors[i].isFlavorJavaFileListType()) {
                    //              event.acceptDrag(DnDConstants.ACTION_COPY);
                    /*_I*/{_prof.prof.cnt[6043]++;/*I_*/return true;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            //      event.rejectDrag();
            /*_I*/{_prof.prof.cnt[6044]++;/*I_*/return false;/*_I*/}/*I_*/
        }

        private boolean isDropAcceptable(DropTargetDropEvent event) {/*_I*/_prof.prof.cnt[6045]++;/*I_*/
            // check if there is at least one File Type in the list
            /*_I*/_prof.prof.cnt[6046]++;/*I_*/DataFlavor[] flavors = event.getCurrentDataFlavors();
            /*_I*/{_prof.prof.cnt[6047]++;/*I_*/for (int i = 0; i < flavors.length; i++) {
                /*_I*/{_prof.prof.cnt[6048]++;/*I_*/if (flavors[i].isFlavorJavaFileListType()) {
                    /*_I*/{_prof.prof.cnt[6049]++;/*I_*/return true;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6050]++;/*I_*/return false;/*_I*/}/*I_*/
        }

        public void drop (DropTargetDropEvent dtde) {/*_I*/_prof.prof.cnt[6051]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6052]++;/*I_*/if(!isDropAcceptable(dtde)) {
                /*_I*/{_prof.prof.cnt[6053]++;/*I_*/dtde.rejectDrop();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6054]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6055]++;/*I_*/dtde.acceptDrop(DnDConstants.ACTION_COPY);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6056]++;/*I_*/try {
                /*_I*/_prof.prof.cnt[6057]++;/*I_*/Object data =
                    dtde.getTransferable().getTransferData(DataFlavor.javaFileListFlavor);
                /*_I*/{_prof.prof.cnt[6058]++;/*I_*/if (data == null) {
                    // Shouldn't happen because dragEnter() rejects drags w/out at least
                    // one javaFileListFlavor. But just in case it does ...
                    /*_I*/{_prof.prof.cnt[6059]++;/*I_*/dtde.dropComplete(false);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6060]++;/*I_*/return;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[6061]++;/*I_*/Iterator iterator = ((List)data).iterator();
                /*_I*/{_prof.prof.cnt[6062]++;/*I_*/while (iterator.hasNext()) {
                    /*_I*/_prof.prof.cnt[6063]++;/*I_*/File file = (File)iterator.next();
                    /*_I*/{_prof.prof.cnt[6064]++;/*I_*/load(file);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }
            catch (Exception e) {
                /*_I*/{_prof.prof.cnt[6065]++;/*I_*/JOptionPane.showMessageDialog(getView(),
                                              "Couldn't open dropped file(s). Reason: " + e.getMessage()
                                              //getText("file_not_found")
                                              );/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6066]++;/*I_*/dtde.dropComplete(false);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6067]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6068]++;/*I_*/dtde.dropComplete(true);/*_I*/}/*I_*/
        }

        public void dragEnter (DropTargetDragEvent dtde) {/*_I*/_prof.prof.cnt[6069]++;/*I_*/
            /*_I*/{_prof.prof.cnt[6070]++;/*I_*/if(!isDragAcceptable(dtde)) {
                /*_I*/{_prof.prof.cnt[6071]++;/*I_*/dtde.rejectDrag();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6072]++;/*I_*/return;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }

        public void dragOver (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6073]++;/*I_*/}
        public void dragExit (DropTargetEvent e) {/*_I*/_prof.prof.cnt[6074]++;/*I_*/}
        public void dragScroll (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6075]++;/*I_*/}
        public void dropActionChanged (DropTargetDragEvent e) {/*_I*/_prof.prof.cnt[6076]++;/*I_*/}
    }

   protected class EditCopyAction extends AbstractAction {
       private JTextComponent textComponent;
       public EditCopyAction(JTextComponent textComponent) {
          super(getText("copy"));/*_I*/_prof.prof.cnt[6077]++;/*I_*//*_I*/_prof.prof.cnt[6078]++;/*I_*/ 
          /*_I*/{_prof.prof.cnt[6079]++;/*I_*/this.textComponent = textComponent;/*_I*/}/*I_*/ }
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[6080]++;/*I_*/
           /*_I*/_prof.prof.cnt[6081]++;/*I_*/String selection = textComponent.getSelectedText();
           /*_I*/{_prof.prof.cnt[6082]++;/*I_*/if (selection != null) {
              /*_I*/{_prof.prof.cnt[6083]++;/*I_*/clipboard.setContents(new StringSelection(selection),null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

   private class EditPopupMenu extends JPopupMenu {
      //private JTextComponent textComponent;

    public EditPopupMenu(JTextComponent textComponent) {/*_I*/_prof.prof.cnt[6084]++;/*I_*/
       //this.textComponent = textComponent;        
       	/*_I*/{_prof.prof.cnt[6085]++;/*I_*/this.add(new EditCopyAction(textComponent));/*_I*/}/*I_*/
    }
   }




}
