/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: LineAdapter.java,v 1.2 2003/11/29 17:12:33 christianfoltin Exp $*/

package freemind.modes;

import freemind.main.FreeMindMain;
import freemind.main.Tools;
import java.awt.Color;
import java.awt.Stroke;
import java.awt.BasicStroke;

public abstract class LineAdapter implements MindMapLine {

    protected FreeMindMain frame;
    protected MindMapNode target;
    protected String standardColorPropertyString;
    protected String standardStylePropertyString;
    
	public static final int DEFAULT_WIDTH = -1;
	protected int NORMAL_WIDTH = 1;

    //recursive attributes. may be accessed directly by the save() method.
    protected Color color; 
    protected String style;
    protected int width;
	protected Stroke stroke;

    //
    // Constructors
    //
    public LineAdapter(MindMapNode target,FreeMindMain frame, String standardColorPropertyString, String standardStylePropertyString) {/*_I*/_prof.prof.cnt[6164]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6165]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6166]++;/*I_*/this.target = target;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6167]++;/*I_*/this.standardColorPropertyString = standardColorPropertyString;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6168]++;/*I_*/this.standardStylePropertyString = standardStylePropertyString;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6169]++;/*I_*/width = DEFAULT_WIDTH;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6170]++;/*I_*/stroke = null;/*_I*/}/*I_*/

    }

    //
    // Attributes
    //

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[6171]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6172]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    public Color getColor() {/*_I*/_prof.prof.cnt[6173]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6174]++;/*I_*/if(color==null) {
            /*_I*/_prof.prof.cnt[6175]++;/*I_*/String stdcolor = getFrame().getProperty(standardColorPropertyString);
            /*_I*/{_prof.prof.cnt[6176]++;/*I_*/if (stdcolor != null && stdcolor.length() == 7) {
                /*_I*/{_prof.prof.cnt[6177]++;/*I_*/return Tools.xmlToColor(stdcolor);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6178]++;/*I_*/return Color.red;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6179]++;/*I_*/return color;/*_I*/}/*I_*/
    }

    public void setColor(Color color) {/*_I*/_prof.prof.cnt[6180]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6181]++;/*I_*/this.color = color;/*_I*/}/*I_*/
    }


    public int getWidth() {/*_I*/_prof.prof.cnt[6182]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6183]++;/*I_*/if(width == DEFAULT_WIDTH)
            /*_I*/{_prof.prof.cnt[6184]++;/*I_*/return NORMAL_WIDTH;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6185]++;/*I_*/return width;/*_I*/}/*I_*/ 
    }

    public Stroke getStroke() {/*_I*/_prof.prof.cnt[6186]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6187]++;/*I_*/return stroke;/*_I*/}/*I_*/
    }
		
    public void setWidth(int width) {/*_I*/_prof.prof.cnt[6188]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6189]++;/*I_*/this.width = width;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6190]++;/*I_*/stroke = new BasicStroke(getWidth(),BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);/*_I*/}/*I_*/ 
    }

    public String getStyle() {/*_I*/_prof.prof.cnt[6191]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6192]++;/*I_*/if(style==null) {
            /*_I*/{_prof.prof.cnt[6193]++;/*I_*/return getFrame().getProperty(standardStylePropertyString);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6194]++;/*I_*/return style;/*_I*/}/*I_*/
    }

    public void setStyle(String style) {/*_I*/_prof.prof.cnt[6195]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6196]++;/*I_*/this.style = style;/*_I*/}/*I_*/
    }

    public String toString() {/*_I*/_prof.prof.cnt[6197]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6198]++;/*I_*/return "";/*_I*/}/*I_*/
    }

    public void setTarget(MindMapNode target) {/*_I*/_prof.prof.cnt[6199]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6200]++;/*I_*/this.target = target;/*_I*/}/*I_*/ 
    }


    ///////////
    // Private Methods
    /////////
    /** I see no reason to hide the node, the line belongs to, to the public, but... fc. */
    public MindMapNode getTarget() {/*_I*/_prof.prof.cnt[6201]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6202]++;/*I_*/return target;/*_I*/}/*I_*/
    }

    public Object clone() {/*_I*/_prof.prof.cnt[6203]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6204]++;/*I_*/try {
            /*_I*/_prof.prof.cnt[6205]++;/*I_*/LineAdapter link = (LineAdapter) super.clone();
            // color, ...
            /*_I*/{_prof.prof.cnt[6206]++;/*I_*/link.color = (color==null)?null:new Color(color.getRGB());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6207]++;/*I_*/return link;/*_I*/}/*I_*/
        } catch(java.lang.CloneNotSupportedException e) {
            /*_I*/{_prof.prof.cnt[6208]++;/*I_*/return null;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    
}
