/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: LinkRegistryAdapter.java,v 1.10 2003/12/22 11:14:52 christianfoltin Exp $*/

package freemind.modes;

import freemind.modes.MindMapNode;
import freemind.modes.mindmapmode.MindMapNodeModel;
import freemind.modes.mindmapmode.MindMapArrowLinkModel;
import java.util.HashMap; 
import java.util.Random;
import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.util.ListIterator;



/** Interface for the registry, which manages the ids of nodes and the existing links in a map.
    Thus, this interface is bound to a map model, because other maps have a different registry.*/
public class LinkRegistryAdapter implements MindMapLinkRegistry {
    ////////////////////////////////////////////////////////////////////////////////////////
    ////   State Model                                                                 /////
    ////////////////////////////////////////////////////////////////////////////////////////
    /** State parent interface.*/
    public class ID_BasicStateAdapter implements ID_BasicState{
        protected String ID;
        /** Returns null for many states.*/
        public String getID() {/*_I*/_prof.prof.cnt[5021]++;/*I_*/ /*_I*/{_prof.prof.cnt[5022]++;/*I_*/return ID;/*_I*/}/*I_*/ };
        public String toString() {/*_I*/_prof.prof.cnt[5023]++;/*I_*/ /*_I*/{_prof.prof.cnt[5024]++;/*I_*/return "ID_BasicState, ID="+((ID==null)?"null":ID);/*_I*/}/*I_*/ };
    };
    /** This state interface expresses the state that a node is blank (i.e. without an id, normal state).*/
    public class ID_BlankAdapter extends ID_BasicStateAdapter implements ID_Blank  {};
    /** This state interface expresses the state that a node has an ID, but is abstract.*/
    protected abstract class ID_UsedStateAdapter  extends ID_BasicStateAdapter implements ID_UsedState {
        protected MindMapNode target;
        public MindMapNode getTarget() {/*_I*/_prof.prof.cnt[5025]++;/*I_*/ /*_I*/{_prof.prof.cnt[5026]++;/*I_*/return this.target;/*_I*/}/*I_*/ };
        public ID_UsedStateAdapter(MindMapNode target, String ID) {/*_I*/_prof.prof.cnt[5027]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5028]++;/*I_*/this.target = target;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5029]++;/*I_*/this.ID = ID;/*_I*/}/*I_*/
        };
        /** For cloning.*/
        protected ID_UsedStateAdapter() {/*_I*/_prof.prof.cnt[5030]++;/*I_*/
        };
        protected void clone(ID_UsedStateAdapter state) {/*_I*/_prof.prof.cnt[5031]++;/*I_*/
            /*_I*/{_prof.prof.cnt[5032]++;/*I_*/this.target = state.target;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5033]++;/*I_*/this.ID = state.ID;/*_I*/}/*I_*/
        }
    };
    /** This state interface expresses the state that a node has an ID.*/
    protected class ID_RegisteredAdapter extends ID_UsedStateAdapter implements ID_Registered  {
        public ID_RegisteredAdapter(MindMapNode target, String ID) {
            super(target, ID);/*_I*/_prof.prof.cnt[5034]++;/*I_*//*_I*/_prof.prof.cnt[5035]++;/*I_*/
        };

//         public ID_RegisteredAdapter(ID_PendingAdapter adapter) {
//             clone(adapter);
//         };
    };
//     /** This state interface expresses the state that a node was recently cutted and waits to be inserted at another place.
//         After inserting the states changes to ID_Registered.
//     */
//     public class ID_PendingAdapter  extends ID_UsedStateAdapter implements ID_Pending {
//     };

    ////////////////////////////////////////////////////////////////////////////////////////
    ////   Attributes                                                                  /////
    ////////////////////////////////////////////////////////////////////////////////////////

    protected HashMap /* MindMapNode = Target -> ID_BasicState. */ TargetToID;
    protected HashMap /* id -> vector of links whose TargetToID.get(target) == id.*/  IDToLinks;
    protected HashMap /* id -> vector of links whose TargetToID.get(target) == id and who are cutted recently.*/  IDToCuttedLinks;
    /** The map the registry belongs to.*/
//     protected MindMap map;

    // Logging: for applets the logging must be anonymously. This will be generalized later. fc, 22.12.2003.
    private static java.util.logging.Logger logger = java.util.logging.Logger.getAnonymousLogger(); //getLogger("freemind.modes.LinkRegistryAdapter");

    ////////////////////////////////////////////////////////////////////////////////////////
    ////   Methods                                                                     /////
    ////////////////////////////////////////////////////////////////////////////////////////
    public LinkRegistryAdapter(/*MindMap map*/) {/*_I*/_prof.prof.cnt[5036]++;/*I_*/
//         this.map = map;
        /*_I*/{_prof.prof.cnt[5037]++;/*I_*/TargetToID      = new HashMap();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5038]++;/*I_*/IDToLinks       = new HashMap();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5039]++;/*I_*/IDToCuttedLinks = new HashMap();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5040]++;/*I_*/logger.setLevel(java.util.logging.Level.WARNING);/*_I*/}/*I_*/
        //logger.setLevel(java.util.logging.Level.FINEST);
        /*_I*/{_prof.prof.cnt[5041]++;/*I_*/logger.info("New Registry");/*_I*/}/*I_*/
    };

    protected String generateUniqueID(String proposedID) {/*_I*/_prof.prof.cnt[5042]++;/*I_*/
        /*_I*/_prof.prof.cnt[5043]++;/*I_*/Random ran = new Random();
        /*_I*/_prof.prof.cnt[5044]++;/*I_*/String myProposedID = new String((proposedID != null)?proposedID:"");
        /* The under score is to enable the id to be an ID in the sense of XML/DTD.*/
        /*_I*/{_prof.prof.cnt[5045]++;/*I_*/if(!myProposedID.startsWith("_"))
            /*_I*/{_prof.prof.cnt[5046]++;/*I_*/myProposedID = "_" + myProposedID;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5047]++;/*I_*/String returnValue;
        /*_I*/{_prof.prof.cnt[5048]++;/*I_*/do {
            /*_I*/{_prof.prof.cnt[5049]++;/*I_*/if(!myProposedID.equals("")) {
                // there is a proposal:
                /*_I*/{_prof.prof.cnt[5050]++;/*I_*/returnValue = myProposedID;/*_I*/}/*I_*/
                // this string is tried only once:
                /*_I*/{_prof.prof.cnt[5051]++;/*I_*/myProposedID="";/*_I*/}/*I_*/
            } else {
                /* The prefix is to enable the id to be an ID in the sense of XML/DTD.*/
                /*_I*/{_prof.prof.cnt[5052]++;/*I_*/returnValue = "Freemind_Link_" + Integer.toString(ran.nextInt(2000000000));/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        } while (IDToLinks.containsKey(returnValue));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5053]++;/*I_*/return returnValue;/*_I*/}/*I_*/
    };

    

    /** The main method. Registeres a node with a new (or an existing) node-id. If the state of the id is pending,
     then it is set to registered again.
    */
    public ID_Registered registerLinkTarget(MindMapNode target) {/*_I*/_prof.prof.cnt[5054]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5055]++;/*I_*/return registerLinkTarget(target, null);/*_I*/}/*I_*/
    }

    public ID_Registered registerLinkTarget(MindMapNode target, String proposedID) {/*_I*/_prof.prof.cnt[5056]++;/*I_*/
        // id already exists?
        /*_I*/{_prof.prof.cnt[5057]++;/*I_*/if(TargetToID.containsKey(target)) {
            /*_I*/_prof.prof.cnt[5058]++;/*I_*/ID_BasicState state = (ID_BasicState) TargetToID.get(target);
            /*_I*/{_prof.prof.cnt[5059]++;/*I_*/if(state instanceof ID_Registered) 
                /*_I*/{_prof.prof.cnt[5060]++;/*I_*/return (ID_Registered) state;/*_I*/}/*I_*//*_I*/}/*I_*/
            // blank state.
            // is equal to no state.
        }/*_I*/}/*I_*/
        // generate new id:
        /*_I*/_prof.prof.cnt[5061]++;/*I_*/String newID = generateUniqueID(proposedID);
        /*_I*/_prof.prof.cnt[5062]++;/*I_*/ID_Registered state = new ID_RegisteredAdapter(target, newID);
        /*_I*/{_prof.prof.cnt[5063]++;/*I_*/TargetToID.put(target,state);/*_I*/}/*I_*/
        
        /*_I*/{_prof.prof.cnt[5064]++;/*I_*/logger.info("Register target node:"+target+", with ID="+newID);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5065]++;/*I_*/getAssignedLinksVector(state);/*_I*/}/*I_*//* This is to allocate the link target in the IDToLinks map!.*/
        /*_I*/{_prof.prof.cnt[5066]++;/*I_*/return state;/*_I*/}/*I_*/
    };
        
    public ID_BasicState getState(MindMapNode node) {/*_I*/_prof.prof.cnt[5067]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5068]++;/*I_*/if(TargetToID.containsKey(node))
            /*_I*/{_prof.prof.cnt[5069]++;/*I_*/return (ID_BasicState) TargetToID.get(node);/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5070]++;/*I_*/return new ID_BlankAdapter();/*_I*/}/*I_*/
    };

    protected String getIDString(MindMapNode node) {/*_I*/_prof.prof.cnt[5071]++;/*I_*/ 
        /*_I*/{_prof.prof.cnt[5072]++;/*I_*/if(TargetToID.containsKey(node)) {
            /*_I*/_prof.prof.cnt[5073]++;/*I_*/ID_BasicState state =  (ID_BasicState) TargetToID.get(node);
            /*_I*/{_prof.prof.cnt[5074]++;/*I_*/return state.getID();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5075]++;/*I_*/return null;/*_I*/}/*I_*/
    }
        

    public MindMapNode getTargetForID(String ID){/*_I*/_prof.prof.cnt[5076]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5077]++;/*I_*/for(Iterator i = TargetToID.keySet().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[5078]++;/*I_*/MindMapNode target = (MindMapNode) i.next();
            /*_I*/_prof.prof.cnt[5079]++;/*I_*/ID_BasicState state = (ID_BasicState) TargetToID.get(target);
            /*_I*/{_prof.prof.cnt[5080]++;/*I_*/if((state instanceof ID_Registered) && (state.getID().equals(ID)))
                /*_I*/{_prof.prof.cnt[5081]++;/*I_*/return target;/*_I*/}/*I_*//*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5082]++;/*I_*/return null;/*_I*/}/*I_*/
    }

    
    private Vector  /* of MindMapLink s */ getAssignedLinksVector(ID_Registered state) {/*_I*/_prof.prof.cnt[5083]++;/*I_*/
        /*_I*/_prof.prof.cnt[5084]++;/*I_*/String id = state.getID();
        // look, if target is already present:
        /*_I*/_prof.prof.cnt[5085]++;/*I_*/Vector vec;
        /*_I*/{_prof.prof.cnt[5086]++;/*I_*/if(IDToLinks.containsKey(id) ) {
            /*_I*/{_prof.prof.cnt[5087]++;/*I_*/vec = (Vector) IDToLinks.get(id);/*_I*/}/*I_*/
        } else { 
            /*_I*/{_prof.prof.cnt[5088]++;/*I_*/vec = new Vector();/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5089]++;/*I_*/IDToLinks.put(id,vec);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5090]++;/*I_*/logger.fine("getAssignedLinksVector "+vec);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5091]++;/*I_*/return vec;/*_I*/}/*I_*/
    }

    /** Exception if there are still targets registered.*/
    public void deregisterLinkTarget(MindMapNode target)
        throws java.lang.IllegalArgumentException
    {/*_I*/_prof.prof.cnt[5092]++;/*I_*/
        /*_I*/_prof.prof.cnt[5093]++;/*I_*/ID_BasicState state = getState(target);
        /*_I*/{_prof.prof.cnt[5094]++;/*I_*/if(state instanceof ID_Registered)
            {
                /*_I*/_prof.prof.cnt[5095]++;/*I_*/Vector vec = getAssignedLinksVector((ID_Registered) state);
                /*_I*/{_prof.prof.cnt[5096]++;/*I_*/for(int i = vec.size()-1 ; i >= 0 ; --i) {
                    /*_I*/{_prof.prof.cnt[5097]++;/*I_*/deregisterLink((MindMapLink) vec.get(i));/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                //         if(vec.size() != 0)
                //             throw new java.lang.IllegalArgumentException("Cannot remove a link target, if there are sources pointing to.");
                /*_I*/{_prof.prof.cnt[5098]++;/*I_*/logger.info("Deregister target node:"+target);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5099]++;/*I_*/TargetToID.remove(target);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
    }
                
    /** Method to keep track of the sources associated to a target node. This method also sets the new id to the target. 
        Moreover, it is not required that the target node is already registered. This will be done on the fly.*/
    public void registerLink(MindMapLink link) 
        throws java.lang.IllegalArgumentException
    {/*_I*/_prof.prof.cnt[5100]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5101]++;/*I_*/if((link.getSource()==null)||(link.getTarget()==null)||(link.getDestinationLabel()==null))
            /*_I*/{_prof.prof.cnt[5102]++;/*I_*/throw new java.lang.IllegalArgumentException("Illegal link specification."+link);/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5103]++;/*I_*/MindMapNode source = link.getSource();
        /*_I*/_prof.prof.cnt[5104]++;/*I_*/MindMapNode target = link.getTarget();
        /*_I*/_prof.prof.cnt[5105]++;/*I_*/ID_Registered state = registerLinkTarget(target);
        /*_I*/_prof.prof.cnt[5106]++;/*I_*/Vector vec = getAssignedLinksVector(state);
        // already present?
        /*_I*/{_prof.prof.cnt[5107]++;/*I_*/for(int i = 0 ; i < vec.size(); ++i) {
            /*_I*/{_prof.prof.cnt[5108]++;/*I_*/if(vec.get(i) == link)
                /*_I*/{_prof.prof.cnt[5109]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5110]++;/*I_*/vec.add(link);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5111]++;/*I_*/logger.info("Register link ("+link+") from source node:"+source+" to target " + target);/*_I*/}/*I_*/
    };

        
    public void deregisterLink(MindMapLink link){/*_I*/_prof.prof.cnt[5112]++;/*I_*/
        /*_I*/_prof.prof.cnt[5113]++;/*I_*/MindMapNode source = link.getSource();
        /*_I*/_prof.prof.cnt[5114]++;/*I_*/MindMapNode target = link.getTarget();
        /*_I*/_prof.prof.cnt[5115]++;/*I_*/ID_Registered state = registerLinkTarget(target);
        /*_I*/_prof.prof.cnt[5116]++;/*I_*/Vector vec = getAssignedLinksVector(state);
        /*_I*/{_prof.prof.cnt[5117]++;/*I_*/for(int i = vec.size() -1 ; i >= 0 ; --i) {
            /*_I*/{_prof.prof.cnt[5118]++;/*I_*/logger.fine("Test for equal node:"+source+" to vector(i) " + vec.get(i));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5119]++;/*I_*/if(vec.get(i) == link)
                {
                    /*_I*/{_prof.prof.cnt[5120]++;/*I_*/vec.removeElementAt(i);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[5121]++;/*I_*/logger.info("Deregister link  ("+link+") from source node:"+source+" to target " + target);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    };

    /** Returns a Vector of Nodes that point to the given target node.*/
    public Vector /* of MindMapNode s */ getAllSources(MindMapNode target) {/*_I*/_prof.prof.cnt[5122]++;/*I_*/ 
        /*_I*/_prof.prof.cnt[5123]++;/*I_*/Vector returnValue;
        /*_I*/{_prof.prof.cnt[5124]++;/*I_*/returnValue = new Vector();/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5125]++;/*I_*/ID_BasicState state = getState(target);
        /*_I*/{_prof.prof.cnt[5126]++;/*I_*/if(state instanceof ID_Registered)
            {
                /*_I*/_prof.prof.cnt[5127]++;/*I_*/Vector vec = getAssignedLinksVector((ID_Registered) state);
                /*_I*/{_prof.prof.cnt[5128]++;/*I_*/for(int i = 0 ; i < vec.size(); ++i) {
                    /*_I*/{_prof.prof.cnt[5129]++;/*I_*/returnValue.add( ((MindMapLink) vec.get(i)).getSource() );/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5130]++;/*I_*/return returnValue;/*_I*/}/*I_*/
    };

    public Vector /* of MindMapLink s */ getAllLinks(MindMapNode node) {/*_I*/_prof.prof.cnt[5131]++;/*I_*/ 
        /*_I*/_prof.prof.cnt[5132]++;/*I_*/Vector returnValue = new Vector();
        /*_I*/{_prof.prof.cnt[5133]++;/*I_*/returnValue.addAll(getAllLinksIntoMe( node ));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5134]++;/*I_*/returnValue.addAll(getAllLinksFromMe( node ));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5135]++;/*I_*/logger.fine("All links  ("+returnValue+") from  node:"+node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5136]++;/*I_*/return returnValue;/*_I*/}/*I_*/
    };

    /** @return returns all links to this node.*/
    public Vector /* of MindMapLink s */ getAllLinksIntoMe(MindMapNode target){/*_I*/_prof.prof.cnt[5137]++;/*I_*/
        /*_I*/_prof.prof.cnt[5138]++;/*I_*/Vector returnValue = new Vector();
        /*_I*/_prof.prof.cnt[5139]++;/*I_*/ID_BasicState state = getState(target);
        /*_I*/{_prof.prof.cnt[5140]++;/*I_*/if(getState(target) instanceof ID_Registered)
            {
                /*_I*/_prof.prof.cnt[5141]++;/*I_*/Vector vec = getAssignedLinksVector((ID_Registered) state);
                /* "clone" */
                /*_I*/{_prof.prof.cnt[5142]++;/*I_*/returnValue.addAll( vec );/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5143]++;/*I_*/return returnValue;/*_I*/}/*I_*/
    }

    /** @return returns all links from this node.*/
    public Vector /* of MindMapLink s */ getAllLinksFromMe(MindMapNode source){/*_I*/_prof.prof.cnt[5144]++;/*I_*/
        /*_I*/_prof.prof.cnt[5145]++;/*I_*/Vector returnValue = new Vector();
        /*_I*/_prof.prof.cnt[5146]++;/*I_*/Collection values = IDToLinks.values();
        /*_I*/{_prof.prof.cnt[5147]++;/*I_*/for(Iterator i = values.iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[5148]++;/*I_*/Vector linkVector = (Vector) i.next();
            /*_I*/{_prof.prof.cnt[5149]++;/*I_*/for(int j = 0; j < linkVector.size(); ++j) {
                /*_I*/_prof.prof.cnt[5150]++;/*I_*/MindMapLink link = (MindMapLink) linkVector.get(j);
                /*_I*/{_prof.prof.cnt[5151]++;/*I_*/if(link.getSource() == source)
                    /*_I*/{_prof.prof.cnt[5152]++;/*I_*/returnValue.add(link);/*_I*/}/*I_*//*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5153]++;/*I_*/return returnValue;/*_I*/}/*I_*/
    }

    public String getLabel(MindMapNode target) {/*_I*/_prof.prof.cnt[5154]++;/*I_*/ 
        /*_I*/_prof.prof.cnt[5155]++;/*I_*/ID_BasicState state = getState(target);
        /*_I*/{_prof.prof.cnt[5156]++;/*I_*/if(getState(target) instanceof ID_Registered)
            {
                /*_I*/{_prof.prof.cnt[5157]++;/*I_*/return ((ID_Registered) state).getID();/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5158]++;/*I_*/return null;/*_I*/}/*I_*/
    }

    public void        cutNode(MindMapNode target) {/*_I*/_prof.prof.cnt[5159]++;/*I_*/
        /*_I*/{_prof.prof.cnt[5160]++;/*I_*/logger.entering("LinkRegistryAdapter", "cutNode", target);/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[5161]++;/*I_*/ID_BasicState state = getState(target);
        /*_I*/{_prof.prof.cnt[5162]++;/*I_*/if(state instanceof ID_Registered) {
            // there is a registered target id.
            /*_I*/_prof.prof.cnt[5163]++;/*I_*/String id = getIDString(target);
            // create new vector to the links:
            /*_I*/_prof.prof.cnt[5164]++;/*I_*/Vector vec;
            /*_I*/{_prof.prof.cnt[5165]++;/*I_*/if(IDToCuttedLinks.containsKey(id) ) {
                /*_I*/{_prof.prof.cnt[5166]++;/*I_*/vec = (Vector) IDToCuttedLinks.get(id);/*_I*/}/*I_*/
                // clear this vector:
                /*_I*/{_prof.prof.cnt[5167]++;/*I_*/vec.clear();/*_I*/}/*I_*/
            } else { 
                /*_I*/{_prof.prof.cnt[5168]++;/*I_*/vec = new Vector();/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5169]++;/*I_*/IDToCuttedLinks.put(id,vec);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            // deregister all links to me:
            /*_I*/_prof.prof.cnt[5170]++;/*I_*/Vector links = getAllLinksIntoMe(target);
            /*_I*/{_prof.prof.cnt[5171]++;/*I_*/for(int i = links.size() - 1; i >= 0 ; --i) {
                /*_I*/_prof.prof.cnt[5172]++;/*I_*/MindMapLink link = (MindMapLink) links.get(i);
                /*_I*/{_prof.prof.cnt[5173]++;/*I_*/vec.add(link);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5174]++;/*I_*/logger.info("Adding link ("+link+") to target " + target + " to the cutted nodes from (old) id " + id);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[5175]++;/*I_*/deregisterLink(link);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            // deregister myself to keep the registry tidy.
            /*_I*/{_prof.prof.cnt[5176]++;/*I_*/deregisterLinkTarget(target);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        // deregister all links from me:
        /*_I*/_prof.prof.cnt[5177]++;/*I_*/Vector links = getAllLinksFromMe(target);
        /*_I*/{_prof.prof.cnt[5178]++;/*I_*/for(int i = links.size() - 1; i >= 0 ; --i) {
            /*_I*/_prof.prof.cnt[5179]++;/*I_*/MindMapLink link = (MindMapLink) links.get(i);
            /*_I*/{_prof.prof.cnt[5180]++;/*I_*/deregisterLink(link);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        // and process my sons:
        /*_I*/{_prof.prof.cnt[5181]++;/*I_*/for (ListIterator e = target.childrenUnfolded(); e.hasNext(); ) {
            /*_I*/_prof.prof.cnt[5182]++;/*I_*/MindMapNodeModel child = (MindMapNodeModel)e.next();            
            /*_I*/{_prof.prof.cnt[5183]++;/*I_*/cutNode(child);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5184]++;/*I_*/logger.exiting("LinkRegistryAdapter", "cutNode", target);/*_I*/}/*I_*/
    }
    /** Clears the set of recent cutted nodes.*/
    public void clearCuttedNodeBuffer() {/*_I*/_prof.prof.cnt[5185]++;/*I_*/ /*_I*/{_prof.prof.cnt[5186]++;/*I_*/IDToCuttedLinks.clear();/*_I*/}/*I_*/ };

    /** @return returns all links that have been cutted out recently.*/
    public Vector /* of MindMapLink s*/  getCuttedNode(String oldTargetID) {/*_I*/_prof.prof.cnt[5187]++;/*I_*/ 
        /*_I*/_prof.prof.cnt[5188]++;/*I_*/Vector vec;
        /*_I*/{_prof.prof.cnt[5189]++;/*I_*/if(IDToCuttedLinks.containsKey(oldTargetID) ) {
            /*_I*/{_prof.prof.cnt[5190]++;/*I_*/vec = (Vector) IDToCuttedLinks.get(oldTargetID);/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[5191]++;/*I_*/for(int i = 0; i < vec.size(); ++i) {
                /*_I*/{_prof.prof.cnt[5192]++;/*I_*/vec.set(i, ((MindMapLink) vec.get(i)).clone());/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[5193]++;/*I_*/logger.info("returning link repository ("+vec+") the cutted nodes with old id " + oldTargetID);/*_I*/}/*I_*/
        } else { 
            // error case?
            /*_I*/{_prof.prof.cnt[5194]++;/*I_*/vec = new Vector();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[5195]++;/*I_*/return vec;/*_I*/}/*I_*/
    }


}
