/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: MapAdapter.java,v 1.24 2004/01/28 20:09:27 christianfoltin Exp $*/

package freemind.modes;

import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.main.XMLParseException;
import freemind.modes.StylePattern;
import freemind.view.mindmapview.NodeView;
import freemind.controller.MindMapNodesSelection;

import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Color;
import javax.swing.tree.TreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.EventListenerList;
// Clipboard
import java.awt.datatransfer.*;
import java.io.*;
import java.util.*;

public abstract class MapAdapter implements MindMap {

    private MindMapNode root;
    private EventListenerList treeModelListeners = new EventListenerList();
    /** denotes the amount of changes since the last save. The initial value is one, to ensure, that the model is dirty.*/
    protected int changesPerformedSinceLastSave = 1; 
    protected boolean readOnly = true;
    private Color backgroundColor;
    private File file;
    private FreeMindMain frame;

    // For find next
    private String      findWhat;
    private MindMapNode findFromNode;
    private boolean     findCaseSensitive;
    private LinkedList  findNodeQueue;
    private ArrayList   findNodesUnfoldedByLastFind;


    public MapAdapter (FreeMindMain frame) {/*_I*/_prof.prof.cnt[6420]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6421]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
    }

    //
    // Abstract methods that _must_ be implemented.
    //

    public abstract boolean save(File file); 
    
    public abstract void load(File file) throws FileNotFoundException, IOException, XMLParseException ;

	/**
	 * Attempts to lock the map using semaphore file.
	 * @param file
	 * @return If the map is locked, return the name of the locking user, return null
	 * otherwise.
	 * @throws Exception
	 */
	public String tryToLock(File file) throws Exception {/*_I*/_prof.prof.cnt[6422]++;/*I_*/ 
	   /*_I*/{_prof.prof.cnt[6423]++;/*I_*/return null;/*_I*/}/*I_*/
	}

	public void destroy() {/*_I*/_prof.prof.cnt[6424]++;/*I_*/
		// Do all the necessary destructions in your model,
		// e.g. remove file locks.
	}

    // (PN)
    //    public void close() {
    //    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[6425]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6426]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    //
    // Attributes
    //

    public boolean isSaved() {/*_I*/_prof.prof.cnt[6427]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6428]++;/*I_*/return (changesPerformedSinceLastSave==0);/*_I*/}/*I_*/
    }

	public boolean isReadOnly() {/*_I*/_prof.prof.cnt[6429]++;/*I_*/
		/*_I*/{_prof.prof.cnt[6430]++;/*I_*/return readOnly;/*_I*/}/*I_*/ 
	}

    protected void setSaved(boolean saved) {/*_I*/_prof.prof.cnt[6431]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6432]++;/*I_*/if(saved)
            /*_I*/{_prof.prof.cnt[6433]++;/*I_*/changesPerformedSinceLastSave = 0;/*_I*/}/*I_*/
        else
            /*_I*/{_prof.prof.cnt[6434]++;/*I_*/++changesPerformedSinceLastSave;/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    protected int getNumberOfChangesSinceLastSave() {/*_I*/_prof.prof.cnt[6435]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6436]++;/*I_*/return changesPerformedSinceLastSave;/*_I*/}/*I_*/
    }

    public String getFindWhat() {/*_I*/_prof.prof.cnt[6437]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6438]++;/*I_*/return findWhat;/*_I*/}/*I_*/
    }

    public String getFindFromText() {/*_I*/_prof.prof.cnt[6439]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6440]++;/*I_*/return findFromNode.toString();/*_I*/}/*I_*/
    }

    public Color getBackgroundColor() {/*_I*/_prof.prof.cnt[6441]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6442]++;/*I_*/if (backgroundColor==null) {
	    /*_I*/{_prof.prof.cnt[6443]++;/*I_*/return Tools.xmlToColor(getFrame().getProperty("standardbackgroundcolor"));/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6444]++;/*I_*/return backgroundColor;/*_I*/}/*I_*/
    }

    public void setBackgroundColor(Color backgroundColor) {/*_I*/_prof.prof.cnt[6445]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6446]++;/*I_*/this.backgroundColor = backgroundColor;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6447]++;/*I_*/nodeChanged((MindMapNode)getRoot());/*_I*/}/*I_*///update view anyhow
    }

    public Object getRoot() {/*_I*/_prof.prof.cnt[6448]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6449]++;/*I_*/return root;/*_I*/}/*I_*/
    }

    protected void setRoot(MindMapNode root) {/*_I*/_prof.prof.cnt[6450]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6451]++;/*I_*/this.root = root;/*_I*/}/*I_*/
    }

    /**
     * Change this to always return null if your model doesn't support files.
     */
    public File getFile() {/*_I*/_prof.prof.cnt[6452]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6453]++;/*I_*/return file;/*_I*/}/*I_*/
    }

    /**
     * Return URL of the map (whether as local file or a web location)
     */
    public URL getURL() throws MalformedURLException {/*_I*/_prof.prof.cnt[6454]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6455]++;/*I_*/return getFile() != null ? getFile().toURL() : null;/*_I*/}/*I_*/
    }


    protected void setFile(File file) {/*_I*/_prof.prof.cnt[6456]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6457]++;/*I_*/this.file = file;/*_I*/}/*I_*/
    }

    protected String getText(String textId) {/*_I*/_prof.prof.cnt[6458]++;/*I_*/
	  /*_I*/_prof.prof.cnt[6459]++;/*I_*/String result;
      /*_I*/{_prof.prof.cnt[6460]++;/*I_*/try {
		/*_I*/{_prof.prof.cnt[6461]++;/*I_*/result = getFrame().getResources().getString(textId);/*_I*/}/*I_*/ }
      catch (Exception e) {
      	/*_I*/{_prof.prof.cnt[6462]++;/*I_*/return textId;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6463]++;/*I_*/return result;/*_I*/}/*I_*/ } 

    //
    // Node editing
    //

    public void setFolded( MindMapNode node, boolean folded ) {/*_I*/_prof.prof.cnt[6464]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6465]++;/*I_*/if (node.isFolded() != folded) {
          /*_I*/{_prof.prof.cnt[6466]++;/*I_*/node.setFolded(folded);/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6467]++;/*I_*/getFrame().getView().selectAsTheOnlyOneSelected(node.getViewer());/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6468]++;/*I_*/fireTreeStructureChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
 
    public void setLink( NodeAdapter node, String link ) {/*_I*/_prof.prof.cnt[6469]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6470]++;/*I_*/node.setLink(link);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6471]++;/*I_*/nodeChanged(node);/*_I*/}/*I_*/
    }

    public String getLink( NodeAdapter node ) {/*_I*/_prof.prof.cnt[6472]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6473]++;/*I_*/return node.getLink();/*_I*/}/*I_*/
    }

    public Object[] getPathToRoot( TreeNode node ) {/*_I*/_prof.prof.cnt[6474]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6475]++;/*I_*/return ( ((MindMapNode)node).getPath() ).getPath();/*_I*/}/*I_*///Create Object[] from TreePath
    }

    public void splitNode(MindMapNode node, int caretPosition, String newText) {/*_I*/_prof.prof.cnt[6476]++;/*I_*/}
    
    //
    // cut'n'paste
    //
    public final Transferable cut(MindMapNode node) {/*_I*/_prof.prof.cnt[6477]++;/*I_*/
       // This cut does not need any reimplementation in child Classes!
       /*_I*/_prof.prof.cnt[6478]++;/*I_*/Transferable t = copy(node);
       /*_I*/{_prof.prof.cnt[6479]++;/*I_*/removeNodeFromParent(node);/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6480]++;/*I_*/return t;/*_I*/}/*I_*/
    }

    public final Transferable cut() {/*_I*/_prof.prof.cnt[6481]++;/*I_*/
       /*_I*/_prof.prof.cnt[6482]++;/*I_*/Transferable t = copy();
       // clear all recently cutted links from the registry:
       /*_I*/{_prof.prof.cnt[6483]++;/*I_*/getLinkRegistry().clearCuttedNodeBuffer();/*_I*/}/*I_*/

       // sort selectedNodes list by depth, in order to guarantee that sons are deleted first:
       /*_I*/_prof.prof.cnt[6484]++;/*I_*/LinkedList sortedNodes = getFrame().getView().getSelectedsByDepth();
       /*_I*/{_prof.prof.cnt[6485]++;/*I_*/for(Iterator i = sortedNodes.iterator();i.hasNext();) {
          /*_I*/_prof.prof.cnt[6486]++;/*I_*/MindMapNode selectedNode = ((NodeView)i.next()).getModel();
          /*_I*/{_prof.prof.cnt[6487]++;/*I_*/getLinkRegistry().cutNode(selectedNode);/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6488]++;/*I_*/try {
              /*_I*/{_prof.prof.cnt[6489]++;/*I_*/removeNodeFromParent(selectedNode);/*_I*/}/*I_*/ 
          } catch(IllegalArgumentException e) {
              /*_I*/{_prof.prof.cnt[6490]++;/*I_*/System.err.println("Error occured during cut. The application was not able to cut the node " + selectedNode + ".");/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6491]++;/*I_*/return t;/*_I*/}/*I_*/ }

   public Transferable copy(MindMapNode node) {/*_I*/_prof.prof.cnt[6492]++;/*I_*/
     /*_I*/{_prof.prof.cnt[6493]++;/*I_*/return null;/*_I*/}/*I_*/
   }

   public Transferable copy() {/*_I*/_prof.prof.cnt[6494]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6495]++;/*I_*/return copy(getFrame().getView().getSelectedNodesSortedByY(), null);/*_I*/}/*I_*/ }

   public Transferable copySingle() {/*_I*/_prof.prof.cnt[6496]++;/*I_*/
      /*_I*/_prof.prof.cnt[6497]++;/*I_*/MindMapNode selectedNode = (MindMapNode)getFrame().getView().getSelected().getModel();
      /*_I*/_prof.prof.cnt[6498]++;/*I_*/ArrayList selectedNodes  = new ArrayList();
      /*_I*/{_prof.prof.cnt[6499]++;/*I_*/selectedNodes.add(selectedNode.shallowCopy());/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6500]++;/*I_*/return copy(selectedNodes, selectedNode.toString());/*_I*/}/*I_*/ }

   private Transferable copy(ArrayList selectedNodes, String inPlainText) {/*_I*/_prof.prof.cnt[6501]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6502]++;/*I_*/try {
         /*_I*/_prof.prof.cnt[6503]++;/*I_*/String forNodesFlavor = "";
         /*_I*/_prof.prof.cnt[6504]++;/*I_*/boolean firstLoop = true;
         /*_I*/{_prof.prof.cnt[6505]++;/*I_*/for(Iterator it = selectedNodes.iterator();it.hasNext();) {
            /*_I*/_prof.prof.cnt[6506]++;/*I_*/MindMapNode tmpNode =  (MindMapNode)it.next();
            /*_I*/{_prof.prof.cnt[6507]++;/*I_*/if (firstLoop) {
               /*_I*/{_prof.prof.cnt[6508]++;/*I_*/firstLoop = false;/*_I*/}/*I_*/ }
            else {
               /*_I*/{_prof.prof.cnt[6509]++;/*I_*/forNodesFlavor += "<nodeseparator>";/*_I*/}/*I_*/ }/*_I*/}/*I_*/

            // v This is not a smart solution. While copy() handles multiple flavors,
            //   copy(node) handles only string flavor, but with the meaning of nodes flavor.
            //   It does no harm apart from being not very nice and confusing.           
            /*_I*/{_prof.prof.cnt[6510]++;/*I_*/forNodesFlavor += copy(tmpNode).getTransferData(DataFlavor.stringFlavor);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

         /*_I*/_prof.prof.cnt[6511]++;/*I_*/String plainText = inPlainText != null ? inPlainText : getAsPlainText(selectedNodes);
         /*_I*/{_prof.prof.cnt[6512]++;/*I_*/return new MindMapNodesSelection
            (forNodesFlavor, plainText, getAsRTF(selectedNodes), "");/*_I*/}/*I_*/ }
         //return new StringSelection(forClipboard); }


      catch (UnsupportedFlavorException ex) { /*_I*/{_prof.prof.cnt[6513]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/ }
      catch (IOException ex) { /*_I*/{_prof.prof.cnt[6514]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6515]++;/*I_*/return null;/*_I*/}/*I_*/ }

    public String getAsPlainText(List mindMapNodes) {/*_I*/_prof.prof.cnt[6516]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6517]++;/*I_*/return "";/*_I*/}/*I_*/ }
 

    public String getAsRTF(List mindMapNodes) {/*_I*/_prof.prof.cnt[6518]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6519]++;/*I_*/return "";/*_I*/}/*I_*/ }

    public void paste(Transferable t, MindMapNode parent) {/*_I*/_prof.prof.cnt[6520]++;/*I_*/
        /*_I*/_prof.prof.cnt[6521]++;/*I_*/boolean isLeft = false;
        /*_I*/{_prof.prof.cnt[6522]++;/*I_*/if(parent.isLeft()!= null)
            /*_I*/{_prof.prof.cnt[6523]++;/*I_*/isLeft = parent.isLeft().getValue();/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6524]++;/*I_*/paste(t, /*target=*/parent, /*asSibling=*/ false, isLeft);/*_I*/}/*I_*/ }

    public void paste(Transferable t, MindMapNode target, boolean asSibling, boolean isLeft) {/*_I*/_prof.prof.cnt[6525]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6526]++;/*I_*/if (t != null) {
            // In MapAdapter it does basically nothing.
            /*_I*/{_prof.prof.cnt[6527]++;/*I_*/nodeStructureChanged(asSibling ? target.getParent() : target);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    public void paste(MindMapNode node, MindMapNode parent) {/*_I*/_prof.prof.cnt[6528]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6529]++;/*I_*/if (node != null) {
            /*_I*/{_prof.prof.cnt[6530]++;/*I_*/insertNodeInto(node, parent);/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[6531]++;/*I_*/nodeStructureChanged(parent);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    public String getRestoreable() {/*_I*/_prof.prof.cnt[6532]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6533]++;/*I_*/return null;/*_I*/}/*I_*/
    }


    public MindMapLinkRegistry getLinkRegistry() {/*_I*/_prof.prof.cnt[6534]++;/*I_*/ /*_I*/{_prof.prof.cnt[6535]++;/*I_*/return null;/*_I*/}/*I_*/ }


    public void insertNodeIntoNoEvent(MindMapNode newChild, MindMapNode parent) {/*_I*/_prof.prof.cnt[6536]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6537]++;/*I_*/insertNodeIntoNoEvent(newChild, parent, false);/*_I*/}/*I_*/ }

    public void insertNodeIntoNoEvent(MindMapNode newChild, MindMapNode parent,
                                      boolean asSibling){/*_I*/_prof.prof.cnt[6538]++;/*I_*/
        // Daniel: This is a hackery in the sense that I don't
        // understand the method "nodesWereInserted".

        // This code is based on empirical observation that in
        // some cases avoiding the mentioned method does not throw
        // exception.

        // The reason why we do such a thing at all is that calling the
        // method "nodesWereInserted" dramatically lowers the performance.

        // The default position is after the last node
        /*_I*/{_prof.prof.cnt[6539]++;/*I_*/if (asSibling) {
           /*_I*/_prof.prof.cnt[6540]++;/*I_*/MindMapNode oldpa = parent.getParentNode();
           //insertNodeInto(node, parent, parent.getChildPosition(target)); //}
           /*_I*/{_prof.prof.cnt[6541]++;/*I_*/oldpa.insert(newChild, oldpa.getChildPosition(parent));/*_I*/}/*I_*/ } //     parent.getRealChildCount()); }
        // } 
        else {
           /*_I*/{_prof.prof.cnt[6542]++;/*I_*/parent.insert(newChild, parent.getChildCount());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
    }

    public void insertNodeInto(MindMapNode newChild,
			       MindMapNode parent){/*_I*/_prof.prof.cnt[6543]++;/*I_*/
        // The default position is after the last node
        /*_I*/{_prof.prof.cnt[6544]++;/*I_*/insertNodeInto(newChild, parent, parent.getChildCount());/*_I*/}/*I_*/
    }

    /**
     * Use this method to add children because it will cause the appropriate event.
     */
    public void insertNodeInto(MutableTreeNode newChild,
			       MutableTreeNode parent, int index){/*_I*/_prof.prof.cnt[6545]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6546]++;/*I_*/parent.insert(newChild, index);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6547]++;/*I_*/nodesWereInserted(parent, new int[]{ index });/*_I*/}/*I_*/
        // ^ Daniel: I don't understand the reason why this is here.  But it
        // is necessary, because user's action "New Node" throws
        // an exception otherwise.
    }

    /**
     * The direction is used if side left and right are present. then the next suitable place on the same side#
     is searched. if there is no such place, then the side is changed.
     @return returns the new index.
     */
    public int moveNodeTo(MindMapNode newChild, MindMapNode parent, int index, int direction){/*_I*/_prof.prof.cnt[6548]++;/*I_*/
        /*_I*/_prof.prof.cnt[6549]++;/*I_*/int newIndex = index;
        /*_I*/{_prof.prof.cnt[6550]++;/*I_*/if(newChild.isLeft() != null) {
            /*_I*/_prof.prof.cnt[6551]++;/*I_*/boolean iAmLeft = newChild.isLeft().getValue();
            /*_I*/_prof.prof.cnt[6552]++;/*I_*/int maxIndex = parent.getChildCount(); 
            /*_I*/_prof.prof.cnt[6553]++;/*I_*/Vector sortedNodes        = new Vector();
            /*_I*/_prof.prof.cnt[6554]++;/*I_*/Vector sortedNodesIndices = new Vector();
            // definition of leftRightBoundary: all elements with index >= leftRightBoundary are right.
            /*_I*/_prof.prof.cnt[6555]++;/*I_*/int leftRightBoundary=0;
            // sort the nodes according to their direction and position:
            // and find myself:
            /*_I*/_prof.prof.cnt[6556]++;/*I_*/Tools.IntHolder myPositionInVector=null;
            /*_I*/{_prof.prof.cnt[6557]++;/*I_*/for(int i = 0 ; i < maxIndex; ++i) {
                /*_I*/_prof.prof.cnt[6558]++;/*I_*/TreeNode nextItem = parent.getChildAt(i);
                /*_I*/{_prof.prof.cnt[6559]++;/*I_*/if(!(nextItem instanceof MindMapNode)){
                    /*_I*/{_prof.prof.cnt[6560]++;/*I_*/System.err.println("Strange error I in method insertNodeInto of MapAdapter.");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6561]++;/*I_*/return newIndex;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[6562]++;/*I_*/MindMapNode nextNode = (MindMapNode) nextItem;
                /*_I*/{_prof.prof.cnt[6563]++;/*I_*/if(nextNode.isLeft() != null && nextNode.isLeft().getValue() == true){
                    // left
                    /*_I*/{_prof.prof.cnt[6564]++;/*I_*/sortedNodes.add       (leftRightBoundary, nextNode);/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6565]++;/*I_*/sortedNodesIndices.add(leftRightBoundary, new Tools.IntHolder(i));/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6566]++;/*I_*/leftRightBoundary++;/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6567]++;/*I_*/if(i == index)
                        /*_I*/{_prof.prof.cnt[6568]++;/*I_*/myPositionInVector = (Tools.IntHolder) sortedNodesIndices.get(leftRightBoundary-1);/*_I*/}/*I_*//*_I*/}/*I_*/
                } else {
                    // right
                    /*_I*/{_prof.prof.cnt[6569]++;/*I_*/sortedNodes.add(nextNode);/*_I*/}/*I_*/ // at the end.
                    /*_I*/{_prof.prof.cnt[6570]++;/*I_*/sortedNodesIndices.add(new Tools.IntHolder(i));/*_I*/}/*I_*/ // at the end.
                    /*_I*/{_prof.prof.cnt[6571]++;/*I_*/if(i == index)
                        /*_I*/{_prof.prof.cnt[6572]++;/*I_*/myPositionInVector = (Tools.IntHolder) sortedNodesIndices.get(sortedNodesIndices.size()-1);/*_I*/}/*I_*//*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6573]++;/*I_*/if(myPositionInVector == null){
                /*_I*/{_prof.prof.cnt[6574]++;/*I_*/System.err.println("Strange error II in method insertNodeInto of MapAdapter.");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6575]++;/*I_*/return newIndex;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[6576]++;/*I_*/int oldPositionInVector = sortedNodesIndices.indexOf(myPositionInVector);
            /*_I*/_prof.prof.cnt[6577]++;/*I_*/int newPositionInVector = oldPositionInVector + direction;
            /*_I*/_prof.prof.cnt[6578]++;/*I_*/boolean passedFrontiers=false;
            /*_I*/{_prof.prof.cnt[6579]++;/*I_*/if(newPositionInVector < 0) {
                /*_I*/{_prof.prof.cnt[6580]++;/*I_*/newPositionInVector = sortedNodesIndices.size()-1;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6581]++;/*I_*/passedFrontiers = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6582]++;/*I_*/if(newPositionInVector  >= sortedNodesIndices.size()) {
                /*_I*/{_prof.prof.cnt[6583]++;/*I_*/newPositionInVector = 0;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6584]++;/*I_*/passedFrontiers = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6585]++;/*I_*/newIndex = ((Tools.IntHolder) sortedNodesIndices.get(newPositionInVector)).getValue();/*_I*/}/*I_*/
            // test if frontiers passed:
            /*_I*/{_prof.prof.cnt[6586]++;/*I_*/if(((newPositionInVector >= leftRightBoundary) && oldPositionInVector < leftRightBoundary) ||
               ((newPositionInVector < leftRightBoundary) && oldPositionInVector >= leftRightBoundary)) {
                /*_I*/{_prof.prof.cnt[6587]++;/*I_*/newPositionInVector = oldPositionInVector;/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6588]++;/*I_*/passedFrontiers = true;/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6589]++;/*I_*/if(passedFrontiers) {
                // flip:
                /*_I*/{_prof.prof.cnt[6590]++;/*I_*/newChild.setLeft(!iAmLeft);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6591]++;/*I_*/return newIndex;/*_I*/}/*I_*/
    }

    /**
     * Joerg: Message this to remove node from its parent. This will message
     * nodesWereRemoved to create the appropriate event. This is the
     * preferred way to remove a node as it handles the event creation
     * for you.
     */
    public void removeNodeFromParent(MutableTreeNode node) {/*_I*/_prof.prof.cnt[6592]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6593]++;/*I_*/removeNodeFromParent(node, /*notify=*/ true);/*_I*/}/*I_*/ }

    public void removeNodeFromParent(MutableTreeNode node, boolean notify) {/*_I*/_prof.prof.cnt[6594]++;/*I_*/
	/*_I*/_prof.prof.cnt[6595]++;/*I_*/MutableTreeNode parent = (MutableTreeNode)node.getParent();
	    
	/*_I*/{_prof.prof.cnt[6596]++;/*I_*/if (parent == null)
           /*_I*/{_prof.prof.cnt[6597]++;/*I_*/throw new IllegalArgumentException("node does not have a parent.");/*_I*/}/*I_*//*_I*/}/*I_*/

	/*_I*/_prof.prof.cnt[6598]++;/*I_*/int[] childIndex = new int[1];
	/*_I*/_prof.prof.cnt[6599]++;/*I_*/Object[] removedArray = new Object[1];
	    
	/*_I*/{_prof.prof.cnt[6600]++;/*I_*/childIndex[0] = parent.getIndex(node);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6601]++;/*I_*/parent.remove(node);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6602]++;/*I_*/if (notify) {
           /*_I*/{_prof.prof.cnt[6603]++;/*I_*/removedArray[0] = node;/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6604]++;/*I_*/nodesWereRemoved(parent, childIndex, removedArray);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void changeNode(MindMapNode node, String newText) {/*_I*/_prof.prof.cnt[6605]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6606]++;/*I_*/node.setUserObject(newText);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6607]++;/*I_*/nodeChanged(node);/*_I*/}/*I_*/
    }

    //
    // Interface TreeModel
    //

    public Object getChild( Object parent, int index ) {/*_I*/_prof.prof.cnt[6608]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6609]++;/*I_*/return ( (TreeNode)parent ).getChildAt( index );/*_I*/}/*I_*/
    }

    public int getChildCount( Object parent ) {/*_I*/_prof.prof.cnt[6610]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6611]++;/*I_*/return ( (TreeNode)parent ).getChildCount();/*_I*/}/*I_*/
    }

    public int getIndexOfChild( Object parent, Object child ) {/*_I*/_prof.prof.cnt[6612]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6613]++;/*I_*/return ( (TreeNode)parent ).getIndex( (TreeNode)child );/*_I*/}/*I_*/
    }
	
    public boolean isLeaf( Object node ) {/*_I*/_prof.prof.cnt[6614]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6615]++;/*I_*/return ( (TreeNode)node ).isLeaf();/*_I*/}/*I_*/
    }

    public void addTreeModelListener( TreeModelListener l ) {/*_I*/_prof.prof.cnt[6616]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6617]++;/*I_*/treeModelListeners.add( TreeModelListener.class, l );/*_I*/}/*I_*/
    }

    public void removeTreeModelListener( TreeModelListener l ) {/*_I*/_prof.prof.cnt[6618]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6619]++;/*I_*/treeModelListeners.remove( TreeModelListener.class, l );/*_I*/}/*I_*/
    }

    public void valueForPathChanged( TreePath path, Object newValue ) {/*_I*/_prof.prof.cnt[6620]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6621]++;/*I_*/( (MutableTreeNode)path.getLastPathComponent() ).setUserObject( newValue );/*_I*/}/*I_*/
    }

    public void applyPattern(NodeAdapter node, StylePattern pattern) {/*_I*/_prof.prof.cnt[6622]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6623]++;/*I_*/applyPattern(node, pattern, true /* = visible */);/*_I*/}/*I_*/
    }

    protected void applyPattern(NodeAdapter node, StylePattern pattern, boolean visible) {/*_I*/_prof.prof.cnt[6624]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6625]++;/*I_*/if (pattern.getAppliesToNode()) {
           /*_I*/{_prof.prof.cnt[6626]++;/*I_*/if (pattern.getText() != null) {
              /*_I*/{_prof.prof.cnt[6627]++;/*I_*/node.setUserObject(pattern.getText());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6628]++;/*I_*/node.setColor(pattern.getNodeColor());/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6629]++;/*I_*/node.setStyle(pattern.getNodeStyle());/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6630]++;/*I_*/if (pattern.getAppliesToNodeIcon()) {
              /*_I*/{_prof.prof.cnt[6631]++;/*I_*/if (pattern.getNodeIcon() == null) {
                 /*_I*/{_prof.prof.cnt[6632]++;/*I_*/while (node.removeLastIcon()>0) {}/*_I*/}/*I_*/}
              else {
                 /*_I*/{_prof.prof.cnt[6633]++;/*I_*/node.addIcon(pattern.getNodeIcon());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/ // fc, 28.9.2003
           /*_I*/{_prof.prof.cnt[6634]++;/*I_*/if (pattern.getAppliesToNodeFont()) {
              /*_I*/{_prof.prof.cnt[6635]++;/*I_*/node.setFont(pattern.getNodeFont());/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[6636]++;/*I_*/node.estabilishOwnFont();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[6637]++;/*I_*/if (pattern.getAppliesToEdge()) {
           /*_I*/_prof.prof.cnt[6638]++;/*I_*/EdgeAdapter edge = (EdgeAdapter)node.getEdge();
           /*_I*/{_prof.prof.cnt[6639]++;/*I_*/edge.setColor(pattern.getEdgeColor());/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6640]++;/*I_*/edge.setStyle(pattern.getEdgeStyle());/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[6641]++;/*I_*/edge.setWidth(pattern.getEdgeWidth());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        
        /*_I*/{_prof.prof.cnt[6642]++;/*I_*/if(pattern.getAppliesToChildren()) {
             /*_I*/{_prof.prof.cnt[6643]++;/*I_*/for (ListIterator i = node.childrenUnfolded(); i.hasNext(); ) {
                 /*_I*/_prof.prof.cnt[6644]++;/*I_*/NodeAdapter child = (NodeAdapter) i.next();
                 /*_I*/{_prof.prof.cnt[6645]++;/*I_*/applyPattern(child, pattern.getChildrenStylePattern(), (visible)?(!node.isFolded()):false);/*_I*/}/*I_*/
             }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6646]++;/*I_*/if(visible)
            /*_I*/{_prof.prof.cnt[6647]++;/*I_*/nodeChanged(node);/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }

    // find

    public boolean find(MindMapNode node, String what, boolean caseSensitive) {/*_I*/_prof.prof.cnt[6648]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6649]++;/*I_*/findNodesUnfoldedByLastFind = new ArrayList();/*_I*/}/*I_*/
       /*_I*/_prof.prof.cnt[6650]++;/*I_*/LinkedList nodes = new LinkedList();
       /*_I*/{_prof.prof.cnt[6651]++;/*I_*/nodes.addFirst(node);/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6652]++;/*I_*/findFromNode = node;/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6653]++;/*I_*/if (!caseSensitive) {
          /*_I*/{_prof.prof.cnt[6654]++;/*I_*/what = what.toLowerCase();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6655]++;/*I_*/return find(nodes, what, caseSensitive);/*_I*/}/*I_*/ }

    public boolean findNext() {/*_I*/_prof.prof.cnt[6656]++;/*I_*/
       // Precodition: findWhat != null. We check the precodition but give no message.

       // The logic of find next is vulnerable. find next relies on the queue
       // of nodes from previous find / find next. However, between previous
       // find / find next and this find next, nodes could have been deleted
       // or moved. The logic expects that no changes happened, even that no
       // node has been folded / unfolded.

       // You may want to come with more correct solution, but this one
       // works for most uses, and does not cause any big trouble except
       // perhaps for some uncaught exceptions. As a result, it is not very
       // nice, but far from critical and working quite fine.

       /*_I*/{_prof.prof.cnt[6657]++;/*I_*/if (findWhat != null) {
          /*_I*/{_prof.prof.cnt[6658]++;/*I_*/return find(findNodeQueue, findWhat, findCaseSensitive);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6659]++;/*I_*/return false;/*_I*/}/*I_*/ }

    private boolean find(LinkedList /*queue of MindMapNode*/ nodes, String what, boolean caseSensitive) {/*_I*/_prof.prof.cnt[6660]++;/*I_*/
       // Precondition: if !caseSensitive then >>what<< is in lowercase.

       // Fold the path of previously found node
       /*_I*/_prof.prof.cnt[6661]++;/*I_*/boolean thereWereNodesToBeFolded = !findNodesUnfoldedByLastFind.isEmpty();
       /*_I*/{_prof.prof.cnt[6662]++;/*I_*/if (!findNodesUnfoldedByLastFind.isEmpty()) {

          //if (false) {
          /*_I*/_prof.prof.cnt[6663]++;/*I_*/ListIterator i = findNodesUnfoldedByLastFind.listIterator
             (findNodesUnfoldedByLastFind.size());
          /*_I*/{_prof.prof.cnt[6664]++;/*I_*/while (i.hasPrevious()) {
             /*_I*/_prof.prof.cnt[6665]++;/*I_*/MindMapNode node = (MindMapNode)i.previous();
             /*_I*/{_prof.prof.cnt[6666]++;/*I_*/try {
                /*_I*/{_prof.prof.cnt[6667]++;/*I_*/setFolded(node, true);/*_I*/}/*I_*/ }
             catch (Exception e) {}/*_I*/}/*I_*/}/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6668]++;/*I_*/findNodesUnfoldedByLastFind = new ArrayList();/*_I*/}/*I_*/ }/*_I*/}/*I_*/

       // We implement width-first search.
       /*_I*/{_prof.prof.cnt[6669]++;/*I_*/while (!nodes.isEmpty()) {
          /*_I*/_prof.prof.cnt[6670]++;/*I_*/MindMapNode node = (MindMapNode)nodes.removeFirst();
          // Add children to the queue
          /*_I*/{_prof.prof.cnt[6671]++;/*I_*/for (ListIterator i = node.childrenUnfolded(); i.hasNext(); ) {
             /*_I*/{_prof.prof.cnt[6672]++;/*I_*/nodes.addLast(i.next());/*_I*/}/*I_*/ }/*_I*/}/*I_*/

          /*_I*/_prof.prof.cnt[6673]++;/*I_*/String nodeText = caseSensitive ?
             node.toString() : node.toString().toLowerCase();
          /*_I*/{_prof.prof.cnt[6674]++;/*I_*/if (nodeText.indexOf(what) >= 0) {             // Found
              /*_I*/{_prof.prof.cnt[6675]++;/*I_*/displayNode(node, findNodesUnfoldedByLastFind);/*_I*/}/*I_*/
              
             // Save the state for find next
             /*_I*/{_prof.prof.cnt[6676]++;/*I_*/findWhat          = what;/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[6677]++;/*I_*/findCaseSensitive = caseSensitive;/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[6678]++;/*I_*/findNodeQueue     = nodes;/*_I*/}/*I_*/

             /*_I*/{_prof.prof.cnt[6679]++;/*I_*/return true;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

       /*_I*/{_prof.prof.cnt[6680]++;/*I_*/if (thereWereNodesToBeFolded) {
          /*_I*/{_prof.prof.cnt[6681]++;/*I_*/getFrame().getView().centerNode(findFromNode.getViewer());/*_I*/}/*I_*/ }
       else {
          /*_I*/{_prof.prof.cnt[6682]++;/*I_*/getFrame().getView().scrollNodeToVisible(findFromNode.getViewer());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6683]++;/*I_*/getFrame().getView().selectAsTheOnlyOneSelected(findFromNode.getViewer());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6684]++;/*I_*/frame.getController().obtainFocusForSelected();/*_I*/}/*I_*/

       /*_I*/{_prof.prof.cnt[6685]++;/*I_*/return false;/*_I*/}/*I_*/ }

    public void displayNode(MindMapNode node, ArrayList NodesUnfoldedByDisplay) {/*_I*/_prof.prof.cnt[6686]++;/*I_*/
             // Unfold the path to the node
             /*_I*/_prof.prof.cnt[6687]++;/*I_*/Object[] path = getPathToRoot(node); 
             // Iterate the path with the exception of the last node
             /*_I*/{_prof.prof.cnt[6688]++;/*I_*/for (int i = 0; i < path.length - 1; i++) {
                /*_I*/_prof.prof.cnt[6689]++;/*I_*/MindMapNode nodeOnPath = (MindMapNode)path[i];
                //System.out.println(nodeOnPath);
                /*_I*/{_prof.prof.cnt[6690]++;/*I_*/if (nodeOnPath.isFolded()) {
                    /*_I*/{_prof.prof.cnt[6691]++;/*I_*/if(findNodesUnfoldedByLastFind != null) 
                        /*_I*/{_prof.prof.cnt[6692]++;/*I_*/findNodesUnfoldedByLastFind.add(nodeOnPath);/*_I*/}/*I_*//*_I*/}/*I_*/
                   /*_I*/{_prof.prof.cnt[6693]++;/*I_*/setFolded(nodeOnPath, false);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

             // Select the node and scroll to it.
             /*_I*/{_prof.prof.cnt[6694]++;/*I_*/getFrame().getView().centerNode(node.getViewer());/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[6695]++;/*I_*/getFrame().getView().selectAsTheOnlyOneSelected(node.getViewer());/*_I*/}/*I_*/
             /*_I*/{_prof.prof.cnt[6696]++;/*I_*/frame.getController().obtainFocusForSelected();/*_I*/}/*I_*/
    }

    //
    // API for updating the view.
    //

    //
    // TreeNodesRemoved
    //

    /**
     * Invoke this method after you've removed some TreeNodes from
     * node.  childIndices should be the index of the removed elements and
     * must be sorted in ascending order. And removedChildren should be
     * the array of the children objects that were removed.
     */
    protected void nodesWereRemoved(TreeNode parent, int[] childIndices,
				 Object[] removedChildren) {/*_I*/_prof.prof.cnt[6697]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6698]++;/*I_*/setSaved(false);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6699]++;/*I_*/if(parent != null && childIndices != null) {
	    /*_I*/{_prof.prof.cnt[6700]++;/*I_*/fireTreeNodesRemoved(this, getPathToRoot( (MindMapNode)parent ), childIndices, 
				 removedChildren);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    //
    // TreeNodesInserted
    //

    /**
     * Invoke this method after you've inserted some TreeNodes into
     * node.  childIndices should be the index of the new elements and
     * must be sorted in ascending order.
     */
    protected void nodesWereInserted(TreeNode node, int[] childIndices) {/*_I*/_prof.prof.cnt[6701]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6702]++;/*I_*/if (treeModelListeners != null && node != null && 
           childIndices != null && childIndices.length > 0) {
          /*_I*/{_prof.prof.cnt[6703]++;/*I_*/setSaved(false);/*_I*/}/*I_*/

	  /*_I*/_prof.prof.cnt[6704]++;/*I_*/Object[] newChildren = new Object[childIndices.length];	    
          /*_I*/{_prof.prof.cnt[6705]++;/*I_*/for(int counter = 0; counter < childIndices.length; counter++) {
             /*_I*/{_prof.prof.cnt[6706]++;/*I_*/newChildren[counter] = node.getChildAt(childIndices[counter]);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

          /*_I*/{_prof.prof.cnt[6707]++;/*I_*/fireTreeNodesInserted(this, getPathToRoot( (MindMapNode)node ), childIndices, newChildren);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }
    
    //
    // TreeNodesChanged
    //

    /**
      * Invoke this method after you've changed how node is to be
      * represented in the tree. 
      */
    protected void nodeChanged(TreeNode node) {/*_I*/_prof.prof.cnt[6708]++;/*I_*/

        // Tell the mode controller that the node was changed, for the case
        // he is interested.
        /*_I*/{_prof.prof.cnt[6709]++;/*I_*/frame.getController().getMode().getModeController().nodeChanged((MindMapNode)node);/*_I*/}/*I_*/ 

        /*_I*/{_prof.prof.cnt[6710]++;/*I_*/if (treeModelListeners != null && node != null) {
            /*_I*/_prof.prof.cnt[6711]++;/*I_*/TreeNode parent = node.getParent();

            /*_I*/{_prof.prof.cnt[6712]++;/*I_*/if (parent != null) {
                /*_I*/_prof.prof.cnt[6713]++;/*I_*/int anIndex = parent.getIndex(node);
                /*_I*/{_prof.prof.cnt[6714]++;/*I_*/if (anIndex != -1) {
                    /*_I*/_prof.prof.cnt[6715]++;/*I_*/int[] cIndexs = new int[1];

                    /*_I*/{_prof.prof.cnt[6716]++;/*I_*/cIndexs[0] = anIndex;/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6717]++;/*I_*/nodesChanged(parent, cIndexs);/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
	    }
	    else /*_I*/{_prof.prof.cnt[6718]++;/*I_*/if (((MindMapNode)node).isRoot()) {
		/*_I*/{_prof.prof.cnt[6719]++;/*I_*/nodesChanged(node, null);/*_I*/}/*I_*/
	    }/*_I*/}/*I_*//*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    /**
      * Invoke this method after you've changed how the children identified by
      * childIndicies are to be represented in the tree.
      */
    protected void nodesChanged(TreeNode node, int[] childIndices) {/*_I*/_prof.prof.cnt[6720]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6721]++;/*I_*/setSaved(false);/*_I*/}/*I_*/

       /*_I*/{_prof.prof.cnt[6722]++;/*I_*/if (node != null) {
          /*_I*/{_prof.prof.cnt[6723]++;/*I_*/if (childIndices != null) {
             /*_I*/_prof.prof.cnt[6724]++;/*I_*/int cCount = childIndices.length;
             
             /*_I*/{_prof.prof.cnt[6725]++;/*I_*/if (cCount > 0) {
                /*_I*/_prof.prof.cnt[6726]++;/*I_*/Object[] cChildren = new Object[cCount];
                
                /*_I*/{_prof.prof.cnt[6727]++;/*I_*/for(int counter = 0; counter < cCount; counter++)
                   /*_I*/{_prof.prof.cnt[6728]++;/*I_*/cChildren[counter] = node.getChildAt
                      (childIndices[counter]);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6729]++;/*I_*/fireTreeNodesChanged(this, getPathToRoot(node), childIndices, cChildren);/*_I*/}/*I_*/
             }/*_I*/}/*I_*/
          }
          else /*_I*/{_prof.prof.cnt[6730]++;/*I_*/if (((MindMapNode)node).isRoot()) {
             /*_I*/{_prof.prof.cnt[6731]++;/*I_*/fireTreeNodesChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/
          }/*_I*/}/*I_*//*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

    //
    // TreeStructureChanged
    //    

    /**
      * Invoke this method if you've totally changed the children of
      * node and its childrens children...  This will post a
      * treeStructureChanged event.
      */
    protected void nodeStructureChanged(TreeNode node) {/*_I*/_prof.prof.cnt[6732]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6733]++;/*I_*/setSaved(false);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6734]++;/*I_*/if (node != null) {
           /*_I*/{_prof.prof.cnt[6735]++;/*I_*/fireTreeStructureChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    /**
     * Invoke this method if you've modified the TreeNodes upon which this
     * model depends.  The model will notify all of its listeners that the
     * model has changed below the node <code>node</code> (PENDING).
     */
    protected void reload(TreeNode node) {/*_I*/_prof.prof.cnt[6736]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6737]++;/*I_*/if (node != null) {
           /*_I*/{_prof.prof.cnt[6738]++;/*I_*/fireTreeStructureChanged(this, getPathToRoot(node), null, null);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}


    /////////
    // private methods. Internal implementation.
    ////////

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesRemoved(Object source, Object[] path, 
					int[] childIndices, 
					Object[] children) {/*_I*/_prof.prof.cnt[6739]++;/*I_*/
	// Guaranteed to return a non-null array
	/*_I*/_prof.prof.cnt[6740]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
	/*_I*/_prof.prof.cnt[6741]++;/*I_*/TreeModelEvent e = null;
	// Process the listeners last to first, notifying
	// those that are interested in this event
	/*_I*/{_prof.prof.cnt[6742]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
	    /*_I*/{_prof.prof.cnt[6743]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
		// Lazily create the event:
		/*_I*/{_prof.prof.cnt[6744]++;/*I_*/if (e == null)
		    /*_I*/{_prof.prof.cnt[6745]++;/*I_*/e = new TreeModelEvent(source, path, 
					   childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[6746]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesRemoved(e);/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/          
	}/*_I*/}/*I_*/
    }

    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesInserted(Object source, Object[] path, 
					 int[] childIndices, 
					 Object[] children) {/*_I*/_prof.prof.cnt[6747]++;/*I_*/
	// Guaranteed to return a non-null array
	/*_I*/_prof.prof.cnt[6748]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
	/*_I*/_prof.prof.cnt[6749]++;/*I_*/TreeModelEvent e = null;
	// Process the listeners last to first, notifying
	// those that are interested in this event
	/*_I*/{_prof.prof.cnt[6750]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[6751]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[6752]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[6753]++;/*I_*/e = new TreeModelEvent(source, path, 
                                           childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6754]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesInserted(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeNodesChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {/*_I*/_prof.prof.cnt[6755]++;/*I_*/
        // Guaranteed to return a non-null array
        /*_I*/_prof.prof.cnt[6756]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
        /*_I*/_prof.prof.cnt[6757]++;/*I_*/TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        /*_I*/{_prof.prof.cnt[6758]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[6759]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[6760]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[6761]++;/*I_*/e = new TreeModelEvent(source, path, 
                                           childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6762]++;/*I_*/((TreeModelListener)listeners[i+1]).treeNodesChanged(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    private void fireTreeStructureChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {/*_I*/_prof.prof.cnt[6763]++;/*I_*/
        // Guaranteed to return a non-null array
        /*_I*/_prof.prof.cnt[6764]++;/*I_*/Object[] listeners = treeModelListeners.getListenerList();
        /*_I*/_prof.prof.cnt[6765]++;/*I_*/TreeModelEvent e = null;
        // Process the listeners last to first, notifying
        // those that are interested in this event
        /*_I*/{_prof.prof.cnt[6766]++;/*I_*/for (int i = listeners.length-2; i>=0; i-=2) {
            /*_I*/{_prof.prof.cnt[6767]++;/*I_*/if (listeners[i]==TreeModelListener.class) {
                // Lazily create the event:
                /*_I*/{_prof.prof.cnt[6768]++;/*I_*/if (e == null)
                    /*_I*/{_prof.prof.cnt[6769]++;/*I_*/e = new TreeModelEvent(source, path, childIndices, children);/*_I*/}/*I_*//*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6770]++;/*I_*/((TreeModelListener)listeners[i+1]).treeStructureChanged(e);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/          
        }/*_I*/}/*I_*/
    }
}








//     private void setFont(NodeModel node, String font) {
// 	node.setFont(font);
// 	nodeStructureChanged(node);
//     }

//     private void setFontSize(NodeModel node, int fontSize) {
// 	node.setFontSize(fontSize);
// 	nodeStructureChanged(node);
//     }

//     private void setUnderlined(NodeModel node) {
// 	if (isUnderlined(node)) {
// 	    node.setUnderlined(false);
// 	} else {
// 	    node.setUnderlined(true);
// 	}
// 	nodeChanged(node);
//     }

//     private void setNormalFont(NodeModel node) {
// 	node.setItalic(false);
// 	node.setBold(false);
// 	node.setUnderlined(false);
// 	nodeChanged(node);
//     }

//     private void setBold(NodeModel node) {
// 	if (isBold(node)) {
// 	    node.setBold(false);
// 	} else {
// 	    node.setBold(true);
// 	}
// 	nodeChanged(node);
//     }

//     private void setItalic(NodeModel node) {
// 	if (isItalic(node)) {
// 	    node.setItalic(false);
// 	} else {
// 	    node.setItalic(true);
// 	}
// 	nodeChanged(node);
//     }

//     private void setEdgeStyle(NodeModel node, String style) {
// 	EdgeModel edge = (EdgeModel)node.getEdge();
// 	edge.setStyle(style);
// 	nodeStructureChanged(node);
//     }

//     private void setNodeStyle(NodeModel node, String style) {
// 	node.setStyle(style);
// 	nodeStructureChanged(node);
//     }

//     private void setNodeColor(NodeModel node, Color color) {
// 	node.setColor(color);
// 	nodeChanged(node);
//     }

//     private void setEdgeColor(NodeModel node, Color color) {
// 	((EdgeModel)node.getEdge()).setColor(color);
// 	nodeChanged(node);
//     }
