/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: NodeAdapter.java,v 1.20 2003/12/17 21:04:53 christianfoltin Exp $*/

package freemind.modes;

import freemind.modes.MindIcon;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.view.mindmapview.NodeView;
import java.util.*;
import java.awt.Color;
import java.awt.Font;
import javax.swing.tree.TreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import java.util.Vector;

/**
 * This class represents a single Node of a Tree. It contains direct handles 
 * to its parent and children and to its view.
 */
public abstract class NodeAdapter implements MindMapNode {
	
    protected Object userObject = "no text";
    private String link = null; //Change this to vector in future for full graph support

    //these Attributes have default values, so it can be useful to directly access them in
    //the save() method instead of using getXXX(). This way the stored file is smaller and looks better.
    //(if the default is used, it is not stored) Look at mindmapmode for an example.
    protected String style;
    /**stores the icons associated with this node.*/
    protected Vector/*<MindIcon>*/ icons = new Vector();
//     /**stores the label associated with this node:*/
//     protected String mLabel;
    /** parameters of an eventually associated cloud*/
    protected MindMapCloud cloud;

    protected Color color;
    protected boolean folded;
    private Tools.BooleanHolder left;

    protected List children;
    private MindMapNode preferredChild; 

    protected Font font;
    protected boolean underlined = false;

    private MindMapNode parent;
    private MindMapEdge edge;//the edge which leads to this node, only root has none
    //In future it has to hold more than one view, maybe with a Vector in which the index specifies
    //the MapView which contains the NodeViews
    private NodeView viewer = null;
    private FreeMindMain frame;
    private static final boolean ALLOWSCHILDREN = true;
    private static final boolean ISLEAF = false; //all nodes may have children

    //
    // Constructors
    //

    protected NodeAdapter(FreeMindMain frame) {/*_I*/_prof.prof.cnt[6209]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6210]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
    }

    protected NodeAdapter(Object userObject, FreeMindMain frame) {/*_I*/_prof.prof.cnt[6211]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6212]++;/*I_*/this.userObject = userObject;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6213]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/
    }

    public String getLink() {/*_I*/_prof.prof.cnt[6214]++;/*I_*/
 	/*_I*/{_prof.prof.cnt[6215]++;/*I_*/return link;/*_I*/}/*I_*/
    }
    
    public void setLink(String link) {/*_I*/_prof.prof.cnt[6216]++;/*I_*/
 	/*_I*/{_prof.prof.cnt[6217]++;/*I_*/this.link = link;/*_I*/}/*I_*/
    }

    public FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[6218]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6219]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

    //
    // Interface MindMapNode
    //

    //
    // get/set methods
    //

    public NodeView getViewer() {/*_I*/_prof.prof.cnt[6220]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6221]++;/*I_*/return viewer;/*_I*/}/*I_*/
    }

    public void setViewer( NodeView viewer ) {/*_I*/_prof.prof.cnt[6222]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6223]++;/*I_*/this.viewer = viewer;/*_I*/}/*I_*/
    }

    /** Creates the TreePath recursively */
    public TreePath getPath() {/*_I*/_prof.prof.cnt[6224]++;/*I_*/
	/*_I*/_prof.prof.cnt[6225]++;/*I_*/Vector pathVector = new Vector();
	/*_I*/_prof.prof.cnt[6226]++;/*I_*/TreePath treePath;
	/*_I*/{_prof.prof.cnt[6227]++;/*I_*/this.addToPathVector( pathVector );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6228]++;/*I_*/treePath = new TreePath( pathVector.toArray() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6229]++;/*I_*/return treePath;/*_I*/}/*I_*/
    }

    public MindMapEdge getEdge() {/*_I*/_prof.prof.cnt[6230]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6231]++;/*I_*/return edge;/*_I*/}/*I_*/
    }

    public void setEdge( MindMapEdge edge ) {/*_I*/_prof.prof.cnt[6232]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6233]++;/*I_*/this.edge = edge;/*_I*/}/*I_*/
    }

    public MindMapCloud getCloud() {/*_I*/_prof.prof.cnt[6234]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6235]++;/*I_*/return cloud;/*_I*/}/*I_*/
    }

    public void setCloud( MindMapCloud cloud ) {/*_I*/_prof.prof.cnt[6236]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6237]++;/*I_*/this.cloud = cloud;/*_I*/}/*I_*/
    }

    /**A Node-Style like MindMapNode.STYLE_FORK or MindMapNode.STYLE_BUBBLE*/
    public String getStyle() {/*_I*/_prof.prof.cnt[6238]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6239]++;/*I_*/if (style==null) {
	    /*_I*/{_prof.prof.cnt[6240]++;/*I_*/if(this.isRoot()) {
		/*_I*/{_prof.prof.cnt[6241]++;/*I_*/return getFrame().getProperty("standardnodestyle");/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[6242]++;/*I_*/return getParentNode().getStyle();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6243]++;/*I_*/return style;/*_I*/}/*I_*/
    }


    /**The Foreground/Font Color*/
    public Color getColor() {/*_I*/_prof.prof.cnt[6244]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6245]++;/*I_*/return color;/*_I*/}/*I_*/ }

    //////
    // The set methods. I'm not sure if they should be here or in the implementing class.
    /////

    public void setStyle(String style) {/*_I*/_prof.prof.cnt[6246]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6247]++;/*I_*/this.style = style;/*_I*/}/*I_*/
    }

    public void setColor(Color color) {/*_I*/_prof.prof.cnt[6248]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6249]++;/*I_*/this.color = color;/*_I*/}/*I_*/
    }

    //   
    //  font handling
    // 

    
    //   Remark to setBold and setItalic implemetation
    //
    // Using deriveFont() is a bad idea, because it does not really choose
    // the appropriate face. For example, instead of choosing face
    // "Arial Bold", it derives the bold face from "Arial".

    // Node holds font only in the case that the font is not default.

    public void estabilishOwnFont() {/*_I*/_prof.prof.cnt[6250]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6251]++;/*I_*/font = (font != null) ? font : getFrame().getController().getDefaultFont();/*_I*/}/*I_*/ }

    public void setBold(boolean bold) {/*_I*/_prof.prof.cnt[6252]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6253]++;/*I_*/if (bold != isBold()) {
           /*_I*/{_prof.prof.cnt[6254]++;/*I_*/toggleBold();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void toggleBold() {/*_I*/_prof.prof.cnt[6255]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6256]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6257]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle() ^ Font.BOLD, font.getSize())));/*_I*/}/*I_*/ }

    public void setItalic(boolean italic) {/*_I*/_prof.prof.cnt[6258]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6259]++;/*I_*/if (italic != isItalic()) {
           /*_I*/{_prof.prof.cnt[6260]++;/*I_*/toggleItalic();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void toggleItalic() {/*_I*/_prof.prof.cnt[6261]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6262]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6263]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle() ^ Font.ITALIC, font.getSize())));/*_I*/}/*I_*/ }

    public void setUnderlined(boolean underlined) {/*_I*/_prof.prof.cnt[6264]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6265]++;/*I_*/this.underlined = underlined;/*_I*/}/*I_*/ }

    public void setFont(Font font) {/*_I*/_prof.prof.cnt[6266]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6267]++;/*I_*/this.font = font;/*_I*/}/*I_*/ }

    public MindMapNode getParentNode() {/*_I*/_prof.prof.cnt[6268]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6269]++;/*I_*/return parent;/*_I*/}/*I_*/ }

    public void setFontSize(int fontSize) {/*_I*/_prof.prof.cnt[6270]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6271]++;/*I_*/estabilishOwnFont();/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6272]++;/*I_*/setFont(getFrame().getController().getFontThroughMap
               (new Font(font.getFamily(), font.getStyle(), fontSize)));/*_I*/}/*I_*/ }

    public Font getFont() {/*_I*/_prof.prof.cnt[6273]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6274]++;/*I_*/return font;/*_I*/}/*I_*/ }

    public String getFontSize(){/*_I*/_prof.prof.cnt[6275]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6276]++;/*I_*/if (getFont() != null){
          /*_I*/{_prof.prof.cnt[6277]++;/*I_*/return new Integer (getFont().getSize()).toString();/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[6278]++;/*I_*/return getFrame().getProperty("defaultfontsize");/*_I*/}/*I_*/}/*_I*/}/*I_*/
    }

    public String getFontFamilyName(){/*_I*/_prof.prof.cnt[6279]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6280]++;/*I_*/if (getFont() != null){
          /*_I*/{_prof.prof.cnt[6281]++;/*I_*/return getFont().getFamily();/*_I*/}/*I_*/}
       else {
          /*_I*/{_prof.prof.cnt[6282]++;/*I_*/return getFrame().getProperty("defaultfont");/*_I*/}/*I_*/}/*_I*/}/*I_*/
    }

    public boolean isBold() {/*_I*/_prof.prof.cnt[6283]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6284]++;/*I_*/return font != null ? font.isBold() : false;/*_I*/}/*I_*/ }

    public boolean isItalic() {/*_I*/_prof.prof.cnt[6285]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6286]++;/*I_*/return font != null ? font.isItalic() : false;/*_I*/}/*I_*/ }

    public boolean isUnderlined() {/*_I*/_prof.prof.cnt[6287]++;/*I_*/        // not implemented
       /*_I*/{_prof.prof.cnt[6288]++;/*I_*/return underlined;/*_I*/}/*I_*/ }

    public boolean isFolded() {/*_I*/_prof.prof.cnt[6289]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6290]++;/*I_*/return folded;/*_I*/}/*I_*/ }

    // fc, 24.9.2003:
    public Vector/*<MindIcon>*/ getIcons() {/*_I*/_prof.prof.cnt[6291]++;/*I_*/ /*_I*/{_prof.prof.cnt[6292]++;/*I_*/return icons;/*_I*/}/*I_*/};

    public void   addIcon(MindIcon _icon) {/*_I*/_prof.prof.cnt[6293]++;/*I_*/ /*_I*/{_prof.prof.cnt[6294]++;/*I_*/icons.add(_icon);/*_I*/}/*I_*/ };

    public int   removeLastIcon() {/*_I*/_prof.prof.cnt[6295]++;/*I_*/ /*_I*/{_prof.prof.cnt[6296]++;/*I_*/if(icons.size() > 0) /*_I*/{_prof.prof.cnt[6297]++;/*I_*/icons.setSize(icons.size()-1);/*_I*/}/*I_*//*_I*/}/*I_*/ /*_I*/{_prof.prof.cnt[6298]++;/*I_*/return icons.size();/*_I*/}/*I_*/};

    // end, fc, 24.9.2003

//     public     String getLabel() { return mLabel; }

//     public     void setLabel(String newLabel) { mLabel = newLabel; /* bad hack: registry fragen.*/ };

//     public Vector/* of NodeLinkStruct*/ getReferences() { return mNodeLinkVector; };

//     public void removeReferenceAt(int i) {
//         if(mNodeLinkVector.size() > i) {
//             mNodeLinkVector.removeElementAt(i);
//         } else {
//             /* exception. */
//         }
//     }
    
//     public     void addReference(MindMapLink mindMapLink) { mNodeLinkVector.add(mindMapLink); };


    /**
     *  True iff one of node's <i>strict</i> descendants is folded. A node N
     *  is not its strict descendant - the fact that node itself is folded
     *  is not sufficient to return true.
     */
    public boolean hasFoldedStrictDescendant() {/*_I*/_prof.prof.cnt[6299]++;/*I_*/
       
       /*_I*/{_prof.prof.cnt[6300]++;/*I_*/for (ListIterator e = childrenUnfolded(); e.hasNext(); ) {
          /*_I*/_prof.prof.cnt[6301]++;/*I_*/NodeAdapter child = (NodeAdapter)e.next();
          /*_I*/{_prof.prof.cnt[6302]++;/*I_*/if (child.isFolded() || child.hasFoldedStrictDescendant()) {
             /*_I*/{_prof.prof.cnt[6303]++;/*I_*/return true;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[6304]++;/*I_*/return false;/*_I*/}/*I_*/
    }

    public void setFolded(boolean folded) {/*_I*/_prof.prof.cnt[6305]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6306]++;/*I_*/this.folded = folded;/*_I*/}/*I_*/
    }

    protected MindMapNode basicCopy() {/*_I*/_prof.prof.cnt[6307]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6308]++;/*I_*/return null;/*_I*/}/*I_*/ }
	
    public MindMapNode shallowCopy() {/*_I*/_prof.prof.cnt[6309]++;/*I_*/
       /*_I*/_prof.prof.cnt[6310]++;/*I_*/MindMapNode copy = basicCopy();
       /*_I*/{_prof.prof.cnt[6311]++;/*I_*/copy.setColor(getColor());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6312]++;/*I_*/copy.setFont(getFont());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6313]++;/*I_*/copy.setLink(getLink());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6314]++;/*I_*/if(isLeft() != null)
           /*_I*/{_prof.prof.cnt[6315]++;/*I_*/copy.setLeft(isLeft().getValue());/*_I*/}/*I_*//*_I*/}/*I_*/
       /*_I*/_prof.prof.cnt[6316]++;/*I_*/Vector icons = getIcons();
       /*_I*/{_prof.prof.cnt[6317]++;/*I_*/for(int i = 0; i < icons.size(); ++i) {
           /*_I*/{_prof.prof.cnt[6318]++;/*I_*/copy.addIcon((MindIcon) icons.get(i));/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6319]++;/*I_*/return copy;/*_I*/}/*I_*/ }

    //
    // other
    //

    public String toString() {/*_I*/_prof.prof.cnt[6320]++;/*I_*/
	/*_I*/_prof.prof.cnt[6321]++;/*I_*/String string = userObject.toString();
// (PN) %%%
// Why? If because of presentation, this level shall be self responsible... 
// Model shall NOT change the real data!!!
//
//	if (string.equals("")) {
//	    string = "   ";
//	}
	/*_I*/{_prof.prof.cnt[6322]++;/*I_*/return string;/*_I*/}/*I_*/
    }

    /**
     * Returns whether the argument is parent
     * or parent of one of the grandpa's of this node.
     * (transitive)
     */
    public boolean isDescendantOf(MindMapNode node) {/*_I*/_prof.prof.cnt[6323]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6324]++;/*I_*/if(this.isRoot())
	    /*_I*/{_prof.prof.cnt[6325]++;/*I_*/return false;/*_I*/}/*I_*/
	else /*_I*/{_prof.prof.cnt[6326]++;/*I_*/if (node == getParentNode())
	    /*_I*/{_prof.prof.cnt[6327]++;/*I_*/return true;/*_I*/}/*I_*/
	else
	    /*_I*/{_prof.prof.cnt[6328]++;/*I_*/return getParentNode().isDescendantOf(node);/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
    }	    
	 
    public boolean isRoot() {/*_I*/_prof.prof.cnt[6329]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6330]++;/*I_*/return (parent==null);/*_I*/}/*I_*/
    }

    public boolean hasChildren() {/*_I*/_prof.prof.cnt[6331]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6332]++;/*I_*/return children != null && !children.isEmpty();/*_I*/}/*I_*/ }

    public int getChildPosition(MindMapNode childNode) {/*_I*/_prof.prof.cnt[6333]++;/*I_*/
       /*_I*/_prof.prof.cnt[6334]++;/*I_*/int position = 0;
       /*_I*/{_prof.prof.cnt[6335]++;/*I_*/for (ListIterator i=children.listIterator(); i.hasNext(); ++position) {
          /*_I*/{_prof.prof.cnt[6336]++;/*I_*/if (((MindMapNode)i.next()) == childNode) {
             /*_I*/{_prof.prof.cnt[6337]++;/*I_*/return position;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6338]++;/*I_*/return -1;/*_I*/}/*I_*/
    }

    public ListIterator childrenUnfolded() {/*_I*/_prof.prof.cnt[6339]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6340]++;/*I_*/return children != null ? children.listIterator() :
           Collections.EMPTY_LIST.listIterator();/*_I*/}/*I_*/ }

    public ListIterator childrenFolded() {/*_I*/_prof.prof.cnt[6341]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6342]++;/*I_*/if (isFolded()) {
          /*_I*/{_prof.prof.cnt[6343]++;/*I_*/return Collections.EMPTY_LIST.listIterator();/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[6344]++;/*I_*/return childrenUnfolded();/*_I*/}/*I_*/
    }

    //
    //  Interface TreeNode
    //

    /**
     * AFAIK there is no way to get an enumeration out of a linked list. So this exception must be
     * thrown, or we can't implement TreeNode anymore (maybe we shouldn't?)
     */
    public Enumeration children() {/*_I*/_prof.prof.cnt[6345]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6346]++;/*I_*/throw new UnsupportedOperationException("Use childrenFolded or childrenUnfolded instead");/*_I*/}/*I_*/
    }
	
    public boolean getAllowsChildren() {/*_I*/_prof.prof.cnt[6347]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6348]++;/*I_*/return ALLOWSCHILDREN;/*_I*/}/*I_*/
    }
	
    public TreeNode getChildAt( int childIndex ) {/*_I*/_prof.prof.cnt[6349]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6350]++;/*I_*/if (isFolded()) {
	    /*_I*/{_prof.prof.cnt[6351]++;/*I_*/return null;/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[6352]++;/*I_*/return (TreeNode)children.get( childIndex );/*_I*/}/*I_*/
    }

    public int getChildCount() {/*_I*/_prof.prof.cnt[6353]++;/*I_*/
       /*_I*/{_prof.prof.cnt[6354]++;/*I_*/return children == null ? 0 : children.size();/*_I*/}/*I_*/
    }

// (PN)
//    public int getChildCount() {
//	if (isFolded()) {
//	    return 0;
//	}
//	return children.size();
//    }
//    // Daniel: ^ The name of this method is confusing. It does nto convey
//    // the meaning, at least not to me.

    public int getIndex( TreeNode node ) {/*_I*/_prof.prof.cnt[6355]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6356]++;/*I_*/return children.indexOf( (MindMapNode)node );/*_I*/}/*I_*/ //uses equals()
    } 

    public TreeNode getParent() {/*_I*/_prof.prof.cnt[6357]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6358]++;/*I_*/return parent;/*_I*/}/*I_*/
    }

    public boolean isLeaf() {/*_I*/_prof.prof.cnt[6359]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6360]++;/*I_*/return getChildCount() == 0;/*_I*/}/*I_*/
    }

    // fc, 16.12.2003 left-right bug:
    public Tools.BooleanHolder isLeft() {/*_I*/_prof.prof.cnt[6361]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6362]++;/*I_*/return left;/*_I*/}/*I_*/
    }
    
    public void setLeft(boolean isLeft){/*_I*/_prof.prof.cnt[6363]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6364]++;/*I_*/if(left == null)
            /*_I*/{_prof.prof.cnt[6365]++;/*I_*/left = new Tools.BooleanHolder();/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6366]++;/*I_*/left.setValue(isLeft);/*_I*/}/*I_*/
    }

    //
    //  Interface MutableTreeNode
    //
    
    //do all remove methods have to work recursively to make the 
    //Garbage Collection work (Nodes in removed Sub-Trees reference each other)?
    
    public void insert( MutableTreeNode child, int index) {/*_I*/_prof.prof.cnt[6367]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6368]++;/*I_*/if (index < 0) { // add to the end (used in xml load) (PN) 
          /*_I*/{_prof.prof.cnt[6369]++;/*I_*/index = getChildCount();/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6370]++;/*I_*/children.add( index, child );/*_I*/}/*I_*/
        }
        else {           // mind preferred child :-)
          /*_I*/{_prof.prof.cnt[6371]++;/*I_*/children.add( index, child );/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[6372]++;/*I_*/preferredChild = (MindMapNode)child;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[6373]++;/*I_*/child.setParent( this );/*_I*/}/*I_*/
    }
    
    
    public void remove( int index ) {/*_I*/_prof.prof.cnt[6374]++;/*I_*/
        /*_I*/_prof.prof.cnt[6375]++;/*I_*/MutableTreeNode node = (MutableTreeNode)children.get(index);
        /*_I*/{_prof.prof.cnt[6376]++;/*I_*/if (node == this.preferredChild) { // mind preferred child :-) (PN) 
          /*_I*/{_prof.prof.cnt[6377]++;/*I_*/if (children.size() > index + 1) {
            /*_I*/{_prof.prof.cnt[6378]++;/*I_*/this.preferredChild = (MindMapNode)(children.get(index + 1));/*_I*/}/*I_*/
          }
          else {
            /*_I*/{_prof.prof.cnt[6379]++;/*I_*/this.preferredChild = (index > 0) ? 
                (MindMapNode)(children.get(index - 1)) : null;/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6380]++;/*I_*/node.setParent(null);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6381]++;/*I_*/children.remove( index );/*_I*/}/*I_*/
    }
    
    public void remove( MutableTreeNode node ) {/*_I*/_prof.prof.cnt[6382]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6383]++;/*I_*/if (node == this.preferredChild) { // mind preferred child :-) (PN) 
          /*_I*/_prof.prof.cnt[6384]++;/*I_*/int index = children.indexOf(node);
          /*_I*/{_prof.prof.cnt[6385]++;/*I_*/if (children.size() > index + 1) {
            /*_I*/{_prof.prof.cnt[6386]++;/*I_*/this.preferredChild = (MindMapNode)(children.get(index + 1));/*_I*/}/*I_*/
          }
          else {
            /*_I*/{_prof.prof.cnt[6387]++;/*I_*/this.preferredChild = (index > 0) ? 
                (MindMapNode)(children.get(index - 1)) : null;/*_I*/}/*I_*/
          }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6388]++;/*I_*/node.setParent(null);/*_I*/}/*I_*/
    	/*_I*/{_prof.prof.cnt[6389]++;/*I_*/children.remove( node );/*_I*/}/*I_*/
    }
    
    public MindMapNode getPreferredChild() {/*_I*/_prof.prof.cnt[6390]++;/*I_*/ // mind preferred child :-) (PN) 
      /*_I*/{_prof.prof.cnt[6391]++;/*I_*/if (this.children.contains(this.preferredChild)) {
        /*_I*/{_prof.prof.cnt[6392]++;/*I_*/return this.preferredChild;/*_I*/}/*I_*/
      }
      else /*_I*/{_prof.prof.cnt[6393]++;/*I_*/if (!isLeaf()) {
        /*_I*/{_prof.prof.cnt[6394]++;/*I_*/return (MindMapNode)(this.children.get((getChildCount() + 1) / 2 - 1));/*_I*/}/*I_*/
      }
      else {
        /*_I*/{_prof.prof.cnt[6395]++;/*I_*/return null;/*_I*/}/*I_*/
      }/*_I*/}/*I_*//*_I*/}/*I_*/
    }
    public void setPreferredChild(MindMapNode node) {/*_I*/_prof.prof.cnt[6396]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6397]++;/*I_*/this.preferredChild = node;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6398]++;/*I_*/if (node == null) {
        /*_I*/{_prof.prof.cnt[6399]++;/*I_*/return;/*_I*/}/*I_*/
      }
      else /*_I*/{_prof.prof.cnt[6400]++;/*I_*/if (this.parent != null) {
        // set also preffered child of parents...
        /*_I*/{_prof.prof.cnt[6401]++;/*I_*/this.parent.setPreferredChild(this);/*_I*/}/*I_*/
      }/*_I*/}/*I_*//*_I*/}/*I_*/
    }

    public void removeFromParent() {/*_I*/_prof.prof.cnt[6402]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[6403]++;/*I_*/parent.remove( this );/*_I*/}/*I_*/
    }

    public void setParent( MutableTreeNode newParent ) {/*_I*/_prof.prof.cnt[6404]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6405]++;/*I_*/parent = (MindMapNode) newParent;/*_I*/}/*I_*/
    }

    public void setParent( MindMapNode newParent ) {/*_I*/_prof.prof.cnt[6406]++;/*I_*/
    	/*_I*/{_prof.prof.cnt[6407]++;/*I_*/parent = newParent;/*_I*/}/*I_*/
    }

    public void setUserObject( Object object ) {/*_I*/_prof.prof.cnt[6408]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6409]++;/*I_*/userObject = object;/*_I*/}/*I_*/
    }

    ////////////////
    // Private methods. Internal Implementation
    //////////////

    /** Recursive Method for getPath() */
    private void addToPathVector( Vector pathVector ) {/*_I*/_prof.prof.cnt[6410]++;/*I_*/
	/*_I*/{_prof.prof.cnt[6411]++;/*I_*/pathVector.add( 0, this );/*_I*/}/*I_*/ //Add myself to beginning of Vector
	/*_I*/{_prof.prof.cnt[6412]++;/*I_*/if ( parent != null ) {
	    /*_I*/{_prof.prof.cnt[6413]++;/*I_*/( (NodeAdapter)parent ).addToPathVector( pathVector );/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    public int getNodeLevel() {/*_I*/_prof.prof.cnt[6414]++;/*I_*/  //for cursor navigation within a level (PN)  
      /*_I*/_prof.prof.cnt[6415]++;/*I_*/int level = 0;
      /*_I*/_prof.prof.cnt[6416]++;/*I_*/MindMapNode parent;
      /*_I*/{_prof.prof.cnt[6417]++;/*I_*/for (parent = this; !parent.isRoot(); parent = parent.getParentNode()) {
        /*_I*/{_prof.prof.cnt[6418]++;/*I_*/level++;/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6419]++;/*I_*/return level;/*_I*/}/*I_*/
    }
}
