/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: XMLElementAdapter.java,v 1.4 2004/01/24 22:36:48 christianfoltin Exp $*/

package freemind.modes;

import freemind.main.XMLElement;
import freemind.main.FreeMindMain;
import freemind.main.Tools;
import freemind.modes.MindIcon;
import freemind.modes.MindMapLinkRegistry;
import freemind.modes.NodeAdapter;

import java.awt.Font;
import java.util.Vector;
import java.util.HashMap;
import java.util.Collection;
import java.util.Iterator;

public abstract class XMLElementAdapter extends XMLElement {

   private Object           userObject = null;
   private FreeMindMain     frame;
   private NodeAdapter      mapChild   = null;

   //   Font attributes

   private String fontName; 
   private int    fontStyle = 0;
   private int    fontSize = 0;

   //   Icon attributes

   private String iconName; 

    // arrow link attributes:
    protected Vector ArrowLinkAdapters;
    protected HashMap /* id -> target */  IDToTarget;

   //   Overhead methods

   public XMLElementAdapter(FreeMindMain frame) {/*_I*/_prof.prof.cnt[6817]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6818]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/ 
      /*_I*/{_prof.prof.cnt[6819]++;/*I_*/this.ArrowLinkAdapters = new Vector();/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6820]++;/*I_*/this.IDToTarget = new HashMap();/*_I*/}/*I_*/
   }

    protected XMLElementAdapter(FreeMindMain frame, Vector ArrowLinkAdapters, HashMap IDToTarget) {/*_I*/_prof.prof.cnt[6821]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6822]++;/*I_*/this.frame = frame;/*_I*/}/*I_*/ 
        /*_I*/{_prof.prof.cnt[6823]++;/*I_*/this.ArrowLinkAdapters = ArrowLinkAdapters;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[6824]++;/*I_*/this.IDToTarget = IDToTarget;/*_I*/}/*I_*/
    }

    /** abstract method to create elements of my type (factory).*/
    abstract protected XMLElement  createAnotherElement();
    abstract protected NodeAdapter createNodeAdapter(FreeMindMain     frame);
    abstract protected EdgeAdapter createEdgeAdapter(NodeAdapter node, FreeMindMain frame);
    abstract protected CloudAdapter createCloudAdapter(NodeAdapter node, FreeMindMain frame);
    abstract protected ArrowLinkAdapter createArrowLinkAdapter(NodeAdapter source, NodeAdapter target, FreeMindMain frame);
    
    protected FreeMindMain getFrame() {/*_I*/_prof.prof.cnt[6825]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6826]++;/*I_*/return frame;/*_I*/}/*I_*/
    }

   public Object getUserObject() {/*_I*/_prof.prof.cnt[6827]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6828]++;/*I_*/return userObject;/*_I*/}/*I_*/ }

   public NodeAdapter getMapChild() {/*_I*/_prof.prof.cnt[6829]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6830]++;/*I_*/return mapChild;/*_I*/}/*I_*/ }

   //   Real parsing methods

   public void setName(String name)  {/*_I*/_prof.prof.cnt[6831]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6832]++;/*I_*/super.setName(name);/*_I*/}/*I_*/
      // Create user object based on name
      /*_I*/{_prof.prof.cnt[6833]++;/*I_*/if (name.equals("node")) {
         /*_I*/{_prof.prof.cnt[6834]++;/*I_*/userObject = createNodeAdapter(frame);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6835]++;/*I_*/if (name.equals("edge")) {
         /*_I*/{_prof.prof.cnt[6836]++;/*I_*/userObject = createEdgeAdapter(null, frame);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6837]++;/*I_*/if (name.equals("cloud")) {
          /*_I*/{_prof.prof.cnt[6838]++;/*I_*/userObject = createCloudAdapter(null, frame);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6839]++;/*I_*/if (name.equals("arrowlink")) {
          /*_I*/{_prof.prof.cnt[6840]++;/*I_*/userObject = createArrowLinkAdapter(null, null, frame);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

   public void addChild(XMLElement child) {/*_I*/_prof.prof.cnt[6841]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6842]++;/*I_*/if (getName().equals("map")) {
         /*_I*/{_prof.prof.cnt[6843]++;/*I_*/mapChild = (NodeAdapter)child.getUserObject();/*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[6844]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[6845]++;/*I_*/if (userObject instanceof NodeAdapter) {
         /*_I*/_prof.prof.cnt[6846]++;/*I_*/NodeAdapter node = (NodeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[6847]++;/*I_*/if (child.getUserObject() instanceof NodeAdapter) {
            /*_I*/{_prof.prof.cnt[6848]++;/*I_*/node.insert((NodeAdapter)child.getUserObject(),
                        -1);/*_I*/}/*I_*/ } // to the end without preferable... (PN)
                     // node.getRealChildCount()); }
         else /*_I*/{_prof.prof.cnt[6849]++;/*I_*/if (child.getUserObject() instanceof EdgeAdapter) {
            /*_I*/_prof.prof.cnt[6850]++;/*I_*/EdgeAdapter edge = (EdgeAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[6851]++;/*I_*/edge.setTarget(node);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6852]++;/*I_*/node.setEdge(edge);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6853]++;/*I_*/if (child.getUserObject() instanceof CloudAdapter) {
            /*_I*/_prof.prof.cnt[6854]++;/*I_*/CloudAdapter cloud = (CloudAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[6855]++;/*I_*/cloud.setTarget(node);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[6856]++;/*I_*/node.setCloud(cloud);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6857]++;/*I_*/if (child.getUserObject() instanceof ArrowLinkAdapter) {
            /*_I*/_prof.prof.cnt[6858]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter)child.getUserObject();
            /*_I*/{_prof.prof.cnt[6859]++;/*I_*/arrowLink.setSource(node);/*_I*/}/*I_*/
            // annotate this link: (later processed by caller.).
            //System.out.println("arrowLink="+arrowLink);
            /*_I*/{_prof.prof.cnt[6860]++;/*I_*/ArrowLinkAdapters.add(arrowLink);/*_I*/}/*I_*/
         }
         else /*_I*/{_prof.prof.cnt[6861]++;/*I_*/if (child.getName().equals("font")) {
            /*_I*/{_prof.prof.cnt[6862]++;/*I_*/node.setFont((Font)child.getUserObject());/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6863]++;/*I_*/if (child.getName().equals("icon")) {
             /*_I*/{_prof.prof.cnt[6864]++;/*I_*/node.addIcon((MindIcon)child.getUserObject());/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*/}

   public void setAttribute(String name, Object value) {/*_I*/_prof.prof.cnt[6865]++;/*I_*/
      // We take advantage of precondition that value != null.
      /*_I*/_prof.prof.cnt[6866]++;/*I_*/String sValue = value.toString();
      /*_I*/{_prof.prof.cnt[6867]++;/*I_*/if (ignoreCase) {
         /*_I*/{_prof.prof.cnt[6868]++;/*I_*/name = name.toUpperCase();/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[6869]++;/*I_*/if (userObject instanceof NodeAdapter) {
         // 
         /*_I*/_prof.prof.cnt[6870]++;/*I_*/NodeAdapter node = (NodeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[6871]++;/*I_*/if (name.equals("TEXT")) {
            /*_I*/{_prof.prof.cnt[6872]++;/*I_*/node.setUserObject(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6873]++;/*I_*/if (name.equals("FOLDED")) {
            /*_I*/{_prof.prof.cnt[6874]++;/*I_*/if (sValue.equals("true")) {
               /*_I*/{_prof.prof.cnt[6875]++;/*I_*/node.setFolded(true);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
         else /*_I*/{_prof.prof.cnt[6876]++;/*I_*/if (name.equals("POSITION")) {
             // fc, 17.12.2003: Remove the left/right bug.
             /*_I*/{_prof.prof.cnt[6877]++;/*I_*/node.setLeft(sValue.equals("left"));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6878]++;/*I_*/if (name.equals("COLOR")) {
            /*_I*/{_prof.prof.cnt[6879]++;/*I_*/if (sValue.length() == 7) {
               /*_I*/{_prof.prof.cnt[6880]++;/*I_*/node.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
         else /*_I*/{_prof.prof.cnt[6881]++;/*I_*/if (name.equals("LINK")) {
            /*_I*/{_prof.prof.cnt[6882]++;/*I_*/node.setLink(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6883]++;/*I_*/if (name.equals("STYLE")) {
            /*_I*/{_prof.prof.cnt[6884]++;/*I_*/node.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6885]++;/*I_*/if (name.equals("ID")) {
             // do not set label but annotate in list:
             //System.out.println("(sValue, node) = " + sValue + ", "+  node);
             /*_I*/{_prof.prof.cnt[6886]++;/*I_*/IDToTarget.put(sValue, node);/*_I*/}/*I_*/
         }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[6887]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[6888]++;/*I_*/if (userObject instanceof EdgeAdapter) {
         /*_I*/_prof.prof.cnt[6889]++;/*I_*/EdgeAdapter edge = (EdgeAdapter)userObject;
         /*_I*/{_prof.prof.cnt[6890]++;/*I_*/if (name.equals("STYLE")) {
	    /*_I*/{_prof.prof.cnt[6891]++;/*I_*/edge.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6892]++;/*I_*/if (name.equals("COLOR")) {
	    /*_I*/{_prof.prof.cnt[6893]++;/*I_*/edge.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6894]++;/*I_*/if (name.equals("WIDTH")) {
            /*_I*/{_prof.prof.cnt[6895]++;/*I_*/if (sValue.equals("thin")) {
               /*_I*/{_prof.prof.cnt[6896]++;/*I_*/edge.setWidth(EdgeAdapter.WIDTH_THIN);/*_I*/}/*I_*/ }
            else {
               /*_I*/{_prof.prof.cnt[6897]++;/*I_*/edge.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[6898]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[6899]++;/*I_*/if (userObject instanceof CloudAdapter) {
         /*_I*/_prof.prof.cnt[6900]++;/*I_*/CloudAdapter cloud = (CloudAdapter)userObject;
         /*_I*/{_prof.prof.cnt[6901]++;/*I_*/if (name.equals("STYLE")) {
	    /*_I*/{_prof.prof.cnt[6902]++;/*I_*/cloud.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6903]++;/*I_*/if (name.equals("COLOR")) {
	    /*_I*/{_prof.prof.cnt[6904]++;/*I_*/cloud.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6905]++;/*I_*/if (name.equals("WIDTH")) {
               /*_I*/{_prof.prof.cnt[6906]++;/*I_*/cloud.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ 
         }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[6907]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[6908]++;/*I_*/if (userObject instanceof ArrowLinkAdapter) {
         /*_I*/_prof.prof.cnt[6909]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter)userObject;
         /*_I*/{_prof.prof.cnt[6910]++;/*I_*/if (name.equals("STYLE")) {
             /*_I*/{_prof.prof.cnt[6911]++;/*I_*/arrowLink.setStyle(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6912]++;/*I_*/if (name.equals("COLOR")) {
             /*_I*/{_prof.prof.cnt[6913]++;/*I_*/arrowLink.setColor(Tools.xmlToColor(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6914]++;/*I_*/if (name.equals("DESTINATION")) {
             /*_I*/{_prof.prof.cnt[6915]++;/*I_*/arrowLink.setDestinationLabel(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6916]++;/*I_*/if (name.equals("REFERENCETEXT")) {
             /*_I*/{_prof.prof.cnt[6917]++;/*I_*/arrowLink.setReferenceText((sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6918]++;/*I_*/if (name.equals("STARTINCLINATION")) {
             /*_I*/{_prof.prof.cnt[6919]++;/*I_*/arrowLink.setStartInclination(Tools.xmlToPoint(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6920]++;/*I_*/if (name.equals("ENDINCLINATION")) {
             /*_I*/{_prof.prof.cnt[6921]++;/*I_*/arrowLink.setEndInclination(Tools.xmlToPoint(sValue));/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6922]++;/*I_*/if (name.equals("STARTARROW")) {
             /*_I*/{_prof.prof.cnt[6923]++;/*I_*/arrowLink.setStartArrow(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6924]++;/*I_*/if (name.equals("ENDARROW")) {
             /*_I*/{_prof.prof.cnt[6925]++;/*I_*/arrowLink.setEndArrow(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6926]++;/*I_*/if (name.equals("WIDTH")) {
             /*_I*/{_prof.prof.cnt[6927]++;/*I_*/arrowLink.setWidth(Integer.parseInt(sValue));/*_I*/}/*I_*/ 
         }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
         /*_I*/{_prof.prof.cnt[6928]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

      /*_I*/{_prof.prof.cnt[6929]++;/*I_*/if (getName().equals("font")) {
         /*_I*/{_prof.prof.cnt[6930]++;/*I_*/if (name.equals("SIZE")) {
            /*_I*/{_prof.prof.cnt[6931]++;/*I_*/fontSize = Integer.parseInt(sValue);/*_I*/}/*I_*/ }
         else /*_I*/{_prof.prof.cnt[6932]++;/*I_*/if (name.equals("NAME")) {
            /*_I*/{_prof.prof.cnt[6933]++;/*I_*/fontName = sValue;/*_I*/}/*I_*/ }             

         // Styling
         else /*_I*/{_prof.prof.cnt[6934]++;/*I_*/if (sValue.equals("true")) {
            /*_I*/{_prof.prof.cnt[6935]++;/*I_*/if (name.equals("BOLD")) {
               /*_I*/{_prof.prof.cnt[6936]++;/*I_*/fontStyle+=Font.BOLD;/*_I*/}/*I_*/ }
            else /*_I*/{_prof.prof.cnt[6937]++;/*I_*/if (name.equals("ITALIC")) {
               /*_I*/{_prof.prof.cnt[6938]++;/*I_*/fontStyle+=Font.ITALIC;/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/}/*_I*/}/*I_*/
      /* icons */
      /*_I*/{_prof.prof.cnt[6939]++;/*I_*/if (getName().equals("icon")) {
         /*_I*/{_prof.prof.cnt[6940]++;/*I_*/if (name.equals("BUILTIN")) {
            /*_I*/{_prof.prof.cnt[6941]++;/*I_*/iconName = sValue;/*_I*/}/*I_*/ }/*_I*/}/*I_*/ 
      }/*_I*/}/*I_*/
  }

   protected void completeElement() {/*_I*/_prof.prof.cnt[6942]++;/*I_*/
      /*_I*/{_prof.prof.cnt[6943]++;/*I_*/if (getName().equals("font")) {
         /*_I*/{_prof.prof.cnt[6944]++;/*I_*/userObject =  frame.getController().getFontThroughMap
            (new Font(fontName, fontStyle, fontSize));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
      /* icons */
            /*_I*/{_prof.prof.cnt[6945]++;/*I_*/if (getName().equals("icon")) {
         /*_I*/{_prof.prof.cnt[6946]++;/*I_*/userObject =  new MindIcon(iconName);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
   }

    /** Completes the links within the map. They are registered in the registry.*/
    public void processUnfinishedLinks(MindMapLinkRegistry registry) {/*_I*/_prof.prof.cnt[6947]++;/*I_*/
        // add labels to the nodes:
        /*_I*/{_prof.prof.cnt[6948]++;/*I_*/setIDs(IDToTarget, registry);/*_I*/}/*I_*/
        // complete arrow links with right labels:
        /*_I*/{_prof.prof.cnt[6949]++;/*I_*/for(int i = 0; i < ArrowLinkAdapters.size(); ++i) {
            /*_I*/_prof.prof.cnt[6950]++;/*I_*/ArrowLinkAdapter arrowLink = (ArrowLinkAdapter) ArrowLinkAdapters.get(i);
            /*_I*/_prof.prof.cnt[6951]++;/*I_*/String oldID = arrowLink.getDestinationLabel();
            /*_I*/_prof.prof.cnt[6952]++;/*I_*/NodeAdapter target = null;
            /*_I*/_prof.prof.cnt[6953]++;/*I_*/String newID = null;
            // find oldID in target list:
            /*_I*/{_prof.prof.cnt[6954]++;/*I_*/if(IDToTarget.containsKey(oldID)) {
                // link present in the xml text
                /*_I*/{_prof.prof.cnt[6955]++;/*I_*/target = (NodeAdapter) IDToTarget.get(oldID);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6956]++;/*I_*/newID = registry.getLabel(target);/*_I*/}/*I_*/
            } else /*_I*/{_prof.prof.cnt[6957]++;/*I_*/if(registry.getTargetForID(oldID) != null) {
                // link is already present in the map (paste).
                /*_I*/{_prof.prof.cnt[6958]++;/*I_*/target = (NodeAdapter) registry.getTargetForID(oldID);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6959]++;/*I_*/if(target == null) {
                    // link target is in nowhere-land
                    /*_I*/{_prof.prof.cnt[6960]++;/*I_*/System.err.println("Cannot find the label " + oldID + " in the map. The link "+arrowLink+" is not restored.");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6961]++;/*I_*/continue;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6962]++;/*I_*/newID = registry.getLabel(target);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6963]++;/*I_*/if( ! newID.equals(oldID) ) {
                    /*_I*/{_prof.prof.cnt[6964]++;/*I_*/System.err.println("Servere internal error. Looked for id " + oldID + " but found "+newID + " in the node " + target+".");/*_I*/}/*I_*/
                    /*_I*/{_prof.prof.cnt[6965]++;/*I_*/continue;/*_I*/}/*I_*/
                }/*_I*/}/*I_*/
            } else {                
                // link target is in nowhere-land
                /*_I*/{_prof.prof.cnt[6966]++;/*I_*/System.err.println("Cannot find the label " + oldID + " in the map. The link "+arrowLink+" is not restored.");/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6967]++;/*I_*/continue;/*_I*/}/*I_*/
            }/*_I*/}/*I_*//*_I*/}/*I_*/
            // set the new ID:
            /*_I*/{_prof.prof.cnt[6968]++;/*I_*/arrowLink.setDestinationLabel(newID);/*_I*/}/*I_*/
            // set the target:
            /*_I*/{_prof.prof.cnt[6969]++;/*I_*/arrowLink.setTarget(target);/*_I*/}/*I_*/
            // add the arrowLink:
            //System.out.println("Node = " + target+ ", oldID="+oldID+", newID="+newID);
            /*_I*/{_prof.prof.cnt[6970]++;/*I_*/registry.registerLink(arrowLink);/*_I*/}/*I_*/

        }/*_I*/}/*I_*/
    }


    /**Recursive method to set the ids of the nodes.*/
    private void setIDs(HashMap IDToTarget, MindMapLinkRegistry registry) {/*_I*/_prof.prof.cnt[6971]++;/*I_*/
        /*_I*/{_prof.prof.cnt[6972]++;/*I_*/for(Iterator i = IDToTarget.keySet().iterator(); i.hasNext();) {
            /*_I*/_prof.prof.cnt[6973]++;/*I_*/String key = (String) i.next();
            /*_I*/_prof.prof.cnt[6974]++;/*I_*/NodeAdapter target = (NodeAdapter) IDToTarget.get(key);
            /*_I*/_prof.prof.cnt[6975]++;/*I_*/MindMapLinkRegistry.ID_Registered newState = registry.registerLinkTarget(target, key /* Proposed name for the target, is changed by the registry, if already present.*/);
            /*_I*/_prof.prof.cnt[6976]++;/*I_*/String newId = newState.getID();
            // and in the cutted case: 
            // search for links to this ids that have been cutted earlier:
            /*_I*/_prof.prof.cnt[6977]++;/*I_*/Vector cuttedLinks = registry.getCuttedNode(key /* old target id*/);
            /*_I*/{_prof.prof.cnt[6978]++;/*I_*/for(int j=0; j < cuttedLinks.size(); ++j) {
                /*_I*/_prof.prof.cnt[6979]++;/*I_*/ArrowLinkAdapter link = (ArrowLinkAdapter) cuttedLinks.get(j);
                // repair link
                /*_I*/{_prof.prof.cnt[6980]++;/*I_*/link.setTarget(target);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[6981]++;/*I_*/link.setDestinationLabel(newId);/*_I*/}/*I_*/
                // and set it:
                /*_I*/{_prof.prof.cnt[6982]++;/*I_*/registry.registerLink(link);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }


}
