/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: BrowseController.java,v 1.13 2003/12/22 11:15:18 christianfoltin Exp $*/

package freemind.modes.browsemode;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;

import java.util.HashSet;
import java.util.Vector;

import freemind.main.Tools;
import freemind.modes.ControllerAdapter;
import freemind.modes.MapAdapter;
import freemind.modes.MindMapNode;
import freemind.modes.Mode;
import freemind.view.mindmapview.NodeView;

public class BrowseController extends ControllerAdapter {

    private JPopupMenu popupmenu;
    private JToolBar toolbar;

    Action toggleFolded;
    Action toggleChildrenFolded;
    Action find;
    Action findNext;
    Action followLink;
    Action nodeUp;
    Action nodeDown;

    // disable edit in browse mode (PN)
    public void edit(KeyEvent e, boolean addNew, boolean editLong) {/*_I*/_prof.prof.cnt[2882]++;/*I_*/ }
    public void addNew(final NodeView target, 
                        final int newNodeMode, 
                        final KeyEvent e) {/*_I*/_prof.prof.cnt[2883]++;/*I_*/ }

    public BrowseController(Mode mode) {/*_I*/_prof.prof.cnt[2884]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2885]++;/*I_*/super.setMode(mode);/*_I*/}/*I_*/

        // Daniel: Actions are initialized here and not above because of
        // some error it would produce. Not studied in more detail.
        /*_I*/{_prof.prof.cnt[2886]++;/*I_*/toggleFolded = new ToggleFoldedAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2887]++;/*I_*/toggleChildrenFolded = new ToggleChildrenFoldedAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2888]++;/*I_*/find = new FindAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2889]++;/*I_*/findNext = new FindNextAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2890]++;/*I_*/followLink = new FollowLinkAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2891]++;/*I_*/nodeUp = new NodeUpAction();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2892]++;/*I_*/nodeDown = new NodeDownAction();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2893]++;/*I_*/popupmenu = new BrowsePopupMenu(this);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2894]++;/*I_*/toolbar = new BrowseToolBar(this);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2895]++;/*I_*/setAllActions(false);/*_I*/}/*I_*/
    }

    public MapAdapter newModel() {/*_I*/_prof.prof.cnt[2896]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2897]++;/*I_*/return new BrowseMapModel(getFrame());/*_I*/}/*I_*/
    }

    public void doubleClick() {/*_I*/_prof.prof.cnt[2898]++;/*I_*/
         /*_I*/{_prof.prof.cnt[2899]++;/*I_*/if (getSelected().getLink() == null) { // If link exists, follow the link; toggle folded otherwise
             /*_I*/{_prof.prof.cnt[2900]++;/*I_*/toggleFolded();/*_I*/}/*I_*/
         } else { 
	        /*_I*/{_prof.prof.cnt[2901]++;/*I_*/loadURL();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    protected MindMapNode newNode() {/*_I*/_prof.prof.cnt[2902]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2903]++;/*I_*/return new BrowseNodeModel(getText("new_node"), getFrame());/*_I*/}/*I_*/
    }

    //get/set methods

    JMenu getEditMenu() {/*_I*/_prof.prof.cnt[2904]++;/*I_*/
	/*_I*/_prof.prof.cnt[2905]++;/*I_*/JMenu editMenu = new JMenu();
	/*_I*/{_prof.prof.cnt[2906]++;/*I_*/editMenu.add(getPopupMenu());/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2907]++;/*I_*/return editMenu;/*_I*/}/*I_*/
    }

    JMenu getFileMenu() {/*_I*/_prof.prof.cnt[2908]++;/*I_*/
	/*_I*/_prof.prof.cnt[2909]++;/*I_*/JMenu fileMenu = new JMenu();
	/*_I*/{_prof.prof.cnt[2910]++;/*I_*/return fileMenu;/*_I*/}/*I_*/
    }

    public JPopupMenu getPopupMenu() {/*_I*/_prof.prof.cnt[2911]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2912]++;/*I_*/return popupmenu;/*_I*/}/*I_*/
    }

    /** Link implementation: If this is a link, we want to make a popup with at least removelink available.*/
    public JPopupMenu getPopupForModel(java.lang.Object obj) {/*_I*/_prof.prof.cnt[2913]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2914]++;/*I_*/if( obj instanceof BrowseArrowLinkModel) {
            // yes, this is a link.
            /*_I*/_prof.prof.cnt[2915]++;/*I_*/BrowseArrowLinkModel link = (BrowseArrowLinkModel) obj;
            /*_I*/_prof.prof.cnt[2916]++;/*I_*/JPopupMenu arrowLinkPopup = new JPopupMenu();

            /*_I*/{_prof.prof.cnt[2917]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(link.getSource().toString(), link.getSource()));/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[2918]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(link.getTarget().toString(), link.getTarget()));/*_I*/}/*I_*/ 

            /*_I*/{_prof.prof.cnt[2919]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            // add all links from target and from source:
            /*_I*/_prof.prof.cnt[2920]++;/*I_*/HashSet NodeAlreadyVisited = new HashSet();
            /*_I*/{_prof.prof.cnt[2921]++;/*I_*/NodeAlreadyVisited.add(link.getSource());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2922]++;/*I_*/NodeAlreadyVisited.add(link.getTarget());/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[2923]++;/*I_*/Vector links = getModel().getLinkRegistry().getAllLinks(link.getSource());
            /*_I*/{_prof.prof.cnt[2924]++;/*I_*/links.addAll(getModel().getLinkRegistry().getAllLinks(link.getTarget()));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2925]++;/*I_*/for(int i = 0; i < links.size(); ++i) {
                /*_I*/_prof.prof.cnt[2926]++;/*I_*/BrowseArrowLinkModel foreign_link = (BrowseArrowLinkModel) links.get(i);
                /*_I*/{_prof.prof.cnt[2927]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getTarget())) {
                    /*_I*/{_prof.prof.cnt[2928]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(foreign_link.getTarget().toString(), foreign_link.getTarget()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[2929]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getSource())) {
                    /*_I*/{_prof.prof.cnt[2930]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(foreign_link.getSource().toString(), foreign_link.getSource()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2931]++;/*I_*/return arrowLinkPopup;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2932]++;/*I_*/return null;/*_I*/}/*I_*/
    }


    //convenience methods
    private BrowseMapModel getModel() {/*_I*/_prof.prof.cnt[2933]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2934]++;/*I_*/return (BrowseMapModel)getController().getModel();/*_I*/}/*I_*/
    }

    private BrowseNodeModel getSelected() {/*_I*/_prof.prof.cnt[2935]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2936]++;/*I_*/return (BrowseNodeModel)getView().getSelected().getModel();/*_I*/}/*I_*/
    }

    BrowseToolBar getToolBar() {/*_I*/_prof.prof.cnt[2937]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2938]++;/*I_*/return (BrowseToolBar)toolbar;/*_I*/}/*I_*/
    }

    public void loadURL(String relative) {/*_I*/_prof.prof.cnt[2939]++;/*I_*/
        /*_I*/_prof.prof.cnt[2940]++;/*I_*/URL absolute = null;
        /*_I*/{_prof.prof.cnt[2941]++;/*I_*/try {
            /*_I*/_prof.prof.cnt[2942]++;/*I_*/BrowseMapModel map = (BrowseMapModel)getMap();
            /*_I*/{_prof.prof.cnt[2943]++;/*I_*/if (map != null) {
                /*_I*/{_prof.prof.cnt[2944]++;/*I_*/absolute = new URL( map.getURL(), relative);/*_I*/}/*I_*/	
            } else {
                /*_I*/{_prof.prof.cnt[2945]++;/*I_*/absolute = new URL( relative );/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            //	    absolute = new URL(relative);
            /*_I*/{_prof.prof.cnt[2946]++;/*I_*/getFrame().out(absolute.toString());/*_I*/}/*I_*/
        } catch (MalformedURLException ex) {
            /*_I*/{_prof.prof.cnt[2947]++;/*I_*/getController().errorMessage(getText("url_error") + " " + ex.getMessage());/*_I*/}/*I_*/
            //getFrame().err(getText("url_error"));
            /*_I*/{_prof.prof.cnt[2948]++;/*I_*/return;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        /*_I*/_prof.prof.cnt[2949]++;/*I_*/String type = Tools.getExtension(absolute.getFile());
        /*_I*/{_prof.prof.cnt[2950]++;/*I_*/try {
            /*_I*/{_prof.prof.cnt[2951]++;/*I_*/if (type.equals("mm")) {
                /*_I*/{_prof.prof.cnt[2952]++;/*I_*/getFrame().setWaitingCursor(true);/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[2953]++;/*I_*/load(absolute);/*_I*/}/*I_*/
            } else {
                /*_I*/{_prof.prof.cnt[2954]++;/*I_*/getFrame().openDocument(absolute);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        } catch (Exception ex) {
            /*_I*/{_prof.prof.cnt[2955]++;/*I_*/getController().errorMessage(getText("url_load_error")+absolute);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2956]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/
            //for some reason, this exception is thrown anytime...
        } finally {
            /*_I*/{_prof.prof.cnt[2957]++;/*I_*/getFrame().setWaitingCursor(false);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        
    }

    public void loadURL() {/*_I*/_prof.prof.cnt[2958]++;/*I_*/
	/*_I*/_prof.prof.cnt[2959]++;/*I_*/String link = getSelected().getLink();
	/*_I*/{_prof.prof.cnt[2960]++;/*I_*/if (link != null) {
	    /*_I*/{_prof.prof.cnt[2961]++;/*I_*/loadURL(link);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    private void load(URL url) throws Exception {/*_I*/_prof.prof.cnt[2962]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2963]++;/*I_*/getToolBar().setURLField(url.toString());/*_I*/}/*I_*/
    	/*_I*/_prof.prof.cnt[2964]++;/*I_*/BrowseMapModel model = (BrowseMapModel)newModel();
	/*_I*/{_prof.prof.cnt[2965]++;/*I_*/model.load(url);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2966]++;/*I_*/newMap(model);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2967]++;/*I_*/mapOpened(true);/*_I*/}/*I_*/
    }




    /**
     * Enabled/Disabled all actions that are dependent on
     * whether there is a map open or not.
     */
    protected void setAllActions(boolean enabled) {/*_I*/_prof.prof.cnt[2968]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2969]++;/*I_*/toggleFolded.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2970]++;/*I_*/toggleChildrenFolded.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2971]++;/*I_*/followLink.setEnabled(enabled);/*_I*/}/*I_*/
    }

    //////////
    // Actions
    /////////

    private class FollowLinkAction extends AbstractAction {
	FollowLinkAction() {
	    super(getText("follow_link"));/*_I*/_prof.prof.cnt[2972]++;/*I_*//*_I*/_prof.prof.cnt[2973]++;/*I_*/
	}
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[2974]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[2975]++;/*I_*/loadURL();/*_I*/}/*I_*/
	}
    }
}
