/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: MindMapController.java,v 1.35 2004/02/06 06:04:25 christianfoltin Exp $*/

package freemind.modes.mindmapmode;

import freemind.main.Tools;
import freemind.main.XMLParseException;
import freemind.modes.MindMapNode;
import freemind.modes.NodeAdapter;
import freemind.modes.Mode;
import freemind.modes.ControllerAdapter;
import freemind.modes.MapAdapter;
import freemind.modes.EdgeAdapter;
import freemind.modes.StylePattern;
import freemind.modes.MindIcon;
import freemind.modes.MindMapCloud;
import freemind.modes.mindmapmode.MindMapArrowLinkModel;
import freemind.view.mindmapview.NodeView;
// link registry.
import freemind.modes.MindMapLinkRegistry;

import java.io.*;
import java.util.*;
import java.util.HashSet;
import java.net.URL;
import java.net.MalformedURLException;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.JRadioButtonMenuItem;





public class MindMapController extends ControllerAdapter {

    //    Mode mode;
    private JPopupMenu popupmenu;
    //private JToolBar toolbar;
    private MindMapToolBar toolbar;
    private boolean addAsChildMode = false;

    Action newMap = new NewMapAction(this);
    Action open = new OpenAction(this);
    Action save = new SaveAction(this);
    Action saveAs = new SaveAsAction(this);
    Action exportToHTML = new ExportToHTMLAction(this);
    Action exportBranchToHTML = new ExportBranchToHTMLAction(this);

    Action edit = new EditAction();
    Action editLong = new EditLongAction();
    Action newChild = new NewChildAction();
    Action newChildWithoutFocus = new NewChildWithoutFocusAction();
    Action newSibling = new NewSiblingAction();
    Action newPreviousSibling = new NewPreviousSiblingAction();
    Action remove = new RemoveAction();
    Action toggleFolded = new ToggleFoldedAction();
    Action toggleChildrenFolded = new ToggleChildrenFoldedAction();
    Action setLinkByFileChooser = new SetLinkByFileChooserAction();
	Action setImageByFileChooser = new SetImageByFileChooserAction();
    Action setLinkByTextField = new SetLinkByTextFieldAction();
    Action followLink = new FollowLinkAction();
    Action exportBranch = new ExportBranchAction();
    Action importBranch = new ImportBranchAction();
    Action importLinkedBranch = new ImportLinkedBranchAction();
    Action importLinkedBranchWithoutRoot = new ImportLinkedBranchWithoutRootAction();
    Action importExplorerFavorites = new ImportExplorerFavoritesAction();
    Action importFolderStructure = new ImportFolderStructureAction();
    Action joinNodes = new JoinNodesAction();
    Action nodeUp = new NodeUpAction();
    Action nodeDown = new NodeDownAction();
    Action find = new FindAction();
    Action findNext = new FindNextAction();

    Action fork = new ForkAction();
    Action bubble = new BubbleAction();
    Action nodeColor = new NodeColorAction();
    Action nodeColorBlend = new NodeGeneralAction ("blend_color", null,
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3269]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3270]++;/*I_*/map.blendNodeColor(node);/*_I*/}/*I_*/ }});

    Action edgeColor = new EdgeColorAction();
    Action edgeWidths[] = {
       new EdgeWidthAction(EdgeAdapter.WIDTH_PARENT),
       new EdgeWidthAction(EdgeAdapter.WIDTH_THIN),
       new EdgeWidthAction(1),
       new EdgeWidthAction(2),
       new EdgeWidthAction(4),
       new EdgeWidthAction(8)
    };
    Action edgeStyles[] = {
       new EdgeStyleAction("linear"),
       new EdgeStyleAction("bezier"),
       new EdgeStyleAction("sharp_linear"),
       new EdgeStyleAction("sharp_bezier")
    };
    Action cloudColor = new CloudColorAction();

    Action italic = new NodeGeneralAction ("italic", "images/Italic24.gif",
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3271]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3272]++;/*I_*/map.setItalic(node);/*_I*/}/*I_*/ }});
    Action bold   = new NodeGeneralAction ("bold", "images/Bold24.gif",
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3273]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3274]++;/*I_*/map.setBold(node);/*_I*/}/*I_*/ }});
    Action cloud   = new NodeGeneralAction ("cloud", "images/Cloud24.gif",
       new SingleNodeOperation() { private MindMapCloud lastCloud;
           private MindMapNodeModel nodeOfLastCloud;
           public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3275]++;/*I_*/
               // store last color to enable if the node is switched on and off.
               /*_I*/{_prof.prof.cnt[3276]++;/*I_*/if(node.getCloud() != null) {
                   /*_I*/{_prof.prof.cnt[3277]++;/*I_*/lastCloud = node.getCloud();/*_I*/}/*I_*/
                   /*_I*/{_prof.prof.cnt[3278]++;/*I_*/nodeOfLastCloud = node;/*_I*/}/*I_*/
               }/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[3279]++;/*I_*/map.setCloud(node);/*_I*/}/*I_*/ 
               // restore color:
               /*_I*/{_prof.prof.cnt[3280]++;/*I_*/if((node.getCloud() != null) && (node == nodeOfLastCloud)) {
                   /*_I*/{_prof.prof.cnt[3281]++;/*I_*/node.setCloud(lastCloud);/*_I*/}/*I_*/
               }/*_I*/}/*I_*/

           }
       });
    Action normalFont = new NodeGeneralAction ("normal", "images/Normal24.gif",
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3282]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3283]++;/*I_*/map.setNormalFont(node);/*_I*/}/*I_*/ }});
    Action increaseNodeFont = new NodeGeneralAction ("increase_node_font_size", null,
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3284]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3285]++;/*I_*/map.increaseFontSize(node,1);/*_I*/}/*I_*/ }});
    Action decreaseNodeFont = new NodeGeneralAction ("decrease_node_font_size", null,
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3286]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3287]++;/*I_*/map.increaseFontSize(node,-1);/*_I*/}/*I_*/ }});

    // Extension Actions
    Action patterns[] = new Action[0]; // Make sure it is initialized
    Vector iconActions = new Vector(); //fc
    Action removeLastIcon = new NodeGeneralAction ("remove_last_icon", "images/remove.png",
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3288]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3289]++;/*I_*/map.removeLastIcon(node);/*_I*/}/*I_*/ }});
    Action removeAllIcons = new NodeGeneralAction ("remove_all_icons", "images/edittrash.png",
       new SingleNodeOperation() { public void apply(MindMapMapModel map, MindMapNodeModel node) {/*_I*/_prof.prof.cnt[3290]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3291]++;/*I_*/while(map.removeLastIcon(node)>0) {}/*_I*/}/*I_*//*_I*/{_prof.prof.cnt[3292]++;/*I_*/;/*_I*/}/*I_*/ }});

    FileFilter filefilter = new MindMapFilter();

    public MindMapController(Mode mode) {
	super(mode);/*_I*/_prof.prof.cnt[3293]++;/*I_*//*_I*/_prof.prof.cnt[3294]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3295]++;/*I_*/try {
           /*_I*/_prof.prof.cnt[3296]++;/*I_*/File patternsFile = getFrame().getPatternsFile();
           /*_I*/{_prof.prof.cnt[3297]++;/*I_*/if (patternsFile != null && patternsFile.exists()) {
              /*_I*/{_prof.prof.cnt[3298]++;/*I_*/loadPatterns(patternsFile);/*_I*/}/*I_*/ }
           else {
              /*_I*/{_prof.prof.cnt[3299]++;/*I_*/System.out.println("User patterns file "+patternsFile+" not found.");/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[3300]++;/*I_*/loadPatterns(new InputStreamReader(getResource("patterns.xml").openStream()));/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
        catch (XMLParseException e) {
           /*_I*/{_prof.prof.cnt[3301]++;/*I_*/System.err.println("In patterns:"+e);/*_I*/}/*I_*/ }
	catch (Exception ex) {
           /*_I*/{_prof.prof.cnt[3302]++;/*I_*/System.err.println("Patterns not loaded:"+ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        // icon actions:
        /*_I*/{_prof.prof.cnt[3303]++;/*I_*/createIconActions();/*_I*/}/*I_*/

      	/*_I*/{_prof.prof.cnt[3304]++;/*I_*/popupmenu = new MindMapPopupMenu(this);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3305]++;/*I_*/toolbar = new MindMapToolBar(this);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3306]++;/*I_*/setAllActions(false);/*_I*/}/*I_*/ 
   
        // addAsChildMode (use old model of handling CtrN) (PN)
        /*_I*/{_prof.prof.cnt[3307]++;/*I_*/addAsChildMode = Tools.safeEquals(
            getFrame().getProperty("add_as_child"),"true");/*_I*/}/*I_*/
    }

    public MapAdapter newModel() {/*_I*/_prof.prof.cnt[3308]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3309]++;/*I_*/return new MindMapMapModel(getFrame());/*_I*/}/*I_*/ }

    private void loadPatterns(File file) throws Exception {/*_I*/_prof.prof.cnt[3310]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3311]++;/*I_*/createPatterns(StylePattern.loadPatterns(file));/*_I*/}/*I_*/ }

    private void loadPatterns(Reader reader) throws Exception {/*_I*/_prof.prof.cnt[3312]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3313]++;/*I_*/createPatterns(StylePattern.loadPatterns(reader));/*_I*/}/*I_*/ }

    private void createPatterns(List patternsList) throws Exception {/*_I*/_prof.prof.cnt[3314]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3315]++;/*I_*/patterns = new Action[patternsList.size()];/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3316]++;/*I_*/for (int i=0;i<patterns.length;i++) {
        /*_I*/{_prof.prof.cnt[3317]++;/*I_*/patterns[i] = new ApplyPatternAction((StylePattern)patternsList.get(i));/*_I*/}/*I_*/

        // search icons for patterns:
        /*_I*/_prof.prof.cnt[3318]++;/*I_*/MindIcon patternIcon = ((StylePattern)patternsList.get(i)).getNodeIcon();
        /*_I*/{_prof.prof.cnt[3319]++;/*I_*/if (patternIcon != null) {
            /*_I*/{_prof.prof.cnt[3320]++;/*I_*/patterns[i].putValue(Action.SMALL_ICON, patternIcon.getIcon(getFrame()));/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }/*_I*/}/*I_*/}

    private void createIconActions() {/*_I*/_prof.prof.cnt[3321]++;/*I_*/
        /*_I*/_prof.prof.cnt[3322]++;/*I_*/Vector iconNames = MindIcon.getAllIconNames();
        /*_I*/{_prof.prof.cnt[3323]++;/*I_*/for ( int i = 0 ; i < iconNames.size(); ++i ) {
            /*_I*/_prof.prof.cnt[3324]++;/*I_*/String iconName = ((String) iconNames.get(i));
            /*_I*/_prof.prof.cnt[3325]++;/*I_*/MindIcon myIcon     = new MindIcon(iconName);
            /*_I*/_prof.prof.cnt[3326]++;/*I_*/Action myAction = new IconAction(myIcon);
            /*_I*/{_prof.prof.cnt[3327]++;/*I_*/iconActions.add(myAction);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }

    public FileFilter getFileFilter() {/*_I*/_prof.prof.cnt[3328]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3329]++;/*I_*/return filefilter;/*_I*/}/*I_*/ }

    //Node editing
    void setFontSize(int fontSize) {/*_I*/_prof.prof.cnt[3330]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3331]++;/*I_*/for(ListIterator e = getSelecteds().listIterator();e.hasNext();) {
           /*_I*/_prof.prof.cnt[3332]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)e.next();
           /*_I*/{_prof.prof.cnt[3333]++;/*I_*/getModel().setFontSize(selected,fontSize);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    void setFontFamily(String fontFamily) {/*_I*/_prof.prof.cnt[3334]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3335]++;/*I_*/for(ListIterator e = getSelecteds().listIterator();e.hasNext();) {
          /*_I*/_prof.prof.cnt[3336]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)e.next();
          /*_I*/{_prof.prof.cnt[3337]++;/*I_*/getModel().setFontFamily(selected,fontFamily);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

    public void nodeChanged(MindMapNode n) {/*_I*/_prof.prof.cnt[3338]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3339]++;/*I_*/toolbar.selectFontSize(((NodeAdapter)n).getFontSize());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3340]++;/*I_*/toolbar.selectFontName(((NodeAdapter)n).getFontFamilyName());/*_I*/}/*I_*/ }

    public void anotherNodeSelected(MindMapNode n) {/*_I*/_prof.prof.cnt[3341]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3342]++;/*I_*/toolbar.selectFontSize(((NodeAdapter)n).getFontSize());/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3343]++;/*I_*/toolbar.selectFontName(((NodeAdapter)n).getFontFamilyName());/*_I*/}/*I_*/ }

    protected MindMapNode newNode() {/*_I*/_prof.prof.cnt[3344]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3345]++;/*I_*/return new MindMapNodeModel("" // getText("new_node") (PN) nicer when created
                                  , getFrame());/*_I*/}/*I_*/ }

	    //get/set methods

    JMenu getEditMenu() {/*_I*/_prof.prof.cnt[3346]++;/*I_*/
	/*_I*/_prof.prof.cnt[3347]++;/*I_*/JMenu editMenu = new JMenu();

        /*_I*/_prof.prof.cnt[3348]++;/*I_*/JMenu leading = getLeadingNodeMenu();
        /*_I*/_prof.prof.cnt[3349]++;/*I_*/Component[] mc = leading.getMenuComponents();
        /*_I*/{_prof.prof.cnt[3350]++;/*I_*/for (int i = 0; i < mc.length; i++) {
           /*_I*/{_prof.prof.cnt[3351]++;/*I_*/editMenu.add(mc[i]);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3352]++;/*I_*/editMenu.addSeparator();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3353]++;/*I_*/editMenu.add(getNodeMenu());/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3354]++;/*I_*/editMenu.add(getBranchMenu());/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3355]++;/*I_*/editMenu.add(getEdgeMenu());/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3356]++;/*I_*/editMenu.add(getExtensionMenu());/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3357]++;/*I_*/editMenu.add(getIconMenu());/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3358]++;/*I_*/return editMenu;/*_I*/}/*I_*/
    }

    JMenu getFileMenu() {/*_I*/_prof.prof.cnt[3359]++;/*I_*/
	/*_I*/_prof.prof.cnt[3360]++;/*I_*/JMenu fileMenu = new JMenu();
	/*_I*/{_prof.prof.cnt[3361]++;/*I_*/add(fileMenu, newMap, "keystroke_newMap");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3362]++;/*I_*/add(fileMenu, open, "keystroke_open");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3363]++;/*I_*/add(fileMenu, save, "keystroke_save");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3364]++;/*I_*/add(fileMenu, saveAs, "keystroke_saveAs");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3365]++;/*I_*/fileMenu.addSeparator();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3366]++;/*I_*/add(fileMenu, exportToHTML, "keystroke_export_to_html");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3367]++;/*I_*/return fileMenu;/*_I*/}/*I_*/
    }

    JMenu getExtensionMenu() {/*_I*/_prof.prof.cnt[3368]++;/*I_*/
	/*_I*/_prof.prof.cnt[3369]++;/*I_*/JMenu extensionMenu = new JMenu(getText("extension_menu"));
	/*_I*/{_prof.prof.cnt[3370]++;/*I_*/for (int i=0; i<patterns.length; ++i) {          
           /*_I*/_prof.prof.cnt[3371]++;/*I_*/JMenuItem item = extensionMenu.add(patterns[i]);
           /*_I*/{_prof.prof.cnt[3372]++;/*I_*/item.setAccelerator
              (KeyStroke.getKeyStroke
               (getFrame().getProperty("keystroke_apply_pattern_"+(i+1))));/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3373]++;/*I_*/return extensionMenu;/*_I*/}/*I_*/ }

    /* fc, 12.10.2003.*/
    JMenu getIconMenu() {/*_I*/_prof.prof.cnt[3374]++;/*I_*/
	/*_I*/_prof.prof.cnt[3375]++;/*I_*/JMenu iconMenu = new JMenu(getText("icon_menu"));
    /*_I*/{_prof.prof.cnt[3376]++;/*I_*/iconMenu.add(removeLastIcon);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3377]++;/*I_*/iconMenu.add(removeAllIcons);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3378]++;/*I_*/iconMenu.addSeparator();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3379]++;/*I_*/for (int i=0; i<iconActions.size(); ++i) {          
           /*_I*/_prof.prof.cnt[3380]++;/*I_*/JMenuItem item = iconMenu.add((Action) iconActions.get(i));
    }/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3381]++;/*I_*/return iconMenu;/*_I*/}/*I_*/ }

    JMenu getBranchMenu() {/*_I*/_prof.prof.cnt[3382]++;/*I_*/
	/*_I*/_prof.prof.cnt[3383]++;/*I_*/JMenu branchMenu = new JMenu(getText("branch"));

	/*_I*/{_prof.prof.cnt[3384]++;/*I_*/add(branchMenu, exportBranch, "keystroke_export_branch");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3385]++;/*I_*/add(branchMenu, exportBranchToHTML, "keystroke_export_branch_to_html");/*_I*/}/*I_*/



	/*_I*/{_prof.prof.cnt[3386]++;/*I_*/branchMenu.addSeparator();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3387]++;/*I_*/add(branchMenu, importBranch);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3388]++;/*I_*/add(branchMenu, importLinkedBranch);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3389]++;/*I_*/add(branchMenu, importLinkedBranchWithoutRoot);/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3390]++;/*I_*/branchMenu.addSeparator();/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3391]++;/*I_*/add(branchMenu, importExplorerFavorites);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3392]++;/*I_*/add(branchMenu, importFolderStructure);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3393]++;/*I_*/return branchMenu;/*_I*/}/*I_*/
    }

    JMenu getLeadingNodeMenu() {/*_I*/_prof.prof.cnt[3394]++;/*I_*/
       /*_I*/_prof.prof.cnt[3395]++;/*I_*/JMenu leadingEditMenu = new JMenu();
       /*_I*/{_prof.prof.cnt[3396]++;/*I_*/add(leadingEditMenu, edit, "keystroke_edit");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3397]++;/*I_*/add(leadingEditMenu, editLong, "keystroke_edit_long_node");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3398]++;/*I_*/add(leadingEditMenu, newChild, "keystroke_add_child");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3399]++;/*I_*/leadingEditMenu.addSeparator();/*_I*/}/*I_*/

       /*_I*/{_prof.prof.cnt[3400]++;/*I_*/add(leadingEditMenu, cut, "keystroke_cut");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3401]++;/*I_*/add(leadingEditMenu, copy, "keystroke_copy");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3402]++;/*I_*/add(leadingEditMenu, copySingle, "keystroke_copy_single");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3403]++;/*I_*/add(leadingEditMenu, paste, "keystroke_paste");/*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3404]++;/*I_*/return leadingEditMenu;/*_I*/}/*I_*/ }

    JMenu getNodeMenu() {/*_I*/_prof.prof.cnt[3405]++;/*I_*/
	/*_I*/_prof.prof.cnt[3406]++;/*I_*/JMenu nodeMenu = new JMenu(getText("node"));

// currently only hidden feature - needs debugging %%%   
//#  if the property "add_as_child = true" is set,
//#  the old logic of inserting of a new node with Ctrl+N is used.
   
        /*_I*/{_prof.prof.cnt[3407]++;/*I_*/if (addAsChildMode) {
          /*_I*/{_prof.prof.cnt[3408]++;/*I_*/add(nodeMenu, newChildWithoutFocus, "keystroke_add_sibling_before");/*_I*/}/*I_*/
        }
        else {
          /*_I*/{_prof.prof.cnt[3409]++;/*I_*/add(nodeMenu, newPreviousSibling, "keystroke_add_sibling_before");/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3410]++;/*I_*/add(nodeMenu, newSibling, "keystroke_add");/*_I*/}/*I_*/
 	/*_I*/{_prof.prof.cnt[3411]++;/*I_*/add(nodeMenu, remove, "keystroke_remove");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3412]++;/*I_*/add(nodeMenu, joinNodes, "keystroke_join_nodes");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3413]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[3414]++;/*I_*/add(nodeMenu, find, "keystroke_find");/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3415]++;/*I_*/add(nodeMenu, findNext, "keystroke_find_next");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3416]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

 	/*_I*/{_prof.prof.cnt[3417]++;/*I_*/add(nodeMenu, nodeUp, "keystroke_node_up");/*_I*/}/*I_*/
 	/*_I*/{_prof.prof.cnt[3418]++;/*I_*/add(nodeMenu, nodeDown, "keystroke_node_down");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3419]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3420]++;/*I_*/add(nodeMenu, followLink, "keystroke_follow_link");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3421]++;/*I_*/add(nodeMenu, setLinkByFileChooser, "keystroke_set_link_by_filechooser");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3422]++;/*I_*/add(nodeMenu, setLinkByTextField, "keystroke_set_link_by_textfield");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3423]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3424]++;/*I_*/add(nodeMenu, setImageByFileChooser, "keystroke_set_image_by_filechooser");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3425]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3426]++;/*I_*/add(nodeMenu, toggleFolded, "keystroke_toggle_folded");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3427]++;/*I_*/add(nodeMenu, toggleChildrenFolded, "keystroke_toggle_children_folded");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3428]++;/*I_*/nodeMenu.addSeparator();/*_I*/}/*I_*/

	/*_I*/_prof.prof.cnt[3429]++;/*I_*/JMenu nodeStyle = new JMenu(getText("style"));
	/*_I*/{_prof.prof.cnt[3430]++;/*I_*/nodeMenu.add(nodeStyle);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3431]++;/*I_*/add(nodeStyle, fork);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3432]++;/*I_*/add(nodeStyle, bubble);/*_I*/}/*I_*/
    // and the clouds:
	/*_I*/{_prof.prof.cnt[3433]++;/*I_*/nodeStyle.addSeparator();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3434]++;/*I_*/add(nodeStyle, cloud, "keystroke_node_toggle_cloud");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3435]++;/*I_*/add(nodeStyle, cloudColor);/*_I*/}/*I_*/
    
    

	/*_I*/_prof.prof.cnt[3436]++;/*I_*/JMenu nodeFont = new JMenu(getText("font"));
	/*_I*/{_prof.prof.cnt[3437]++;/*I_*/add(nodeFont, increaseNodeFont, "keystroke_node_increase_font_size");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3438]++;/*I_*/add(nodeFont, decreaseNodeFont, "keystroke_node_decrease_font_size");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3439]++;/*I_*/nodeFont.addSeparator();/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3440]++;/*I_*/add(nodeFont, italic,"keystroke_node_toggle_italic");/*_I*/}/*I_*/ 
	/*_I*/{_prof.prof.cnt[3441]++;/*I_*/add(nodeFont, bold,"keystroke_node_toggle_boldface");/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3442]++;/*I_*/nodeMenu.add(nodeFont);/*_I*/}/*I_*/
	//	nodeFont.add(underline);
	/*_I*/{_prof.prof.cnt[3443]++;/*I_*/add(nodeMenu, nodeColor, "keystroke_node_color");/*_I*/}/*I_*/ 
        /*_I*/{_prof.prof.cnt[3444]++;/*I_*/add(nodeMenu, nodeColorBlend, "keystroke_node_color_blend");/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[3445]++;/*I_*/return nodeMenu;/*_I*/}/*I_*/
    }

    JMenu getEdgeMenu() {/*_I*/_prof.prof.cnt[3446]++;/*I_*/
	/*_I*/_prof.prof.cnt[3447]++;/*I_*/JMenu edgeMenu = new JMenu(getText("edge"));
	/*_I*/_prof.prof.cnt[3448]++;/*I_*/JMenu edgeStyle = new JMenu(getText("style"));
	/*_I*/{_prof.prof.cnt[3449]++;/*I_*/edgeMenu.add(edgeStyle);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3450]++;/*I_*/for (int i=0; i<edgeStyles.length; ++i) { 
           /*_I*/{_prof.prof.cnt[3451]++;/*I_*/edgeStyle.add(edgeStyles[i]);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
 	/*_I*/{_prof.prof.cnt[3452]++;/*I_*/add(edgeMenu, edgeColor, "keystroke_edge_color");/*_I*/}/*I_*/
	/*_I*/_prof.prof.cnt[3453]++;/*I_*/JMenu edgeWidth = new JMenu(getText("width"));
	/*_I*/{_prof.prof.cnt[3454]++;/*I_*/edgeMenu.add(edgeWidth);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3455]++;/*I_*/for (int i=0; i<edgeWidths.length; ++i) { 
           /*_I*/{_prof.prof.cnt[3456]++;/*I_*/edgeWidth.add(edgeWidths[i]);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3457]++;/*I_*/return edgeMenu;/*_I*/}/*I_*/ }

    public JPopupMenu getPopupMenu() {/*_I*/_prof.prof.cnt[3458]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3459]++;/*I_*/return popupmenu;/*_I*/}/*I_*/
    }

    /** Link implementation: If this is a link, we want to make a popup with at least removelink available.*/
    public JPopupMenu getPopupForModel(java.lang.Object obj) {/*_I*/_prof.prof.cnt[3460]++;/*I_*/
        /*_I*/{_prof.prof.cnt[3461]++;/*I_*/if( obj instanceof MindMapArrowLinkModel) {
            // yes, this is a link.
            /*_I*/_prof.prof.cnt[3462]++;/*I_*/MindMapArrowLinkModel link = (MindMapArrowLinkModel) obj;
            /*_I*/_prof.prof.cnt[3463]++;/*I_*/JPopupMenu arrowLinkPopup = new JPopupMenu();
            // block the screen while showing popup.
            /*_I*/{_prof.prof.cnt[3464]++;/*I_*/arrowLinkPopup.addPopupMenuListener( this.popupListenerSingleton );/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3465]++;/*I_*/arrowLinkPopup.add(new RemoveArrowLinkAction(link.getSource(), link));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3466]++;/*I_*/arrowLinkPopup.add(new ColorArrowLinkAction(link.getSource(), link));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3467]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            /* The arrow state as radio buttons: */
            /*_I*/_prof.prof.cnt[3468]++;/*I_*/JRadioButtonMenuItem itemnn = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction("none", "images/arrow-mode-none.gif",link.getSource(), link, false, false) );
            /*_I*/{_prof.prof.cnt[3469]++;/*I_*/arrowLinkPopup.add( itemnn );/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3470]++;/*I_*/JRadioButtonMenuItem itemnt = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction("forward", "images/arrow-mode-forward.gif",link.getSource(), link, false, true) );
            /*_I*/{_prof.prof.cnt[3471]++;/*I_*/arrowLinkPopup.add( itemnt );/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3472]++;/*I_*/JRadioButtonMenuItem itemtn = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction("backward", "images/arrow-mode-backward.gif",link.getSource(), link, true, false) );
            /*_I*/{_prof.prof.cnt[3473]++;/*I_*/arrowLinkPopup.add( itemtn );/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3474]++;/*I_*/JRadioButtonMenuItem itemtt = new JRadioButtonMenuItem( new ChangeArrowsInArrowLinkAction("both", "images/arrow-mode-both.gif",link.getSource(), link, true, true) );
            /*_I*/{_prof.prof.cnt[3475]++;/*I_*/arrowLinkPopup.add( itemtt );/*_I*/}/*I_*/
            // select the right one:
            /*_I*/_prof.prof.cnt[3476]++;/*I_*/boolean a = !link.getStartArrow().equals("None");
            /*_I*/_prof.prof.cnt[3477]++;/*I_*/boolean b = !link.getEndArrow().equals("None");
            /*_I*/{_prof.prof.cnt[3478]++;/*I_*/itemtt.setSelected(a&&b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3479]++;/*I_*/itemnt.setSelected(!a&&b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3480]++;/*I_*/itemtn.setSelected(a&&!b);/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3481]++;/*I_*/itemnn.setSelected(!a&&!b);/*_I*/}/*I_*/

            /*_I*/{_prof.prof.cnt[3482]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            
            /*_I*/{_prof.prof.cnt[3483]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(link.getSource().toString(), link.getSource()));/*_I*/}/*I_*/ 
            /*_I*/{_prof.prof.cnt[3484]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(link.getTarget().toString(), link.getTarget()));/*_I*/}/*I_*/ 

            /*_I*/{_prof.prof.cnt[3485]++;/*I_*/arrowLinkPopup.addSeparator();/*_I*/}/*I_*/
            // add all links from target and from source:
            /*_I*/_prof.prof.cnt[3486]++;/*I_*/HashSet NodeAlreadyVisited = new HashSet();
            /*_I*/{_prof.prof.cnt[3487]++;/*I_*/NodeAlreadyVisited.add(link.getSource());/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3488]++;/*I_*/NodeAlreadyVisited.add(link.getTarget());/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3489]++;/*I_*/Vector links = getModel().getLinkRegistry().getAllLinks(link.getSource());
            /*_I*/{_prof.prof.cnt[3490]++;/*I_*/links.addAll(getModel().getLinkRegistry().getAllLinks(link.getTarget()));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3491]++;/*I_*/for(int i = 0; i < links.size(); ++i) {
                /*_I*/_prof.prof.cnt[3492]++;/*I_*/MindMapArrowLinkModel foreign_link = (MindMapArrowLinkModel) links.get(i);
                /*_I*/{_prof.prof.cnt[3493]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getTarget())) {
                    /*_I*/{_prof.prof.cnt[3494]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(foreign_link.getTarget().toString(), foreign_link.getTarget()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
                /*_I*/{_prof.prof.cnt[3495]++;/*I_*/if(NodeAlreadyVisited.add(foreign_link.getSource())) {
                    /*_I*/{_prof.prof.cnt[3496]++;/*I_*/arrowLinkPopup.add(new GotoLinkNodeAction(foreign_link.getSource().toString(), foreign_link.getSource()));/*_I*/}/*I_*/ 
                }/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3497]++;/*I_*/return arrowLinkPopup;/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[3498]++;/*I_*/return null;/*_I*/}/*I_*/
    }


    //convenience methods
    private MindMapMapModel getModel() {/*_I*/_prof.prof.cnt[3499]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3500]++;/*I_*/return (MindMapMapModel)getController().getModel();/*_I*/}/*I_*/
    }

    private MindMapNodeModel getSelected() {/*_I*/_prof.prof.cnt[3501]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3502]++;/*I_*/return (MindMapNodeModel)getView().getSelected().getModel();/*_I*/}/*I_*/
    }

    MindMapToolBar getToolBar() {/*_I*/_prof.prof.cnt[3503]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3504]++;/*I_*/return (MindMapToolBar)toolbar;/*_I*/}/*I_*/
    }

    JToolBar getLeftToolBar() {/*_I*/_prof.prof.cnt[3505]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3506]++;/*I_*/return ((MindMapToolBar)toolbar).getLeftToolBar();/*_I*/}/*I_*/
    }

    /**
     * Enabled/Disabled all actions that are dependent on
     * whether there is a map open or not.
     */
    protected void setAllActions(boolean enabled) {/*_I*/_prof.prof.cnt[3507]++;/*I_*/
	/*_I*/{_prof.prof.cnt[3508]++;/*I_*/edit.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3509]++;/*I_*/editLong.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3510]++;/*I_*/newChildWithoutFocus.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3511]++;/*I_*/newSibling.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3512]++;/*I_*/newPreviousSibling.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3513]++;/*I_*/newChild.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3514]++;/*I_*/remove.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3515]++;/*I_*/toggleFolded.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3516]++;/*I_*/toggleChildrenFolded.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3517]++;/*I_*/setLinkByTextField.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3518]++;/*I_*/setLinkByFileChooser.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3519]++;/*I_*/setImageByFileChooser.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3520]++;/*I_*/followLink.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3521]++;/*I_*/italic.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3522]++;/*I_*/bold.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3523]++;/*I_*/cloud.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3524]++;/*I_*/cloudColor.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3525]++;/*I_*/normalFont.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3526]++;/*I_*/nodeColor.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3527]++;/*I_*/edgeColor.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3528]++;/*I_*/removeLastIcon.setEnabled(enabled);/*_I*/}/*I_*/
    /*_I*/{_prof.prof.cnt[3529]++;/*I_*/removeAllIcons.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3530]++;/*I_*/for (int i=0; i<iconActions.size(); ++i) {          
        /*_I*/{_prof.prof.cnt[3531]++;/*I_*/((Action) iconActions.get(i)).setEnabled(enabled);/*_I*/}/*I_*/
    }/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3532]++;/*I_*/for (int i=0; i<edgeWidths.length; ++i) { 
		/*_I*/{_prof.prof.cnt[3533]++;/*I_*/edgeWidths[i].setEnabled(enabled);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3534]++;/*I_*/fork.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3535]++;/*I_*/bubble.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3536]++;/*I_*/for (int i=0; i<edgeStyles.length; ++i) { 
		/*_I*/{_prof.prof.cnt[3537]++;/*I_*/edgeStyles[i].setEnabled(enabled);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3538]++;/*I_*/for (int i=0; i<patterns.length; ++i) { 
		/*_I*/{_prof.prof.cnt[3539]++;/*I_*/patterns[i].setEnabled(enabled);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3540]++;/*I_*/save.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3541]++;/*I_*/saveAs.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3542]++;/*I_*/getToolBar().setAllActions(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3543]++;/*I_*/exportBranch.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3544]++;/*I_*/importBranch.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3545]++;/*I_*/importLinkedBranch.setEnabled(enabled);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[3546]++;/*I_*/importLinkedBranchWithoutRoot.setEnabled(enabled);/*_I*/}/*I_*/
    }




    //
    //      Actions
    //_______________________________________________________________________________



    // Export and Import
    // _________________


    // This may later be moved to ControllerAdapter. So far there is no reason for it.
    protected class ExportToHTMLAction extends AbstractAction {
	MindMapController c;
	public ExportToHTMLAction(MindMapController controller) {
	    super(getText("export_to_html"));/*_I*/_prof.prof.cnt[3547]++;/*I_*//*_I*/_prof.prof.cnt[3548]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[3549]++;/*I_*/c = controller;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3550]++;/*I_*/
           /*_I*/_prof.prof.cnt[3551]++;/*I_*/File file = new File(c.getModel().getFile()+".html");
           /*_I*/{_prof.prof.cnt[3552]++;/*I_*/if (c.getModel().saveHTML((MindMapNodeModel)c.getModel().getRoot(),file)) {
              /*_I*/{_prof.prof.cnt[3553]++;/*I_*/loadURL(file.toString());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    protected class ExportBranchToHTMLAction extends AbstractAction {
	MindMapController c;
	public ExportBranchToHTMLAction(MindMapController controller) {
	    super(getText("export_branch_to_html"));/*_I*/_prof.prof.cnt[3554]++;/*I_*//*_I*/_prof.prof.cnt[3555]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[3556]++;/*I_*/c = controller;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3557]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3558]++;/*I_*/try {
              /*_I*/_prof.prof.cnt[3559]++;/*I_*/File file = File.createTempFile("tmm", ".html");
              /*_I*/{_prof.prof.cnt[3560]++;/*I_*/if (c.getModel().saveHTML((MindMapNodeModel)getSelected(),file)) {
                 /*_I*/{_prof.prof.cnt[3561]++;/*I_*/loadURL(file.toString());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
           catch (IOException ex) {}/*_I*/}/*I_*/}}

    private class ExportBranchAction extends AbstractAction {
	ExportBranchAction() {
	    super(getText("export_branch"));/*_I*/_prof.prof.cnt[3562]++;/*I_*//*_I*/_prof.prof.cnt[3563]++;/*I_*/
	}
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3564]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3565]++;/*I_*/MindMapNodeModel node = (MindMapNodeModel)getSelected();

	    //if something is wrong, abort.
	    /*_I*/{_prof.prof.cnt[3566]++;/*I_*/if (getMap() == null || node == null || node.isRoot()) {
		/*_I*/{_prof.prof.cnt[3567]++;/*I_*/getFrame().err("Could not export branch.");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3568]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	    //If the current map is not saved yet, save it first.
	    /*_I*/{_prof.prof.cnt[3569]++;/*I_*/if (getMap().getFile() == null) {
		/*_I*/{_prof.prof.cnt[3570]++;/*I_*/getFrame().out("You must save the current map first!");/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3571]++;/*I_*/save();/*_I*/}/*I_*/ }/*_I*/}/*I_*/

	    //Open FileChooser to choose in which file the exported
	    //branch should be stored
	    /*_I*/_prof.prof.cnt[3572]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[3573]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir); 
// 	    if (getMap().getFile().getParentFile() != null) {
//                chooser = new JFileChooser(getMap().getFile().getParentFile()); }
//             else {
//                chooser = new JFileChooser(); }
	    //chooser.setLocale(currentLocale);
	    /*_I*/{_prof.prof.cnt[3574]++;/*I_*/if (getFileFilter() != null) {
               /*_I*/{_prof.prof.cnt[3575]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	    /*_I*/_prof.prof.cnt[3576]++;/*I_*/int returnVal = chooser.showSaveDialog(getSelected().getViewer());
	    /*_I*/{_prof.prof.cnt[3577]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
		/*_I*/_prof.prof.cnt[3578]++;/*I_*/File f = chooser.getSelectedFile();
		/*_I*/_prof.prof.cnt[3579]++;/*I_*/URL link;
		//Force the extension to be .mm
		/*_I*/_prof.prof.cnt[3580]++;/*I_*/String ext = Tools.getExtension(f.getName());
		/*_I*/{_prof.prof.cnt[3581]++;/*I_*/if(!ext.equals("mm")) {
                   /*_I*/{_prof.prof.cnt[3582]++;/*I_*/f = new File(f.getParent(),f.getName()+".mm");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3583]++;/*I_*/try {
                   /*_I*/{_prof.prof.cnt[3584]++;/*I_*/link = f.toURL();/*_I*/}/*I_*/ }
                catch (MalformedURLException ex) {
		    /*_I*/{_prof.prof.cnt[3585]++;/*I_*/JOptionPane.showMessageDialog(getView(),"couldn't create valid URL!");/*_I*/}/*I_*/
		    /*_I*/{_prof.prof.cnt[3586]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/

		//Now make a copy from the node, remove the node from the map and create a new
		//Map with the node as root, store the new Map, add the copy of the node to the parent,
		//and set a link from the copy to the new Map.

		/*_I*/_prof.prof.cnt[3587]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel)node.getParentNode();
		/*_I*/_prof.prof.cnt[3588]++;/*I_*/MindMapNodeModel newNode = new MindMapNodeModel(node.toString(),getFrame());
		/*_I*/{_prof.prof.cnt[3589]++;/*I_*/getModel().removeNodeFromParent(node);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3590]++;/*I_*/node.setParent(null);/*_I*/}/*I_*/
		/*_I*/_prof.prof.cnt[3591]++;/*I_*/MindMapMapModel map = new MindMapMapModel(node,getFrame());
		/*_I*/{_prof.prof.cnt[3592]++;/*I_*/if (getModel().getFile() != null) {
		    /*_I*/{_prof.prof.cnt[3593]++;/*I_*/try{
			//set a link from the new root to the old map
                       /*_I*/{_prof.prof.cnt[3594]++;/*I_*/map.setLink(node, Tools.toRelativeURL(f.toURL(), getModel().getFile().toURL()));/*_I*/}/*I_*/ }
                    catch(MalformedURLException ex) { }/*_I*/}/*I_*/}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3595]++;/*I_*/map.save(f);/*_I*/}/*I_*/

		/*_I*/{_prof.prof.cnt[3596]++;/*I_*/getModel().insertNodeInto(newNode, parent, 0);/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3597]++;/*I_*/try {
		    /*_I*/_prof.prof.cnt[3598]++;/*I_*/String linkString = Tools.toRelativeURL(getModel().getFile().toURL(), f.toURL());
		    /*_I*/{_prof.prof.cnt[3599]++;/*I_*/getModel().setLink(newNode,linkString);/*_I*/}/*I_*/ }
                catch (MalformedURLException ex) {}/*_I*/}/*I_*/
		/*_I*/{_prof.prof.cnt[3600]++;/*I_*/getModel().save(getModel().getFile());/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class ImportBranchAction extends AbstractAction {
	ImportBranchAction() {
           super(getText("import_branch"));/*_I*/_prof.prof.cnt[3601]++;/*I_*//*_I*/_prof.prof.cnt[3602]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3603]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3604]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[3605]++;/*I_*/if (parent == null) {
               /*_I*/{_prof.prof.cnt[3606]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
	    /*_I*/_prof.prof.cnt[3607]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[3608]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir);
            //chooser.setLocale(currentLocale);
            /*_I*/{_prof.prof.cnt[3609]++;/*I_*/if (getFileFilter() != null) {
               /*_I*/{_prof.prof.cnt[3610]++;/*I_*/chooser.addChoosableFileFilter(getFileFilter());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3611]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
            /*_I*/{_prof.prof.cnt[3612]++;/*I_*/if (returnVal==JFileChooser.APPROVE_OPTION) {
               /*_I*/{_prof.prof.cnt[3613]++;/*I_*/try {
                  /*_I*/_prof.prof.cnt[3614]++;/*I_*/MindMapNodeModel node = getModel().loadTree(chooser.getSelectedFile());
                  /*_I*/{_prof.prof.cnt[3615]++;/*I_*/getModel().paste(node, parent);/*_I*/}/*I_*/ }
               catch (Exception ex) {
                  /*_I*/{_prof.prof.cnt[3616]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/}}

    private class ImportLinkedBranchAction extends AbstractAction {
	ImportLinkedBranchAction() {
           super(getText("import_linked_branch"));/*_I*/_prof.prof.cnt[3617]++;/*I_*//*_I*/_prof.prof.cnt[3618]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3619]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3620]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[3621]++;/*I_*/if (parent == null || parent.getLink() == null) {
               /*_I*/{_prof.prof.cnt[3622]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3623]++;/*I_*/URL absolute = null;
            /*_I*/{_prof.prof.cnt[3624]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[3625]++;/*I_*/String relative = parent.getLink();
               /*_I*/{_prof.prof.cnt[3626]++;/*I_*/absolute = Tools.isAbsolutePath(relative) ? new File(relative).toURL() :
                  new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/ }
            catch (MalformedURLException ex) {
               /*_I*/{_prof.prof.cnt[3627]++;/*I_*/JOptionPane.showMessageDialog(getView(),"Couldn't create valid URL for:"+getMap().getFile());/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[3628]++;/*I_*/ex.printStackTrace();/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[3629]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3630]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[3631]++;/*I_*/MindMapNodeModel node = getModel().loadTree(new File(absolute.getFile()));
               /*_I*/{_prof.prof.cnt[3632]++;/*I_*/getModel().paste(node, parent);/*_I*/}/*I_*/ }
            catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[3633]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    /**
     * This is exactly the opposite of exportBranch.
     */
    private class ImportLinkedBranchWithoutRootAction extends AbstractAction {
	ImportLinkedBranchWithoutRootAction() {
           super(getText("import_linked_branch_without_root"));/*_I*/_prof.prof.cnt[3634]++;/*I_*//*_I*/_prof.prof.cnt[3635]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3636]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3637]++;/*I_*/MindMapNodeModel parent = (MindMapNodeModel)getSelected();
	    /*_I*/{_prof.prof.cnt[3638]++;/*I_*/if (parent == null || parent.getLink() == null) {
               /*_I*/{_prof.prof.cnt[3639]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[3640]++;/*I_*/URL absolute = null;
            /*_I*/{_prof.prof.cnt[3641]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[3642]++;/*I_*/String relative = parent.getLink();
               /*_I*/{_prof.prof.cnt[3643]++;/*I_*/absolute = Tools.isAbsolutePath(relative) ? new File(relative).toURL() :
                  new URL(getMap().getFile().toURL(), relative);/*_I*/}/*I_*/ }
            catch (MalformedURLException ex) {
               /*_I*/{_prof.prof.cnt[3644]++;/*I_*/JOptionPane.showMessageDialog(getView(),"Couldn't create valid URL.");/*_I*/}/*I_*/
               /*_I*/{_prof.prof.cnt[3645]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3646]++;/*I_*/try {
               /*_I*/_prof.prof.cnt[3647]++;/*I_*/MindMapNodeModel node = getModel().loadTree(new File(absolute.getFile()));
               /*_I*/{_prof.prof.cnt[3648]++;/*I_*/for (ListIterator i = node.childrenUnfolded();i.hasNext();) {
                  /*_I*/{_prof.prof.cnt[3649]++;/*I_*/getModel().paste((MindMapNodeModel)i.next(), parent);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}
               //getModel().setLink(parent, null); }
            catch (Exception ex) {
               /*_I*/{_prof.prof.cnt[3650]++;/*I_*/handleLoadingException(ex);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class ImportExplorerFavoritesAction extends AbstractAction {
	ImportExplorerFavoritesAction() { super(getText("import_explorer_favorites"));/*_I*/_prof.prof.cnt[3651]++;/*I_*//*_I*/_prof.prof.cnt[3652]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3653]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3654]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[3655]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir);
           /*_I*/{_prof.prof.cnt[3656]++;/*I_*/chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3657]++;/*I_*/chooser.setDialogTitle(getText("select_favorites_folder"));/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[3658]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
           /*_I*/{_prof.prof.cnt[3659]++;/*I_*/if (returnVal == JFileChooser.APPROVE_OPTION) {
              /*_I*/_prof.prof.cnt[3660]++;/*I_*/File folder = chooser.getSelectedFile();
              /*_I*/{_prof.prof.cnt[3661]++;/*I_*/getFrame().out("Importing Favorites ...");/*_I*/}/*I_*/
              //getFrame().repaint(); // Refresh the frame, namely hide dialog and show status
              //getView().updateUI();
              // Problem: the frame should be refreshed here, but I don't know how to do it
              /*_I*/{_prof.prof.cnt[3662]++;/*I_*/getModel().importExplorerFavorites(folder,getSelected(),/*redisplay=*/true);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[3663]++;/*I_*/getFrame().out("Favorites imported.");/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class ImportFolderStructureAction extends AbstractAction {
	ImportFolderStructureAction() { super(getText("import_folder_structure"));/*_I*/_prof.prof.cnt[3664]++;/*I_*//*_I*/_prof.prof.cnt[3665]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3666]++;/*I_*/
	    /*_I*/_prof.prof.cnt[3667]++;/*I_*/java.io.File ioDir = new java.io.File( "input_files" );
            /*_I*/_prof.prof.cnt[3668]++;/*I_*/JFileChooser chooser = new JFileChooser(ioDir);
           /*_I*/{_prof.prof.cnt[3669]++;/*I_*/chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3670]++;/*I_*/chooser.setDialogTitle(getText("select_folder_for_importing"));/*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[3671]++;/*I_*/int returnVal = chooser.showOpenDialog(getFrame().getContentPane());
           /*_I*/{_prof.prof.cnt[3672]++;/*I_*/if (returnVal == JFileChooser.APPROVE_OPTION) {
              /*_I*/_prof.prof.cnt[3673]++;/*I_*/File folder = chooser.getSelectedFile();
              /*_I*/{_prof.prof.cnt[3674]++;/*I_*/getFrame().out("Importing folder structure ...");/*_I*/}/*I_*/
              //getFrame().repaint(); // Refresh the frame, namely hide dialog and show status
              //getView().updateUI();
              // Problem: the frame should be refreshed here, but I don't know how to do it
              /*_I*/{_prof.prof.cnt[3675]++;/*I_*/getModel().importFolderStructure(folder,getSelected(),/*redisplay=*/true);/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[3676]++;/*I_*/getFrame().out("Folder structure imported.");/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}


    // Color
    // __________________

    private class NodeColorAction extends AbstractAction {
       NodeColorAction() { super(getText("node_color"));/*_I*/_prof.prof.cnt[3677]++;/*I_*//*_I*/_prof.prof.cnt[3678]++;/*I_*/ }
       public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3679]++;/*I_*/
          /*_I*/_prof.prof.cnt[3680]++;/*I_*/Color color = JColorChooser.showDialog(getView().getSelected(),"Choose Node Color:",getSelected().getColor() );
          /*_I*/{_prof.prof.cnt[3681]++;/*I_*/if (color==null) {
             /*_I*/{_prof.prof.cnt[3682]++;/*I_*/return;/*_I*/}/*I_*/ }/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[3683]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             /*_I*/_prof.prof.cnt[3684]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
             /*_I*/{_prof.prof.cnt[3685]++;/*I_*/getModel().setNodeColor(selected, color);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class EdgeColorAction extends AbstractAction {
	EdgeColorAction() { super(getText("edge_color"));/*_I*/_prof.prof.cnt[3686]++;/*I_*//*_I*/_prof.prof.cnt[3687]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3688]++;/*I_*/
           /*_I*/_prof.prof.cnt[3689]++;/*I_*/Color color = JColorChooser.showDialog(getView().getSelected(),"Choose Edge Color:",getSelected().getEdge().getColor());
           /*_I*/{_prof.prof.cnt[3690]++;/*I_*/if (color==null) /*_I*/{_prof.prof.cnt[3691]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3692]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
              /*_I*/_prof.prof.cnt[3693]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
              /*_I*/{_prof.prof.cnt[3694]++;/*I_*/getModel().setEdgeColor(selected,color);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}


    private class CloudColorAction extends AbstractAction {
	CloudColorAction() { 
        super(getText("cloud_color"), new ImageIcon(getResource("images/Colors24.gif")));/*_I*/_prof.prof.cnt[3695]++;/*I_*//*_I*/_prof.prof.cnt[3696]++;/*I_*/ 
        /*_I*/{_prof.prof.cnt[3697]++;/*I_*/putValue(Action.SHORT_DESCRIPTION, getValue(Action.NAME));/*_I*/}/*I_*/}
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3698]++;/*I_*/
        /*_I*/_prof.prof.cnt[3699]++;/*I_*/Color selectedColor = null;
        /*_I*/{_prof.prof.cnt[3700]++;/*I_*/if(getSelected().getCloud() != null)
            /*_I*/{_prof.prof.cnt[3701]++;/*I_*/selectedColor = getSelected().getCloud().getColor();/*_I*/}/*I_*//*_I*/}/*I_*/
           /*_I*/_prof.prof.cnt[3702]++;/*I_*/Color color = JColorChooser.showDialog(getView().getSelected(),"Choose Cloud Color:",selectedColor);
           /*_I*/{_prof.prof.cnt[3703]++;/*I_*/if (color==null) /*_I*/{_prof.prof.cnt[3704]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3705]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
              /*_I*/_prof.prof.cnt[3706]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
              /*_I*/{_prof.prof.cnt[3707]++;/*I_*/getModel().setCloudColor(selected,color);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}


    protected class ColorArrowLinkAction extends AbstractAction {
        MindMapNode source;
        MindMapArrowLinkModel arrowLink;
        public ColorArrowLinkAction(MindMapNode source, MindMapArrowLinkModel arrowLink) {
            super(getText("arrow_link_color"), new ImageIcon(getResource("images/Colors24.gif")));/*_I*/_prof.prof.cnt[3708]++;/*I_*//*_I*/_prof.prof.cnt[3709]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3710]++;/*I_*/this.source = source;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3711]++;/*I_*/this.arrowLink = arrowLink;/*_I*/}/*I_*/
        }

        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3712]++;/*I_*/
            /*_I*/_prof.prof.cnt[3713]++;/*I_*/Color selectedColor = arrowLink.getColor();
            /*_I*/_prof.prof.cnt[3714]++;/*I_*/Color color = JColorChooser.showDialog(getView().getSelected(),(String) this.getValue(Action.NAME),selectedColor);
            /*_I*/{_prof.prof.cnt[3715]++;/*I_*/if (color==null) /*_I*/{_prof.prof.cnt[3716]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3717]++;/*I_*/getModel().setArrowLinkColor(source, arrowLink, color);/*_I*/}/*I_*/ 
        }
    }
    


    // Icons
    // __________________

    private class IconAction extends AbstractAction {
        MindIcon icon;
        public IconAction(MindIcon _icon) {
            super(_icon.getDescription(getFrame()), _icon.getIcon(getFrame()));/*_I*/_prof.prof.cnt[3718]++;/*I_*//*_I*/_prof.prof.cnt[3719]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3720]++;/*I_*/putValue(Action.SHORT_DESCRIPTION, _icon.getDescription(getFrame()));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3721]++;/*I_*/this.icon = _icon;/*_I*/}/*I_*/
        };
        
        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3722]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3723]++;/*I_*/for (ListIterator it = getSelecteds().listIterator();it.hasNext();) {
              /*_I*/_prof.prof.cnt[3724]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
              /*_I*/{_prof.prof.cnt[3725]++;/*I_*/(getModel()).addIcon(selected, icon);/*_I*/}/*I_*/ 
            }/*_I*/}/*I_*/
        };
    }

    // ArrowLinks
    // __________________

    protected class RemoveArrowLinkAction extends AbstractAction {
        MindMapNode source;
        MindMapArrowLinkModel arrowLink;
        public RemoveArrowLinkAction(MindMapNode source, MindMapArrowLinkModel arrowLink) {
            super(getText("remove_arrow_link"), new ImageIcon(getResource("images/edittrash.png")));/*_I*/_prof.prof.cnt[3726]++;/*I_*//*_I*/_prof.prof.cnt[3727]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3728]++;/*I_*/this.source = source;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3729]++;/*I_*/this.arrowLink = arrowLink;/*_I*/}/*I_*/
        }

        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3730]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3731]++;/*I_*/getModel().removeReference(source, arrowLink);/*_I*/}/*I_*/
        }
    }

    protected class ChangeArrowsInArrowLinkAction extends AbstractAction {
        MindMapNode source;
        MindMapArrowLinkModel arrowLink;
        boolean hasStartArrow;
        boolean hasEndArrow;
        public ChangeArrowsInArrowLinkAction(String text, String iconPath, MindMapNode source, MindMapArrowLinkModel arrowLink, boolean hasStartArrow, boolean hasEndArrow) {
            super("", iconPath != null ? new ImageIcon(getResource(iconPath)) : null);/*_I*/_prof.prof.cnt[3732]++;/*I_*//*_I*/_prof.prof.cnt[3733]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3734]++;/*I_*/this.source = source;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3735]++;/*I_*/this.arrowLink = arrowLink;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3736]++;/*I_*/this.hasStartArrow = hasStartArrow;/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[3737]++;/*I_*/this.hasEndArrow = hasEndArrow;/*_I*/}/*I_*/
        }

        public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3738]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3739]++;/*I_*/getModel().changeArrowsOfArrowLink(source, arrowLink, hasStartArrow, hasEndArrow);/*_I*/}/*I_*/
        }
    }
    
    // NodeGeneralAction
    // __________________

    private interface SingleNodeOperation {
       public void apply(MindMapMapModel map, MindMapNodeModel node); }

    private class NodeGeneralAction extends AbstractAction {
        SingleNodeOperation singleNodeOperation;
        NodeGeneralAction(String textID, String iconPath, SingleNodeOperation singleNodeOperation) {
           super(getText(textID), iconPath != null ? new ImageIcon(getResource(iconPath)) : null );/*_I*/_prof.prof.cnt[3740]++;/*I_*//*_I*/_prof.prof.cnt[3741]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3742]++;/*I_*/putValue(Action.SHORT_DESCRIPTION, getText(textID));/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[3743]++;/*I_*/this.singleNodeOperation = singleNodeOperation;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3744]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3745]++;/*I_*/for (ListIterator it = getSelecteds().listIterator();it.hasNext();) {
              /*_I*/_prof.prof.cnt[3746]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
              /*_I*/{_prof.prof.cnt[3747]++;/*I_*/singleNodeOperation.apply(getModel(), selected);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    // Edge width
    // __________________

    private String getWidthTitle(int width) {/*_I*/_prof.prof.cnt[3748]++;/*I_*/
       /*_I*/{_prof.prof.cnt[3749]++;/*I_*/if (width==EdgeAdapter.WIDTH_PARENT)
          /*_I*/{_prof.prof.cnt[3750]++;/*I_*/return getText("edge_width_parent");/*_I*/}/*I_*//*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3751]++;/*I_*/if (width==EdgeAdapter.WIDTH_THIN)
          /*_I*/{_prof.prof.cnt[3752]++;/*I_*/return getText("edge_width_thin");/*_I*/}/*I_*//*_I*/}/*I_*/
       /*_I*/{_prof.prof.cnt[3753]++;/*I_*/return Integer.toString(width);/*_I*/}/*I_*/ }

    private class EdgeWidthAction extends AbstractAction {
       int width;
       EdgeWidthAction(int width) {
          super(getWidthTitle(width));/*_I*/_prof.prof.cnt[3754]++;/*I_*//*_I*/_prof.prof.cnt[3755]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3756]++;/*I_*/this.width = width;/*_I*/}/*I_*/ }
       public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3757]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3758]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             /*_I*/_prof.prof.cnt[3759]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
             /*_I*/{_prof.prof.cnt[3760]++;/*I_*/getModel().setEdgeWidth(selected,width);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}


    // Miscelaneous
    // _________________


    private class JoinNodesAction extends AbstractAction {
	JoinNodesAction() { super(getText("join_nodes"));/*_I*/_prof.prof.cnt[3761]++;/*I_*//*_I*/_prof.prof.cnt[3762]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3763]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3764]++;/*I_*/((MindMapMapModel)getView().getModel()).joinNodes();/*_I*/}/*I_*/ }}

    private class FollowLinkAction extends AbstractAction {
	FollowLinkAction() { super(getText("follow_link"));/*_I*/_prof.prof.cnt[3765]++;/*I_*//*_I*/_prof.prof.cnt[3766]++;/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3767]++;/*I_*/
           /*_I*/{_prof.prof.cnt[3768]++;/*I_*/loadURL();/*_I*/}/*I_*/ }}

    private class ForkAction extends AbstractAction {
       ForkAction() { super(getText(MindMapNode.STYLE_FORK));/*_I*/_prof.prof.cnt[3769]++;/*I_*//*_I*/_prof.prof.cnt[3770]++;/*I_*/ }
       public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3771]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3772]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             /*_I*/_prof.prof.cnt[3773]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
             /*_I*/{_prof.prof.cnt[3774]++;/*I_*/getModel().setNodeStyle(selected, MindMapNode.STYLE_FORK);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class BubbleAction extends AbstractAction {
	BubbleAction() { super(getText(MindMapNode.STYLE_BUBBLE));/*_I*/_prof.prof.cnt[3775]++;/*I_*//*_I*/_prof.prof.cnt[3776]++;/*I_*/ }
       public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3777]++;/*I_*/
          /*_I*/{_prof.prof.cnt[3778]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             /*_I*/_prof.prof.cnt[3779]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
             /*_I*/{_prof.prof.cnt[3780]++;/*I_*/getModel().setNodeStyle(selected, MindMapNode.STYLE_BUBBLE);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class EdgeStyleAction extends AbstractAction {
	String style;
	EdgeStyleAction(String style) {
	    super(getText(style));/*_I*/_prof.prof.cnt[3781]++;/*I_*//*_I*/_prof.prof.cnt[3782]++;/*I_*/
            /*_I*/{_prof.prof.cnt[3783]++;/*I_*/this.style = style;/*_I*/}/*I_*/ }	
       public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3784]++;/*I_*/          
          /*_I*/{_prof.prof.cnt[3785]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
             /*_I*/_prof.prof.cnt[3786]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
             /*_I*/{_prof.prof.cnt[3787]++;/*I_*/getModel().setEdgeStyle(selected, style);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}

    private class ApplyPatternAction extends AbstractAction {
        StylePattern pattern;
	ApplyPatternAction(StylePattern pattern) {
	    super(pattern.getName());/*_I*/_prof.prof.cnt[3788]++;/*I_*//*_I*/_prof.prof.cnt[3789]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[3790]++;/*I_*/this.pattern=pattern;/*_I*/}/*I_*/ }
	public void actionPerformed(ActionEvent e) {/*_I*/_prof.prof.cnt[3791]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[3792]++;/*I_*/for(ListIterator it = getSelecteds().listIterator();it.hasNext();) {
		/*_I*/_prof.prof.cnt[3793]++;/*I_*/MindMapNodeModel selected = (MindMapNodeModel)it.next();
                /*_I*/{_prof.prof.cnt[3794]++;/*I_*/((MindMapMapModel)getModel()).applyPattern(selected, pattern);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}}




    // Nonaction classes
    // ________________________________________________________________________

    private class MindMapFilter extends FileFilter {
      public boolean accept(File f) {/*_I*/_prof.prof.cnt[3795]++;/*I_*/
	      /*_I*/{_prof.prof.cnt[3796]++;/*I_*/if (f.isDirectory()) /*_I*/{_prof.prof.cnt[3797]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
	      /*_I*/_prof.prof.cnt[3798]++;/*I_*/String extension = Tools.getExtension(f.getName());
	      /*_I*/{_prof.prof.cnt[3799]++;/*I_*/if (extension != null) {
		  /*_I*/{_prof.prof.cnt[3800]++;/*I_*/if (extension.equals("mm")) {
		     /*_I*/{_prof.prof.cnt[3801]++;/*I_*/return true;/*_I*/}/*I_*/
		  } else {
		     /*_I*/{_prof.prof.cnt[3802]++;/*I_*/return false;/*_I*/}/*I_*/
		  }/*_I*/}/*I_*/
	      }/*_I*/}/*I_*/
	      /*_I*/{_prof.prof.cnt[3803]++;/*I_*/return false;/*_I*/}/*I_*/
       }
	
	   public String getDescription() {/*_I*/_prof.prof.cnt[3804]++;/*I_*/
	      /*_I*/{_prof.prof.cnt[3805]++;/*I_*/return getText("mindmaps_desc");/*_I*/}/*I_*/
	   }
    }
}
