/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: BubbleNodeView.java,v 1.14 2004/01/10 18:22:25 christianfoltin Exp $*/

package freemind.view.mindmapview;

import freemind.modes.MindMapNode;
import java.awt.*;

/**
 * This class represents a single Bubble-Style Node of a MindMap
 * (in analogy to TreeCellRenderer).
 */
public class BubbleNodeView extends NodeView {

    private final static Stroke BOLD_STROKE =
		new BasicStroke(2.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER,
					1f, new float[] {2f, 2f}, 0f);  
    private final static Stroke DEF_STROKE = new BasicStroke();


    public final int LEFT_WIDTH_OVERHEAD = 5;
    public final int LEFT_HEIGHT_OVERHEAD = 2;

    //
    // Constructors
    //
    
    public BubbleNodeView(MindMapNode model, MapView map) {
	super(model,map);/*_I*/_prof.prof.cnt[2232]++;/*I_*//*_I*/_prof.prof.cnt[2233]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2234]++;/*I_*/setHorizontalAlignment(CENTER);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2235]++;/*I_*/setVerticalAlignment(CENTER);/*_I*/}/*I_*/
    }



    public Dimension getPreferredSize() {/*_I*/_prof.prof.cnt[2236]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2237]++;/*I_*/return new Dimension(super.getPreferredSize().width+2*LEFT_WIDTH_OVERHEAD,
                             super.getPreferredSize().height+4);/*_I*/}/*I_*/
    }	  

    public void paintSelected(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2238]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2239]++;/*I_*/if( this.isSelected() ) {
          /*_I*/{_prof.prof.cnt[2240]++;/*I_*/graphics.setColor(selectedColor);/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[2241]++;/*I_*/graphics.fillRoundRect(0,0,size.width-1,size.height-1,10,10);/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

    /**
     * Paints the node
     */
    public void paint(Graphics graphics) {/*_I*/_prof.prof.cnt[2242]++;/*I_*/
	/*_I*/_prof.prof.cnt[2243]++;/*I_*/Graphics2D g = (Graphics2D)graphics;
	/*_I*/_prof.prof.cnt[2244]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2245]++;/*I_*/if (this.getModel()==null) /*_I*/{_prof.prof.cnt[2246]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[2247]++;/*I_*/paintSelected(g, size);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2248]++;/*I_*/paintDragOver(g, size);/*_I*/}/*I_*/

	// change to bold stroke
	//g.setStroke(BOLD_STROKE);                     // Changed by Daniel

        /*_I*/{_prof.prof.cnt[2249]++;/*I_*/setRendering(g);/*_I*/}/*I_*/

	//Draw a standard node
	/*_I*/{_prof.prof.cnt[2250]++;/*I_*/g.setColor(getEdge().getColor());/*_I*/}/*I_*/
	//g.drawOval(0,0,size.width-1,size.height-1);   // Changed by Daniel

        /*_I*/{_prof.prof.cnt[2251]++;/*I_*/if (map.getController().getAntialiasEdges()) {
           /*_I*/{_prof.prof.cnt[2252]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2253]++;/*I_*/g.drawRoundRect(0,0,size.width-1,size.height-1,10,10);/*_I*/}/*I_*/
        // this disables the font antialias if only AntialiasEdges is requested.
        /*_I*/{_prof.prof.cnt[2254]++;/*I_*/if (map.getController().getAntialiasEdges()) {
           /*_I*/{_prof.prof.cnt[2255]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

	// return to std stroke
	/*_I*/{_prof.prof.cnt[2256]++;/*I_*/g.setStroke(DEF_STROKE);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2257]++;/*I_*/super.paint(g);/*_I*/}/*I_*/
    }

    public int getLeftWidthOverhead() {/*_I*/_prof.prof.cnt[2258]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2259]++;/*I_*/return LEFT_WIDTH_OVERHEAD;/*_I*/}/*I_*/ }

    /**
     * Returns the Point where the OutEdge
     * should leave the Node.
     */
    Point getOutPoint() {/*_I*/_prof.prof.cnt[2260]++;/*I_*/
	/*_I*/_prof.prof.cnt[2261]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2262]++;/*I_*/if (isLeft()) {
	    /*_I*/{_prof.prof.cnt[2263]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2264]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    /**
     * Returns the Point where the InEdge
     * should arrive the Node.
     */
    Point getInPoint() {/*_I*/_prof.prof.cnt[2265]++;/*I_*/
	/*_I*/_prof.prof.cnt[2266]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2267]++;/*I_*/if (isLeft()) {
	    /*_I*/{_prof.prof.cnt[2268]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2269]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    /**
     * Returns the relative position of the Edge
     */
    int getAlignment() {/*_I*/_prof.prof.cnt[2270]++;/*I_*/
	    /*_I*/{_prof.prof.cnt[2271]++;/*I_*/return ALIGN_CENTER;/*_I*/}/*I_*/
	}
}

