/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: NodeView.java,v 1.27 2004/01/25 16:41:28 christianfoltin Exp $*/

package freemind.view.mindmapview;

import freemind.main.FreeMind;
import freemind.main.Tools;
import freemind.modes.MindMapNode;
import freemind.modes.NodeAdapter;//This should not be done.
import freemind.modes.MindIcon;

import java.util.LinkedList;
import java.util.ListIterator;
import java.awt.*;
import java.awt.dnd.*;
import java.net.MalformedURLException;

import javax.swing.JLabel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.util.Vector;


/**
 * This class represents a single Node of a MindMap
 * (in analogy to TreeCellRenderer).
 */
public abstract class NodeView extends JLabel {

    protected MindMapNode model;
    protected MapView map;
    protected EdgeView edge;
    protected final static Color selectedColor = new Color(210,210,210); //Color.lightGray; //the Color of the Rectangle of a selected Node
    protected final static Color dragColor = Color.lightGray; //the Color of appearing GradientBox on drag over
    protected int treeHeight;
    private boolean left = true; //is the node left of root?
    int relYPos;//the relative Y Position to it's parent
    private boolean isLong = false;

    public final static int DRAGGED_OVER_NO = 0;
    public final static int DRAGGED_OVER_SON = 1;
    public final static int DRAGGED_OVER_SIBLING = 2;
    /** For RootNodeView.*/
    public final static int DRAGGED_OVER_SON_LEFT = 3;

    protected int isDraggedOver = DRAGGED_OVER_NO;
    public void setDraggedOver(int draggedOver) {/*_I*/_prof.prof.cnt[2392]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2393]++;/*I_*/isDraggedOver = draggedOver;/*_I*/}/*I_*/ }
    public void setDraggedOver(Point p) {/*_I*/_prof.prof.cnt[2394]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2395]++;/*I_*/setDraggedOver( (dropAsSibling(p.getX())) ? NodeView.DRAGGED_OVER_SIBLING : NodeView.DRAGGED_OVER_SON) ;/*_I*/}/*I_*/ }
    public int getDraggedOver() {/*_I*/_prof.prof.cnt[2396]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2397]++;/*I_*/return isDraggedOver;/*_I*/}/*I_*/ }

	final static int ALIGN_BOTTOM = -1;
	final static int ALIGN_CENTER = 0;
	final static int ALIGN_TOP = 1;

    public final int LEFT_WIDTH_OVERHEAD = 0;
    public final int LEFT_HEIGHT_OVERHEAD = 0;

    //
    // Constructors
    //
    
    protected NodeView(MindMapNode model, MapView map) {/*_I*/_prof.prof.cnt[2398]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2399]++;/*I_*/this.model = model;/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2400]++;/*I_*/setMap(map);/*_I*/}/*I_*/

	//Root has no edge
	/*_I*/{_prof.prof.cnt[2401]++;/*I_*/if (!isRoot()) {
	    /*_I*/{_prof.prof.cnt[2402]++;/*I_*/if (getModel().getEdge().getStyle().equals("linear")) {
		/*_I*/{_prof.prof.cnt[2403]++;/*I_*/edge = new LinearEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2404]++;/*I_*/if (getModel().getEdge().getStyle().equals("bezier")) {
		/*_I*/{_prof.prof.cnt[2405]++;/*I_*/edge = new BezierEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2406]++;/*I_*/if (getModel().getEdge().getStyle().equals("sharp_linear")) {
		/*_I*/{_prof.prof.cnt[2407]++;/*I_*/edge = new SharpLinearEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else /*_I*/{_prof.prof.cnt[2408]++;/*I_*/if (getModel().getEdge().getStyle().equals("sharp_bezier")) {
		/*_I*/{_prof.prof.cnt[2409]++;/*I_*/edge = new SharpBezierEdgeView(getParentView(),this);/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[2410]++;/*I_*/System.err.println("Unknown Edge Type.");/*_I*/}/*I_*/
	    }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
	}/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2411]++;/*I_*/addMouseListener( map.getNodeMouseMotionListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2412]++;/*I_*/addMouseMotionListener( map.getNodeMouseMotionListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2413]++;/*I_*/addKeyListener( map.getNodeKeyListener() );/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2414]++;/*I_*/addDragListener( map.getNodeDragListener() );/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2415]++;/*I_*/addDropListener( map.getNodeDropListener() );/*_I*/}/*I_*/
    }

    void addDragListener(DragGestureListener dgl) {/*_I*/_prof.prof.cnt[2416]++;/*I_*/
	/*_I*/_prof.prof.cnt[2417]++;/*I_*/DragSource dragSource = DragSource.getDefaultDragSource();
	/*_I*/{_prof.prof.cnt[2418]++;/*I_*/dragSource.createDefaultDragGestureRecognizer 
           (this, DnDConstants.ACTION_COPY | DnDConstants.ACTION_MOVE | DnDConstants.ACTION_LINK ,dgl);/*_I*/}/*I_*/
    }

    void addDropListener(DropTargetListener dtl) {/*_I*/_prof.prof.cnt[2419]++;/*I_*/
	/*_I*/_prof.prof.cnt[2420]++;/*I_*/DropTarget dropTarget = new DropTarget(this,dtl);
	/*_I*/{_prof.prof.cnt[2421]++;/*I_*/dropTarget.setActive(true);/*_I*/}/*I_*/
    }

    /**
     * Factory method which creates the right NodeView for the model.
     */ 
    protected static NodeView newNodeView(MindMapNode model, MapView map) {/*_I*/_prof.prof.cnt[2422]++;/*I_*/
	/*_I*/_prof.prof.cnt[2423]++;/*I_*/NodeView newView;
	/*_I*/{_prof.prof.cnt[2424]++;/*I_*/if (model.isRoot()) {
	    /*_I*/{_prof.prof.cnt[2425]++;/*I_*/newView = new RootNodeView( model, map );/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2426]++;/*I_*/if (model.getStyle().equals(MindMapNode.STYLE_FORK) ) {
	    /*_I*/{_prof.prof.cnt[2427]++;/*I_*/newView = new ForkNodeView( model, map );/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2428]++;/*I_*/if (model.getStyle().equals(MindMapNode.STYLE_BUBBLE) ) {
	    /*_I*/{_prof.prof.cnt[2429]++;/*I_*/newView = new BubbleNodeView( model, map );/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2430]++;/*I_*/System.err.println("Tried to create a NodeView of unknown Style.");/*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[2431]++;/*I_*/newView = new ForkNodeView(model, map);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2432]++;/*I_*/model.setViewer(newView);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2433]++;/*I_*/map.add(newView);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2434]++;/*I_*/newView.update();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2435]++;/*I_*/return newView;/*_I*/}/*I_*/
    }

    //
    // public methods
    //

    public boolean dropAsSibling(double xCoord) {/*_I*/_prof.prof.cnt[2436]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2437]++;/*I_*/return isLeft() ?
          xCoord > getSize().width*2/3 :
          xCoord < getSize().width/3;/*_I*/}/*I_*/ 
    }

    /** @return true if should be on the left, false otherwise.*/
    public boolean dropPosition (double xCoord) {/*_I*/_prof.prof.cnt[2438]++;/*I_*/
        /* here it is the same as me. */
       /*_I*/{_prof.prof.cnt[2439]++;/*I_*/return isLeft();/*_I*/}/*I_*/ 
    }

    public boolean followLink(double xCoord) {/*_I*/_prof.prof.cnt[2440]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2441]++;/*I_*/return getModel().getLink() != null &&
          (getModel().isRoot() || !getModel().hasChildren() || xCoord < getSize().width/2);/*_I*/}/*I_*/ }

    public void updateCursor(double xCoord) {/*_I*/_prof.prof.cnt[2442]++;/*I_*/
      /*_I*/_prof.prof.cnt[2443]++;/*I_*/int requiredCursor = followLink(xCoord) ? Cursor.HAND_CURSOR : Cursor.DEFAULT_CURSOR;
      /*_I*/{_prof.prof.cnt[2444]++;/*I_*/if (getCursor().getType() != requiredCursor) {
        /*_I*/{_prof.prof.cnt[2445]++;/*I_*/setCursor(new Cursor(requiredCursor));/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
    }

    public boolean isRoot() {/*_I*/_prof.prof.cnt[2446]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2447]++;/*I_*/return model.isRoot();/*_I*/}/*I_*/
    }

    public boolean getIsLong() {/*_I*/_prof.prof.cnt[2448]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2449]++;/*I_*/return isLong;/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isSiblingOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2450]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2451]++;/*I_*/return getParentView() == myNodeView.getParentView();/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isChildOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2452]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2453]++;/*I_*/return getParentView() == myNodeView;/*_I*/}/*I_*/ }

    /* fc, 25.1.2004: Refactoring necessary: should call the model. */
    public boolean isParentOf(NodeView myNodeView) {/*_I*/_prof.prof.cnt[2454]++;/*I_*/ 
       /*_I*/{_prof.prof.cnt[2455]++;/*I_*/return (this == myNodeView.getParentView());/*_I*/}/*I_*/ }

    public MindMapNode getModel() {/*_I*/_prof.prof.cnt[2456]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2457]++;/*I_*/return model;/*_I*/}/*I_*/
    }

    /** Returns the coordinates occupied by the node and its children as a vector of four point per node.*/
    public void getCoordinates(LinkedList inList, int additionalDistanceForConvexHull) {/*_I*/_prof.prof.cnt[2458]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2459]++;/*I_*/inList.addLast(new Point( -additionalDistanceForConvexHull + getX()             ,  -additionalDistanceForConvexHull + getY()              ));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2460]++;/*I_*/inList.addLast(new Point( -additionalDistanceForConvexHull + getX()             ,   additionalDistanceForConvexHull + getY() + getHeight()));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2461]++;/*I_*/inList.addLast(new Point(  additionalDistanceForConvexHull + getX() + getWidth(),   additionalDistanceForConvexHull + getY() + getHeight()));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2462]++;/*I_*/inList.addLast(new Point(  additionalDistanceForConvexHull + getX() + getWidth(),  -additionalDistanceForConvexHull + getY()              ));/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[2463]++;/*I_*/LinkedList childrenViews = getChildrenViews();
        /*_I*/_prof.prof.cnt[2464]++;/*I_*/ListIterator children_it = childrenViews.listIterator();
        /*_I*/{_prof.prof.cnt[2465]++;/*I_*/while(children_it.hasNext()) {
            /*_I*/_prof.prof.cnt[2466]++;/*I_*/NodeView child = (NodeView)children_it.next();
            /*_I*/{_prof.prof.cnt[2467]++;/*I_*/child.getCoordinates(inList, additionalDistanceForConvexHull);/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }   

    /** Changed to remove the printing bug of java.*/
    public Dimension getPreferredSize() {/*_I*/_prof.prof.cnt[2468]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2469]++;/*I_*/if(map.isPrinting()) {
            /*_I*/{_prof.prof.cnt[2470]++;/*I_*/return new Dimension(super.getPreferredSize().width + (int)(10f*map.getZoom()),
                                 super.getPreferredSize().height);/*_I*/}/*I_*/
        } else {
            /*_I*/{_prof.prof.cnt[2471]++;/*I_*/return super.getPreferredSize();/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }	


   public void requestFocus(){/*_I*/_prof.prof.cnt[2472]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2473]++;/*I_*/map.getController().getMode().getModeController().anotherNodeSelected(getModel());/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2474]++;/*I_*/super. requestFocus();/*_I*/}/*I_*/
   }

    public void paint(Graphics graphics) {/*_I*/_prof.prof.cnt[2475]++;/*I_*/
        // background color starts here, fc. 9.11.2003: todo
//           graphics.setColor(Color.yellow);
//           graphics.fillRect(0,0,getWidth(), getHeight());
          // background color ends here, fc. 9.11.2003: todo
	/*_I*/{_prof.prof.cnt[2476]++;/*I_*/super.paint(graphics);/*_I*/}/*I_*/
    }

   public void paintSelected(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2477]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2478]++;/*I_*/if( this.isSelected() ) {
          /*_I*/{_prof.prof.cnt[2479]++;/*I_*/graphics.setColor(selectedColor);/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[2480]++;/*I_*/graphics.fillRect(0,0,size.width, size.height);/*_I*/}/*I_*/
          //g.drawRect(0,0,size.width-1, size.height-2);
       }/*_I*/}/*I_*/
    }

   public void paintDragOver(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2481]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2482]++;/*I_*/if (isDraggedOver == DRAGGED_OVER_SON) {
           /*_I*/{_prof.prof.cnt[2483]++;/*I_*/if (isLeft()) {
              /*_I*/{_prof.prof.cnt[2484]++;/*I_*/graphics.setPaint( new GradientPaint(size.width*3/4,0,map.getBackground(), size.width/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2485]++;/*I_*/graphics.fillRect(0, 0, size.width*3/4, size.height-1);/*_I*/}/*I_*/ }
           else {
              /*_I*/{_prof.prof.cnt[2486]++;/*I_*/graphics.setPaint( new GradientPaint(size.width/4,0,map.getBackground(), size.width*3/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2487]++;/*I_*/graphics.fillRect(size.width/4, 0, size.width-1, size.height-1);/*_I*/}/*I_*/ }/*_I*/}/*I_*/       
	}/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[2488]++;/*I_*/if (isDraggedOver == DRAGGED_OVER_SIBLING) {
            /*_I*/{_prof.prof.cnt[2489]++;/*I_*/graphics.setPaint( new GradientPaint(0,size.height*3/5,map.getBackground(), 0, size.height/5, dragColor));/*_I*/}/*I_*/
            /*_I*/{_prof.prof.cnt[2490]++;/*I_*/graphics.fillRect(0, 0, size.width-1, size.height-1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

    public int getLeftWidthOverhead() {/*_I*/_prof.prof.cnt[2491]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2492]++;/*I_*/return LEFT_WIDTH_OVERHEAD;/*_I*/}/*I_*/ }

    //
    // get/set methods
    //

    int getTreeHeight() {/*_I*/_prof.prof.cnt[2493]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2494]++;/*I_*/return treeHeight;/*_I*/}/*I_*/
    }

    void setTreeHeight(int treeHeight) {/*_I*/_prof.prof.cnt[2495]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2496]++;/*I_*/this.treeHeight = treeHeight;/*_I*/}/*I_*/
    }

    protected boolean isSelected() {/*_I*/_prof.prof.cnt[2497]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2498]++;/*I_*/return (getMap().isSelected(this));/*_I*/}/*I_*/
    }

    /**Is the node left of root?*/
    public boolean isLeft() {/*_I*/_prof.prof.cnt[2499]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2500]++;/*I_*/if(getModel().isLeft() == null)
            /*_I*/{_prof.prof.cnt[2501]++;/*I_*/return true;/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2502]++;/*I_*/return getModel().isLeft().getValue();/*_I*/}/*I_*/
    }

    protected void setLeft(boolean left) {/*_I*/_prof.prof.cnt[2503]++;/*I_*/
        //this.left = left;
        /*_I*/{_prof.prof.cnt[2504]++;/*I_*/getModel().setLeft(left);/*_I*/}/*I_*/
    }
	
//     public boolean isLeftDefault() {
//         return getModel().isLeft() == null;
//     }
    
    protected void setModel( MindMapNode model ) {/*_I*/_prof.prof.cnt[2505]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2506]++;/*I_*/this.model = model;/*_I*/}/*I_*/
    }

    MapView getMap() {/*_I*/_prof.prof.cnt[2507]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2508]++;/*I_*/return map;/*_I*/}/*I_*/
    }

    protected void setMap( MapView map ) {/*_I*/_prof.prof.cnt[2509]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2510]++;/*I_*/this.map = map;/*_I*/}/*I_*/
    }

    EdgeView getEdge() {/*_I*/_prof.prof.cnt[2511]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2512]++;/*I_*/return edge;/*_I*/}/*I_*/
    }

    void setEdge(EdgeView edge) {/*_I*/_prof.prof.cnt[2513]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2514]++;/*I_*/this.edge = edge;/*_I*/}/*I_*/
    }

    protected NodeView getParentView() {/*_I*/_prof.prof.cnt[2515]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2516]++;/*I_*/return getModel().getParentNode().getViewer();/*_I*/}/*I_*/
    }

    /**
     * This method returns the NodeViews that are children of this node.
     */
    public LinkedList getChildrenViews() {/*_I*/_prof.prof.cnt[2517]++;/*I_*/
	/*_I*/_prof.prof.cnt[2518]++;/*I_*/LinkedList childrenViews = new LinkedList();
	/*_I*/_prof.prof.cnt[2519]++;/*I_*/ListIterator it = getModel().childrenUnfolded();
	/*_I*/{_prof.prof.cnt[2520]++;/*I_*/if (it != null) {
           /*_I*/{_prof.prof.cnt[2521]++;/*I_*/while(it.hasNext()) {
              /*_I*/_prof.prof.cnt[2522]++;/*I_*/NodeView view = ((MindMapNode)it.next()).getViewer();
              /*_I*/{_prof.prof.cnt[2523]++;/*I_*/if (view != null) { // Visible view
                 /*_I*/{_prof.prof.cnt[2524]++;/*I_*/childrenViews.add(view);/*_I*/}/*I_*/ // child.getViewer() );
              }/*_I*/}/*I_*/
           }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2525]++;/*I_*/return childrenViews;/*_I*/}/*I_*/
    }
    
    protected LinkedList getSiblingViews() {/*_I*/_prof.prof.cnt[2526]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2527]++;/*I_*/return getParentView().getChildrenViews();/*_I*/}/*I_*/
    }

    /**
     * Returns the Point where the OutEdge
     * should leave the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    Point getOutPoint() {/*_I*/_prof.prof.cnt[2528]++;/*I_*/
	/*_I*/_prof.prof.cnt[2529]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2530]++;/*I_*/if( isRoot() ) {
	    /*_I*/{_prof.prof.cnt[2531]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2532]++;/*I_*/if( isLeft() ) {
	    /*_I*/{_prof.prof.cnt[2533]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2534]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }

    /**
     * Returns the Point where the InEdge
     * should arrive the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    Point getInPoint() {/*_I*/_prof.prof.cnt[2535]++;/*I_*/
	/*_I*/_prof.prof.cnt[2536]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2537]++;/*I_*/if( isRoot() ) {
	    /*_I*/{_prof.prof.cnt[2538]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2539]++;/*I_*/if( isLeft() ) {
	    /*_I*/{_prof.prof.cnt[2540]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2541]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height - 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }


    /**
     * Returns the Point where the Links 
     * should arrive the Node.
     * THIS SHOULD BE DECLARED ABSTRACT AND BE DONE IN BUBBLENODEVIEW ETC.
     */
    Point getLinkPoint() {/*_I*/_prof.prof.cnt[2542]++;/*I_*/
	/*_I*/_prof.prof.cnt[2543]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2544]++;/*I_*/if( isRoot() ) {
	    /*_I*/{_prof.prof.cnt[2545]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else /*_I*/{_prof.prof.cnt[2546]++;/*I_*/if( isLeft() ) {
	    /*_I*/{_prof.prof.cnt[2547]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2548]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
	}/*_I*/}/*I_*//*_I*/}/*I_*/ 
    }

    /**
     * Returns the relative position of the Edge.
	 * This is used by bold edge to know how to shift the line.
     */
    int getAlignment() {/*_I*/_prof.prof.cnt[2549]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2550]++;/*I_*/if( isRoot() )
	    /*_I*/{_prof.prof.cnt[2551]++;/*I_*/return ALIGN_CENTER;/*_I*/}/*I_*//*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2552]++;/*I_*/return ALIGN_BOTTOM;/*_I*/}/*I_*/
    }
	
    //
    // Navigation
    //
    protected NodeView getNextPage() {/*_I*/_prof.prof.cnt[2553]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2554]++;/*I_*/if (isRoot()) {
        /*_I*/{_prof.prof.cnt[2555]++;/*I_*/return this;/*_I*/}/*I_*/ // I'm root
      }/*_I*/}/*I_*/
      /*_I*/_prof.prof.cnt[2556]++;/*I_*/NodeView sibling = getNextSibling();
      /*_I*/{_prof.prof.cnt[2557]++;/*I_*/if (sibling == this) {
        /*_I*/{_prof.prof.cnt[2558]++;/*I_*/return this;/*_I*/}/*I_*/ // at the end
      }/*_I*/}/*I_*/
//      if (sibling.getParentView() != this.getParentView()) {
//        return sibling; // sibling on another page (has different parent)
//      }
      /*_I*/_prof.prof.cnt[2559]++;/*I_*/NodeView nextSibling = sibling.getNextSibling();
      /*_I*/{_prof.prof.cnt[2560]++;/*I_*/while (nextSibling != sibling 
              && sibling.getParentView() == nextSibling.getParentView()) {
        /*_I*/{_prof.prof.cnt[2561]++;/*I_*/sibling = nextSibling;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2562]++;/*I_*/nextSibling = nextSibling.getNextSibling();/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2563]++;/*I_*/return sibling;/*_I*/}/*I_*/ // last on the page
    }
      
    protected NodeView getPreviousPage() {/*_I*/_prof.prof.cnt[2564]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2565]++;/*I_*/if (isRoot()) {
        /*_I*/{_prof.prof.cnt[2566]++;/*I_*/return this;/*_I*/}/*I_*/ // I'm root
      }/*_I*/}/*I_*/
      /*_I*/_prof.prof.cnt[2567]++;/*I_*/NodeView sibling = getPreviousSibling();
      /*_I*/{_prof.prof.cnt[2568]++;/*I_*/if (sibling == this) {
        /*_I*/{_prof.prof.cnt[2569]++;/*I_*/return this;/*_I*/}/*I_*/ // at the end
      }/*_I*/}/*I_*/
//      if (sibling.getParentView() != this.getParentView()) {
//        return sibling; // sibling on another page (has different parent)
//      }
      /*_I*/_prof.prof.cnt[2570]++;/*I_*/NodeView previousSibling = sibling.getPreviousSibling();
      /*_I*/{_prof.prof.cnt[2571]++;/*I_*/while (previousSibling != sibling 
              && sibling.getParentView() == previousSibling.getParentView()) {
        /*_I*/{_prof.prof.cnt[2572]++;/*I_*/sibling = previousSibling;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2573]++;/*I_*/previousSibling = previousSibling.getPreviousSibling();/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2574]++;/*I_*/return sibling;/*_I*/}/*I_*/ // last on the page
    }
    
    protected NodeView getNextSibling() {/*_I*/_prof.prof.cnt[2575]++;/*I_*/
      /*_I*/_prof.prof.cnt[2576]++;/*I_*/NodeView sibling;
      /*_I*/_prof.prof.cnt[2577]++;/*I_*/NodeView nextSibling = this;
      
      // get next sibling even in higher levels
      /*_I*/{_prof.prof.cnt[2578]++;/*I_*/for (sibling = this; !sibling.isRoot(); sibling = sibling.getParentView()) { 
        /*_I*/{_prof.prof.cnt[2579]++;/*I_*/nextSibling = sibling.getNextSiblingSingle();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2580]++;/*I_*/if (sibling != nextSibling) {
          /*_I*/{_prof.prof.cnt[2581]++;/*I_*/break;/*_I*/}/*I_*/ // found sibling
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      
      /*_I*/{_prof.prof.cnt[2582]++;/*I_*/if (sibling.isRoot()) {
        /*_I*/{_prof.prof.cnt[2583]++;/*I_*/return this;/*_I*/}/*I_*/  // didn't find (we are at the end)
      }/*_I*/}/*I_*/
      
      // we have the nextSibling, search in childs
      // untill: leaf, closed node, max level
      /*_I*/{_prof.prof.cnt[2584]++;/*I_*/sibling = nextSibling;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2585]++;/*I_*/while (sibling.getModel().getNodeLevel() < getMap().getSiblingMaxLevel()) {
        // can we drill down?
        /*_I*/{_prof.prof.cnt[2586]++;/*I_*/if (sibling.getChildrenViews().size() <= 0) {
          /*_I*/{_prof.prof.cnt[2587]++;/*I_*/break;/*_I*/}/*I_*/ // no
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2588]++;/*I_*/sibling = (NodeView)(sibling.getChildrenViews().getFirst());/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2589]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getPreviousSibling() {/*_I*/_prof.prof.cnt[2590]++;/*I_*/
      /*_I*/_prof.prof.cnt[2591]++;/*I_*/NodeView sibling;
      /*_I*/_prof.prof.cnt[2592]++;/*I_*/NodeView previousSibling = this;
      
      // get Previous sibling even in higher levels
      /*_I*/{_prof.prof.cnt[2593]++;/*I_*/for (sibling = this; !sibling.isRoot(); sibling = sibling.getParentView()) { 
        /*_I*/{_prof.prof.cnt[2594]++;/*I_*/previousSibling = sibling.getPreviousSiblingSingle();/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2595]++;/*I_*/if (sibling != previousSibling) {
          /*_I*/{_prof.prof.cnt[2596]++;/*I_*/break;/*_I*/}/*I_*/ // found sibling
        }/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      
      /*_I*/{_prof.prof.cnt[2597]++;/*I_*/if (sibling.isRoot()) {
        /*_I*/{_prof.prof.cnt[2598]++;/*I_*/return this;/*_I*/}/*I_*/  // didn't find (we are at the end)
      }/*_I*/}/*I_*/
      
      // we have the PreviousSibling, search in childs
      // untill: leaf, closed node, max level
      /*_I*/{_prof.prof.cnt[2599]++;/*I_*/sibling = previousSibling;/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2600]++;/*I_*/while (sibling.getModel().getNodeLevel() < getMap().getSiblingMaxLevel()) {
        // can we drill down?
        /*_I*/{_prof.prof.cnt[2601]++;/*I_*/if (sibling.getChildrenViews().size() <= 0) {
          /*_I*/{_prof.prof.cnt[2602]++;/*I_*/break;/*_I*/}/*I_*/ // no
        }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2603]++;/*I_*/sibling = (NodeView)(sibling.getChildrenViews().getLast());/*_I*/}/*I_*/
      }/*_I*/}/*I_*/
      /*_I*/{_prof.prof.cnt[2604]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getNextSiblingSingle() {/*_I*/_prof.prof.cnt[2605]++;/*I_*/
	/*_I*/_prof.prof.cnt[2606]++;/*I_*/LinkedList v = null;
	/*_I*/{_prof.prof.cnt[2607]++;/*I_*/if (getParentView().isRoot()) {
	    /*_I*/{_prof.prof.cnt[2608]++;/*I_*/if (this.isLeft()) {
		/*_I*/{_prof.prof.cnt[2609]++;/*I_*/v = ((RootNodeView)getParentView()).getLeft();/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[2610]++;/*I_*/v = ((RootNodeView)getParentView()).getRight();/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2611]++;/*I_*/v = getParentView().getChildrenViews();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/	
	/*_I*/_prof.prof.cnt[2612]++;/*I_*/NodeView sibling;
	/*_I*/{_prof.prof.cnt[2613]++;/*I_*/if (v.size()-1 == v.indexOf(this)) { //this is last, return first
//	    sibling = (NodeView)v.getFirst(); // loop
            /*_I*/{_prof.prof.cnt[2614]++;/*I_*/sibling = this;/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2615]++;/*I_*/sibling = (NodeView)v.get(v.indexOf(this)+1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2616]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    protected NodeView getPreviousSiblingSingle() {/*_I*/_prof.prof.cnt[2617]++;/*I_*/
	/*_I*/_prof.prof.cnt[2618]++;/*I_*/LinkedList v = null;
	/*_I*/{_prof.prof.cnt[2619]++;/*I_*/if (getParentView().isRoot()) {
	    /*_I*/{_prof.prof.cnt[2620]++;/*I_*/if (this.isLeft()) {
		/*_I*/{_prof.prof.cnt[2621]++;/*I_*/v = ((RootNodeView)getParentView()).getLeft();/*_I*/}/*I_*/
	    } else {
		/*_I*/{_prof.prof.cnt[2622]++;/*I_*/v = ((RootNodeView)getParentView()).getRight();/*_I*/}/*I_*/
	    }/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2623]++;/*I_*/v = getParentView().getChildrenViews();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/_prof.prof.cnt[2624]++;/*I_*/NodeView sibling;
	/*_I*/{_prof.prof.cnt[2625]++;/*I_*/if (v.indexOf(this) <= 0) {//this is first, return last
//	    sibling = (NodeView)v.getLast(); // loop
          /*_I*/{_prof.prof.cnt[2626]++;/*I_*/sibling = this;/*_I*/}/*I_*/
	} else {
	    /*_I*/{_prof.prof.cnt[2627]++;/*I_*/sibling = (NodeView)v.get(v.indexOf(this)-1);/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2628]++;/*I_*/return sibling;/*_I*/}/*I_*/
    }

    //
    // Update from Model
    //

    void insert() {/*_I*/_prof.prof.cnt[2629]++;/*I_*/
       /*_I*/_prof.prof.cnt[2630]++;/*I_*/ListIterator it = getModel().childrenFolded();
       /*_I*/{_prof.prof.cnt[2631]++;/*I_*/while(it.hasNext()) {               
          /*_I*/{_prof.prof.cnt[2632]++;/*I_*/insert((MindMapNode)it.next());/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }

    /**
     * Create views for the newNode and all his descendants, set their
     * isLeft attribute according to this view. Observe that views know
     * about their parents only through their models.
     */

    void insert(MindMapNode newNode) {/*_I*/_prof.prof.cnt[2633]++;/*I_*/
       /*_I*/_prof.prof.cnt[2634]++;/*I_*/NodeView newView = NodeView.newNodeView(newNode,getMap());
       /*_I*/{_prof.prof.cnt[2635]++;/*I_*/newView.setLeft(this.isLeft());/*_I*/}/*I_*/

       /*_I*/_prof.prof.cnt[2636]++;/*I_*/ListIterator it = newNode.childrenFolded();
       /*_I*/{_prof.prof.cnt[2637]++;/*I_*/while (it.hasNext()) {
          /*_I*/_prof.prof.cnt[2638]++;/*I_*/MindMapNode child = (MindMapNode)it.next();
          /*_I*/{_prof.prof.cnt[2639]++;/*I_*/newView.insert(child);/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }
    
    /**
     * This is a bit problematic, because getChildrenViews() only works if 
     * model is not yet removed. (So do not _really_ delete the model before the view 
     * removed (it needs to stay in memory)
     */
    void remove() {/*_I*/_prof.prof.cnt[2640]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2641]++;/*I_*/getMap().remove(this);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2642]++;/*I_*/if (getEdge()!=null) {
           /*_I*/{_prof.prof.cnt[2643]++;/*I_*/getEdge().remove();/*_I*/}/*I_*/ }/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2644]++;/*I_*/getModel().setViewer(null);/*_I*/}/*I_*/ // Let the model know he is invisible
	/*_I*/{_prof.prof.cnt[2645]++;/*I_*/for(ListIterator e = getChildrenViews().listIterator();e.hasNext();) {
           /*_I*/{_prof.prof.cnt[2646]++;/*I_*/((NodeView)e.next()).remove();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}

     void update() {/*_I*/_prof.prof.cnt[2647]++;/*I_*/
        //System.err.println("update");
        // 1) Set color
        /*_I*/_prof.prof.cnt[2648]++;/*I_*/Color color = getModel().getColor();
        /*_I*/{_prof.prof.cnt[2649]++;/*I_*/if (color==null) {
           /*_I*/_prof.prof.cnt[2650]++;/*I_*/String stdcolor = map.getController().getProperty("standardnodecolor");
           /*_I*/{_prof.prof.cnt[2651]++;/*I_*/if (stdcolor.length() == 7) {
              /*_I*/{_prof.prof.cnt[2652]++;/*I_*/color = Tools.xmlToColor(stdcolor);/*_I*/}/*I_*/ }
           else {
              /*_I*/{_prof.prof.cnt[2653]++;/*I_*/color = Color.black;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2654]++;/*I_*/setForeground(color);/*_I*/}/*I_*/

        // 2) Create the icons:
        /*_I*/_prof.prof.cnt[2655]++;/*I_*/MultipleImage iconImages = new MultipleImage(map.getZoom());
        /*_I*/_prof.prof.cnt[2656]++;/*I_*/boolean iconPresent = false;
        /* fc, 06.10.2003: images?*/
        /*_I*/_prof.prof.cnt[2657]++;/*I_*/Vector icons = ((NodeAdapter)getModel()).getIcons();
        /*_I*/{_prof.prof.cnt[2658]++;/*I_*/for(int i = 0; i < icons.size(); ++i) {
            /*_I*/{_prof.prof.cnt[2659]++;/*I_*/iconPresent = true;/*_I*/}/*I_*/
            /*_I*/_prof.prof.cnt[2660]++;/*I_*/MindIcon myicon = (MindIcon) icons.get(i);
            //System.out.println("print the icon " + myicon.toString());
            /*_I*/{_prof.prof.cnt[2661]++;/*I_*/iconImages.addImage((ImageIcon) myicon.getIcon(map.getController().getFrame()));/*_I*/}/*I_*/  
        }/*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[2662]++;/*I_*/String link = ((NodeAdapter)getModel()).getLink();
        /*_I*/{_prof.prof.cnt[2663]++;/*I_*/if ( link != null ) 
            {
                /*_I*/{_prof.prof.cnt[2664]++;/*I_*/iconPresent = true;/*_I*/}/*I_*/
                /*_I*/_prof.prof.cnt[2665]++;/*I_*/ImageIcon icon = new ImageIcon(((NodeAdapter)getModel()).getFrame().getResource
                                          (link.startsWith("mailto:") ? "images/Mail.png" :
                                           (Tools.executableByExtension(link) ? "images/Executable.png" :
                                            "images/Link.png")));
                /*_I*/{_prof.prof.cnt[2666]++;/*I_*/iconImages.addImage(icon);/*_I*/}/*I_*/ 
            }/*_I*/}/*I_*/
//         /* Folded icon by Matthias Schade (mascha2), fc, 20.12.2003*/
//         if (((NodeAdapter)getModel()).isFolded()) {
//             iconPresent = true;
//             ImageIcon icon = new ImageIcon(((NodeAdapter)getModel()).getFrame().getResource("images/Folded.png"));
//             iconImages.addImage(icon); 
//         }
        // DanielPolansky: set icon only if icon is present, because
        // we don't want to insert any additional white space.        
        /*_I*/{_prof.prof.cnt[2667]++;/*I_*/setIcon(iconPresent?iconImages:null);/*_I*/}/*I_*/

        // 3) Determine font
        /*_I*/_prof.prof.cnt[2668]++;/*I_*/Font font = getModel().getFont();
        /*_I*/{_prof.prof.cnt[2669]++;/*I_*/font = font == null ? map.getController().getDefaultFont() : font;/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2670]++;/*I_*/if (font != null) {
           /*_I*/{_prof.prof.cnt[2671]++;/*I_*/if (map.getZoom() != 1F) {
              /*_I*/{_prof.prof.cnt[2672]++;/*I_*/font = font.deriveFont(font.getSize()*map.getZoom());/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[2673]++;/*I_*/setFont(font);/*_I*/}/*I_*/ }
        else {
           // We can survive this trouble.
           /*_I*/{_prof.prof.cnt[2674]++;/*I_*/System.err.println("NodeView.update(): default font is null.");/*_I*/}/*I_*/ }/*_I*/}/*I_*/

        // 4) Set the text
        // Right now, this implementation is quite logical, although it allows
        // for nonconvex feature of nodes starting with <html>.

        /*_I*/_prof.prof.cnt[2675]++;/*I_*/String nodeText = getModel().toString();
        /*_I*/{_prof.prof.cnt[2676]++;/*I_*/if (nodeText.length() < 4) { // (PN)
          /*_I*/{_prof.prof.cnt[2677]++;/*I_*/nodeText = nodeText + new String("   ").substring(nodeText.length());/*_I*/}/*I_*/
        }/*_I*/}/*I_*/

        // Tell if node is long and its width has to be restricted
        // boolean isMultiline = nodeText.indexOf("\n") >= 0;
        /*_I*/_prof.prof.cnt[2678]++;/*I_*/String[] lines = nodeText.split("\n");
        /*_I*/_prof.prof.cnt[2679]++;/*I_*/boolean widthMustBeRestricted = false;

        /*_I*/{_prof.prof.cnt[2680]++;/*I_*/lines = nodeText.split("\n");/*_I*/}/*I_*/           
        /*_I*/{_prof.prof.cnt[2681]++;/*I_*/for (int line = 0; line < lines.length; line++) {
           // Compute the width the node would spontaneously take,
           // by preliminarily setting the text.
           /*_I*/{_prof.prof.cnt[2682]++;/*I_*/setText(lines[line]);/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[2683]++;/*I_*/if (widthMustBeRestricted = getPreferredSize().width > 
               map.getZoomed(map.getMaxNodeWidth())) {
              /*_I*/{_prof.prof.cnt[2684]++;/*I_*/break;/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[2685]++;/*I_*/isLong = widthMustBeRestricted || lines.length > 1;/*_I*/}/*I_*/
   
        /*_I*/{_prof.prof.cnt[2686]++;/*I_*/if (nodeText.startsWith("<html>")) {
           // Make it possible to use relative img references in HTML using tag <base>.
           /*_I*/{_prof.prof.cnt[2687]++;/*I_*/if (nodeText.indexOf("<img")>=0 && nodeText.indexOf("<base ") < 0 ) {
              /*_I*/{_prof.prof.cnt[2688]++;/*I_*/try {
                 /*_I*/{_prof.prof.cnt[2689]++;/*I_*/nodeText = "<html><base href=\""+
                    map.getModel().getURL()+"\">"+nodeText.substring(6);/*_I*/}/*I_*/ }
              catch (MalformedURLException e) {}/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[2690]++;/*I_*/setText(nodeText);/*_I*/}/*I_*/ }
        else /*_I*/{_prof.prof.cnt[2691]++;/*I_*/if (nodeText.startsWith("<table>")) {           	             	  
           /*_I*/{_prof.prof.cnt[2692]++;/*I_*/lines[0] = lines[0].substring(7);/*_I*/}/*I_*/ // remove <table> tag
           /*_I*/_prof.prof.cnt[2693]++;/*I_*/int startingLine = lines[0].matches("\\s*") ? 1 : 0;
           // ^ If the remaining first line is empty, do not draw it
           
           /*_I*/_prof.prof.cnt[2694]++;/*I_*/String text = "<html><table border=1 style=\"border-color: white\">";
           //String[] lines = nodeText.split("\n");
           /*_I*/{_prof.prof.cnt[2695]++;/*I_*/for (int line = startingLine; line < lines.length; line++) {
              /*_I*/{_prof.prof.cnt[2696]++;/*I_*/text += "<tr><td style=\"border-color: white;\">"+
                 Tools.toXMLEscapedText(lines[line]).replaceAll("\t","<td style=\"border-color: white\">");/*_I*/}/*I_*/ }/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[2697]++;/*I_*/setText(text);/*_I*/}/*I_*/ }
        else /*_I*/{_prof.prof.cnt[2698]++;/*I_*/if (isLong) {
           /*_I*/_prof.prof.cnt[2699]++;/*I_*/String text = "<tr><td>";              
           /*_I*/_prof.prof.cnt[2700]++;/*I_*/int maximumLineLength = 0;
           /*_I*/{_prof.prof.cnt[2701]++;/*I_*/for (int line = 0; line < lines.length; line++) {
              /*_I*/{_prof.prof.cnt[2702]++;/*I_*/text += Tools.toXMLEscapedTextWithNBSPizedSpaces(lines[line]) + "<p>";/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2703]++;/*I_*/if (lines[line].length() > maximumLineLength) {
                 /*_I*/{_prof.prof.cnt[2704]++;/*I_*/maximumLineLength = lines[line].length();/*_I*/}/*I_*/ }/*_I*/}/*I_*/}/*_I*/}/*I_*/
           
			/*_I*/{_prof.prof.cnt[2705]++;/*I_*/text += "</td></tr>";/*_I*/}/*I_*/
           /*_I*/{_prof.prof.cnt[2706]++;/*I_*/setText("<html><table"+
                   (!widthMustBeRestricted?">":" width=\""+map.getZoomed(map.getMaxNodeWidth())+"\">")+
                   text+"</table></html>");/*_I*/}/*I_*/ }/*_I*/}/*I_*//*_I*/}/*I_*//*_I*/}/*I_*/
   
        // 5) Complete
        /*_I*/{_prof.prof.cnt[2707]++;/*I_*/repaint();/*_I*/}/*I_*/ // Because of zoom?
    }

    void updateAll() {/*_I*/_prof.prof.cnt[2708]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2709]++;/*I_*/update();/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2710]++;/*I_*/for(ListIterator e = getChildrenViews().listIterator();e.hasNext();) {
	    /*_I*/_prof.prof.cnt[2711]++;/*I_*/NodeView child = (NodeView)e.next();
	    /*_I*/{_prof.prof.cnt[2712]++;/*I_*/child.updateAll();/*_I*/}/*I_*/
	}/*_I*/}/*I_*/
    }

   protected void setRendering(Graphics2D g) {/*_I*/_prof.prof.cnt[2713]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2714]++;/*I_*/if (map.getController().getAntialiasAll()) {
         /*_I*/{_prof.prof.cnt[2715]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);/*_I*/}/*I_*/ }/*_I*/}/*I_*/
//       else
//          g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF); 

   }

}
