/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000-2001  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/*$Id: RootNodeView.java,v 1.14 2004/01/24 22:36:48 christianfoltin Exp $*/

package freemind.view.mindmapview;

import freemind.modes.MindMapNode;
import java.util.LinkedList;
import java.util.ListIterator;
import java.awt.*;

/**
 * This is a RootNode with different placing of children
 * and different painting than a normal NodeView
 */
public class RootNodeView extends NodeView {

    //
    // Constructors
    //
    
    public RootNodeView(MindMapNode model, MapView map) {
	super(model,map);/*_I*/_prof.prof.cnt[2716]++;/*I_*//*_I*/_prof.prof.cnt[2717]++;/*I_*/
    }
    
    public NodeView getParentView() {/*_I*/_prof.prof.cnt[2718]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2719]++;/*I_*/return null;/*_I*/}/*I_*/
    }
    

    /**
     * Returns the Point where the OutEdge
     * should leave the Node.
     */
    Point getOutPoint() {/*_I*/_prof.prof.cnt[2720]++;/*I_*/
	/*_I*/_prof.prof.cnt[2721]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2722]++;/*I_*/return new Point(getLocation().x + size.width, getLocation().y + size.height / 2);/*_I*/}/*I_*/
    }

    /**
     * Returns the Point where the InEdge
     * should arrive the Node.
     */
    Point getInPoint() {/*_I*/_prof.prof.cnt[2723]++;/*I_*/
	/*_I*/_prof.prof.cnt[2724]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2725]++;/*I_*/return new Point(getLocation().x, getLocation().y + size.height / 2);/*_I*/}/*I_*/
    }

    EdgeView getEdge() {/*_I*/_prof.prof.cnt[2726]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2727]++;/*I_*/return null;/*_I*/}/*I_*/
    }

    void setEdge(EdgeView edge) {/*_I*/_prof.prof.cnt[2728]++;/*I_*/
    }

    LinkedList getLeft() {/*_I*/_prof.prof.cnt[2729]++;/*I_*/
	/*_I*/_prof.prof.cnt[2730]++;/*I_*/LinkedList all = getChildrenViews();
	/*_I*/_prof.prof.cnt[2731]++;/*I_*/LinkedList left = new LinkedList();
	/*_I*/{_prof.prof.cnt[2732]++;/*I_*/for (ListIterator e = all.listIterator();e.hasNext();) {
	    /*_I*/_prof.prof.cnt[2733]++;/*I_*/NodeView node = (NodeView)e.next();
	    /*_I*/{_prof.prof.cnt[2734]++;/*I_*/if (node == null) /*_I*/{_prof.prof.cnt[2735]++;/*I_*/continue;/*_I*/}/*I_*//*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[2736]++;/*I_*/if (node.isLeft()) /*_I*/{_prof.prof.cnt[2737]++;/*I_*/left.add(node);/*_I*/}/*I_*//*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2738]++;/*I_*/return left;/*_I*/}/*I_*/
    }

    LinkedList getRight() {/*_I*/_prof.prof.cnt[2739]++;/*I_*/
	/*_I*/_prof.prof.cnt[2740]++;/*I_*/LinkedList all = getChildrenViews();
	/*_I*/_prof.prof.cnt[2741]++;/*I_*/LinkedList right = new LinkedList();
	/*_I*/{_prof.prof.cnt[2742]++;/*I_*/for (ListIterator e = all.listIterator();e.hasNext();) {
	    /*_I*/_prof.prof.cnt[2743]++;/*I_*/NodeView node = (NodeView)e.next();
	    /*_I*/{_prof.prof.cnt[2744]++;/*I_*/if (node == null) /*_I*/{_prof.prof.cnt[2745]++;/*I_*/continue;/*_I*/}/*I_*//*_I*/}/*I_*/
	    /*_I*/{_prof.prof.cnt[2746]++;/*I_*/if (!node.isLeft()) /*_I*/{_prof.prof.cnt[2747]++;/*I_*/right.add(node);/*_I*/}/*I_*//*_I*/}/*I_*/
	}/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2748]++;/*I_*/return right;/*_I*/}/*I_*/
    }

    void insert(MindMapNode newNode) {/*_I*/_prof.prof.cnt[2749]++;/*I_*/
        /*_I*/_prof.prof.cnt[2750]++;/*I_*/NodeView newView = newNodeView(newNode,getMap());
        /*_I*/{_prof.prof.cnt[2751]++;/*I_*/newView.update();/*_I*/}/*I_*/
        // decide left or right only if not actually set:
        /*_I*/{_prof.prof.cnt[2752]++;/*I_*/if(newNode.isLeft()==null)
            /*_I*/{_prof.prof.cnt[2753]++;/*I_*/newView.setLeft( getLeft().size() <= getRight().size() );/*_I*/}/*I_*//*_I*/}/*I_*/
        /*_I*/_prof.prof.cnt[2754]++;/*I_*/ListIterator it = newNode.childrenFolded();
        /*_I*/{_prof.prof.cnt[2755]++;/*I_*/if (it != null) {
            /*_I*/{_prof.prof.cnt[2756]++;/*I_*/for (;it.hasNext();) {
                /*_I*/_prof.prof.cnt[2757]++;/*I_*/MindMapNode child = (MindMapNode)it.next();
                /*_I*/{_prof.prof.cnt[2758]++;/*I_*/newView.insert(child);/*_I*/}/*I_*/
            }/*_I*/}/*I_*/
        }/*_I*/}/*I_*/
    }	

    public boolean dropAsSibling (double xCoord) {/*_I*/_prof.prof.cnt[2759]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2760]++;/*I_*/return false;/*_I*/}/*I_*/
    }

    /** @return true if should be on the left, false otherwise.*/
    public boolean dropPosition (double xCoord) {/*_I*/_prof.prof.cnt[2761]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2762]++;/*I_*/return xCoord < getSize().width*1/2 ;/*_I*/}/*I_*/ 
    }

    public void setDraggedOver(Point p) {/*_I*/_prof.prof.cnt[2763]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2764]++;/*I_*/setDraggedOver ((dropPosition(p.getX())) ? NodeView.DRAGGED_OVER_SON_LEFT : NodeView.DRAGGED_OVER_SON);/*_I*/}/*I_*/ 
    }

    //
    // Navigation
    //

    public NodeView getNextSibling() {/*_I*/_prof.prof.cnt[2765]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2766]++;/*I_*/return this;/*_I*/}/*I_*/
    }

    public NodeView getPreviousSibling() {/*_I*/_prof.prof.cnt[2767]++;/*I_*/
	/*_I*/{_prof.prof.cnt[2768]++;/*I_*/return this;/*_I*/}/*I_*/
    }


    //
    // Layout
    //

    public Dimension getPreferredSize() {/*_I*/_prof.prof.cnt[2769]++;/*I_*/
	/*_I*/_prof.prof.cnt[2770]++;/*I_*/int width = (int)(super.getPreferredSize().width*1.1);
	/*_I*/_prof.prof.cnt[2771]++;/*I_*/int height = (int)(super.getPreferredSize().height*2);
	/*_I*/{_prof.prof.cnt[2772]++;/*I_*/return new Dimension(width,height);/*_I*/}/*I_*/
    }	

    public void paintSelected(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2773]++;/*I_*/
       /*_I*/{_prof.prof.cnt[2774]++;/*I_*/if( this.isSelected() ) {
          /*_I*/{_prof.prof.cnt[2775]++;/*I_*/graphics.setColor(selectedColor);/*_I*/}/*I_*/
          /*_I*/{_prof.prof.cnt[2776]++;/*I_*/graphics.fillOval(1,1,size.width-1,size.height-1);/*_I*/}/*I_*/
       }/*_I*/}/*I_*/
    }


    /**
     * Paints the node
     */
    public void paint(Graphics graphics) {/*_I*/_prof.prof.cnt[2777]++;/*I_*/
	/*_I*/_prof.prof.cnt[2778]++;/*I_*/Graphics2D g = (Graphics2D)graphics;
	/*_I*/_prof.prof.cnt[2779]++;/*I_*/Dimension size = getSize();
	/*_I*/{_prof.prof.cnt[2780]++;/*I_*/if (this.getModel()==null) /*_I*/{_prof.prof.cnt[2781]++;/*I_*/return;/*_I*/}/*I_*//*_I*/}/*I_*/

        /*_I*/{_prof.prof.cnt[2782]++;/*I_*/paintSelected(g, size);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2783]++;/*I_*/paintDragOver(g, size);/*_I*/}/*I_*/

	//Draw a root node
	/*_I*/{_prof.prof.cnt[2784]++;/*I_*/setHorizontalAlignment(CENTER);/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2785]++;/*I_*/g.setColor(Color.gray);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2786]++;/*I_*/g.setStroke(new BasicStroke(1.0f));/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2787]++;/*I_*/setRendering(g);/*_I*/}/*I_*/
	/*_I*/{_prof.prof.cnt[2788]++;/*I_*/g.drawOval(1,1,size.width-2,size.height-2);/*_I*/}/*I_*/
        /*_I*/{_prof.prof.cnt[2789]++;/*I_*/if (!getMap().getController().getAntialiasAll()) {
           /*_I*/{_prof.prof.cnt[2790]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);/*_I*/}/*I_*/ }/*_I*/}/*I_*/

	/*_I*/{_prof.prof.cnt[2791]++;/*I_*/super.paint(g);/*_I*/}/*I_*/
    }

   public void paintDragOver(Graphics2D graphics, Dimension size) {/*_I*/_prof.prof.cnt[2792]++;/*I_*/
        /*_I*/{_prof.prof.cnt[2793]++;/*I_*/if (getDraggedOver() == DRAGGED_OVER_SON) {
              /*_I*/{_prof.prof.cnt[2794]++;/*I_*/graphics.setPaint( new GradientPaint(size.width/4,0,map.getBackground(), size.width*3/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2795]++;/*I_*/graphics.fillRect(size.width/4, 0, size.width-1, size.height-1);/*_I*/}/*I_*/ 
        } else /*_I*/{_prof.prof.cnt[2796]++;/*I_*/if (getDraggedOver() == DRAGGED_OVER_SON_LEFT) {
              /*_I*/{_prof.prof.cnt[2797]++;/*I_*/graphics.setPaint( new GradientPaint(size.width*3/4,0,map.getBackground(), size.width/4, 0, dragColor));/*_I*/}/*I_*/
              /*_I*/{_prof.prof.cnt[2798]++;/*I_*/graphics.fillRect(0, 0, size.width*3/4, size.height-1);/*_I*/}/*I_*/
        }/*_I*/}/*I_*//*_I*/}/*I_*/
    }


   protected void setRendering(Graphics2D g) {/*_I*/_prof.prof.cnt[2799]++;/*I_*/
      /*_I*/{_prof.prof.cnt[2800]++;/*I_*/if (getMap().getController().getAntialiasEdges() || getMap().getController().getAntialiasAll()) {
         /*_I*/{_prof.prof.cnt[2801]++;/*I_*/g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);/*_I*/}/*I_*/ }/*_I*/}/*I_*/}


}



//     /**
//      * Determines the logical Position of all of its direct children.
//      */
//     public void layoutChildren() {
// 	if (getChildrenViews().size() == 0) return;//nothing to do
// 	layoutRoot();
    //    } 
  
